@@ -2,7 +2,7 @@
 #define NEWSBEUTER_CONFIG__H
 
 #define PROGRAM_NAME		PACKAGE
-#define PROGRAM_VERSION		"0.7"
+#define PROGRAM_VERSION		"0.9.1"
 #define PROGRAM_URL			"http://synflood.at/newsbeuter.html"
 #define USER_AGENT			PROGRAM_NAME " rss feedreader " PROGRAM_VERSION " (" PROGRAM_URL ")"
 
@@ -12,6 +12,11 @@
 #ifdef _ENABLE_NLS
 #	include <libintl.h>
 #	include <locale.h>
+
+#	ifdef _
+#		undef _
+#	endif
+
 #	define _(string) gettext(string)
 #else
 #	define _(string) (string)
