@@ -25,6 +25,11 @@ class cache {
 		void catchup_all(rss_feed& feed);
 		void update_rssitem_flags(rss_item& item);
 		std::vector<std::string> get_feed_urls();
+		time_t get_lastmodified(const std::string& feedurl);
+		void set_lastmodified(const std::string& feedurl, time_t lastmod);
+		unsigned int get_unread_count();
+		void mark_item_deleted(const std::string& guid, bool b);
+		void remove_old_deleted_items(const std::string& rssurl, const std::vector<std::string>& guids);
 	private:
 		void populate_tables();
 		void set_pragmas();
