@@ -7,6 +7,8 @@
 
 namespace newsbeuter {
 
+typedef std::pair<rss_feed *, unsigned int> feedptr_pos_pair;
+
 class feedlist_formaction : public formaction {
 	public:
 		feedlist_formaction(view *, std::string formstr);
@@ -14,10 +16,13 @@ class feedlist_formaction : public formaction {
 		virtual void prepare();
 		virtual void init();
 		void set_feedlist(std::vector<rss_feed>& feeds);
+		void update_visible_feeds(std::vector<rss_feed>& feeds);
 		void set_tags(const std::vector<std::string>& t);
 		virtual keymap_hint_entry * get_keymap_hint();
 		rss_feed * get_feed();
 
+		virtual std::string id() const { return "feedlist"; }
+
 		bool jump_to_next_unread_feed(unsigned int& feedpos);
 		bool jump_to_previous_unread_feed(unsigned int& feedpos);
 
@@ -25,16 +30,23 @@ class feedlist_formaction : public formaction {
 
 		virtual void finished_qna(operation op);
 
+		void mark_pos_if_visible(unsigned int pos);
+
+
 	private:
 
 		int get_pos(unsigned int realidx);
-		virtual void process_operation(operation op);
+		virtual void process_operation(operation op, bool automatic = false, std::vector<std::string> * args = NULL);
+
+		void goto_feed(const std::string& str);
+
+		void save_filterpos();
 
 		bool zero_feedpos;
 		unsigned int feeds_shown;
 		bool auto_open;
 		bool quit;
-		std::vector<std::pair<rss_feed *, unsigned int> > visible_feeds;
+		std::vector<feedptr_pos_pair> visible_feeds;
 		std::string tag;
 		std::vector<std::string> tags;
 
@@ -44,6 +56,9 @@ class feedlist_formaction : public formaction {
 		history filterhistory;
 
 		rss_feed search_dummy_feed;
+
+		unsigned int filterpos;
+		bool set_filterpos;
 };
 
 }
