@@ -11,9 +11,10 @@ class help_formaction : public formaction {
 		virtual ~help_formaction();
 		virtual void prepare();
 		virtual void init();
-		keymap_hint_entry * get_keymap_hint();
+		virtual keymap_hint_entry * get_keymap_hint();
+		virtual std::string id() const { return "help"; }
 	private:
-		virtual void process_operation(operation op);
+		virtual void process_operation(operation op, bool automatic = false, std::vector<std::string> * args = NULL);
 		bool quit;
 };
 
