@@ -18,12 +18,14 @@ class itemview_formaction : public formaction {
 		keymap_hint_entry * get_keymap_hint();
 		virtual void handle_cmdline(const std::string& cmd);
 
+		virtual std::string id() const { return "articleview"; }
+
 		virtual void finished_qna(operation op);
 
 		std::vector<std::string> render_html(const std::string& source, std::vector<linkpair>& links, const std::string& feedurl, unsigned int render_width);
 
 	private:
-		virtual void process_operation(operation op);
+		virtual void process_operation(operation op, bool automatic = false, std::vector<std::string> * args = NULL);
 		void set_head(const std::string& s);
 
 		void render_source(std::vector<std::string>& lines, std::string desc, unsigned int width);
