@@ -11,14 +11,14 @@ namespace newsbeuter
 class reloadthread : public thread
 {
 public:
-	reloadthread(controller * c, unsigned int wt_sec, configcontainer * cf);
+	reloadthread(controller * c, time_t wt_sec, configcontainer * cf);
 	virtual ~reloadthread();
 protected:
 	virtual void run();
 private:
 	controller * ctrl;
 	time_t oldtime;
-	unsigned int waittime_sec;
+	time_t waittime_sec;
 	bool suppressed_first;
 	configcontainer * cfg;
 };
