@@ -27,17 +27,20 @@ namespace newsbeuter {
 
 	class view {
 		public:
-			view(controller * );
+			view(controller *);
 			~view();
 			void run();
 			std::string run_modal(formaction * f, const std::string& value = "");
 
+			std::string id() const;
+
 			void set_feedlist(std::vector<rss_feed>& feeds);
+			void update_visible_feeds(std::vector<rss_feed>& feeds);
 			void set_keymap(keymap * k);
 			void set_config_container(configcontainer * cfgcontainer);
-			void show_error(const char * msg);
-			void set_status(const char * msg);
-			void set_status_unlocked(const char * msg);
+			void show_error(const std::string& msg);
+			void set_status(const std::string& msg);
+			void set_status_unlocked(const std::string& msg);
 			inline controller * get_ctrl() { return ctrl; }
 			inline configcontainer * get_cfg() { return cfg; }
 			inline keymap * get_keys() { return keys; }
@@ -46,8 +49,6 @@ namespace newsbeuter {
 			inline unsigned int formaction_stack_size() { return formaction_stack.size(); }
 			char confirm(const std::string& prompt, const std::string& charset);
 
-			void write_item(const rss_item& item, const std::string& filename);
-
 			void push_itemlist(unsigned int pos);
 			void push_itemlist(rss_feed * feed);
 			void push_itemview(rss_feed * f, const std::string& guid);
@@ -57,7 +58,7 @@ namespace newsbeuter {
 
 			std::string run_filebrowser(filebrowser_type type, const std::string& default_filename = "", const std::string& dir = "");
 			std::string select_tag(const std::vector<std::string>& tags);
-			std::string select_filter(const std::vector<std::pair<std::string, std::string> >& tags);
+			std::string select_filter(const std::vector<filter_name_expr_pair>& filters);
 
 			void open_in_browser(const std::string& url);
 
@@ -73,13 +74,11 @@ namespace newsbeuter {
 
 			void notify_itemlist_change(rss_feed& feed);
 
+			std::string ask_user(const std::string& prompt);
+
+			void feedlist_mark_pos_if_visible(unsigned int pos);
+
 		protected:
-			/*
-			bool jump_to_next_unread_item(std::vector<rss_item>& items, bool begin_with_next);
-			bool jump_to_next_unread_feed(bool begin_with_next);
-			bool jump_to_previous_unread_item(std::vector<rss_item>& items, bool begin_with_prev);
-			bool jump_to_previous_unread_feed(bool begin_with_prev);
-			*/
 
 			void set_bindings();
 
