@@ -1,6 +1,7 @@
 #include <cache.h>
 #include <sqlite3.h>
 #include <cstdlib>
+#include <cstring>
 #include <configcontainer.h>
 
 #include <sstream>
@@ -11,6 +12,7 @@
 #include <logger.h>
 #include <config.h>
 #include <exceptions.h>
+#include <utils.h>
 
 using namespace newsbeuter;
 
@@ -25,33 +27,45 @@ struct cb_handler {
 };
 
 static int count_callback(void * handler, int argc, char ** argv, char ** /* azColName */) {
-	cb_handler * cbh = (cb_handler *)handler;
-	// std::cerr << "inside count_callback" << std::endl;
+	cb_handler * cbh = static_cast<cb_handler *>(handler);
+
 	if (argc>0) {
 		std::istringstream is(argv[0]);
 		int x;
 		is >> x;
 		cbh->set_count(x);
 	}
-	// std::cerr << "count_callback: count = " << cbh->count() << std::endl;
+
+	return 0;
+}
+
+static int single_int_callback(void * handler, int argc, char ** argv, char ** /* azColName */) {
+	int * value = static_cast<int *>(handler);
+	if (argc>0 && argv[0]) {
+		std::istringstream is(argv[0]);
+		int x;
+		is >> x;
+		*value = x;
+	}
 	return 0;
 }
 
 static int rssfeed_callback(void * myfeed, int argc, char ** argv, char ** /* azColName */) {
-	rss_feed * feed = (rss_feed *)myfeed;
+	rss_feed * feed = static_cast<rss_feed *>(myfeed);
 	// normaly, this shouldn't happen, but we keep the assert()s here nevertheless
-	assert(argc == 2);
+	assert(argc == 3);
 	assert(argv[0] != NULL);
 	assert(argv[1] != NULL);
+	assert(argv[2] != NULL);
 	feed->set_title(argv[0]);
 	feed->set_link(argv[1]);
-	GetLogger().log(LOG_INFO, "rssfeed_callback: title = %s link = %s",argv[0],argv[1]);
-	// std::cerr << "callback: feed->title = " << feed->title() << std::endl;
+	feed->set_rtl(strcmp(argv[2],"1")==0);
+	GetLogger().log(LOG_INFO, "rssfeed_callback: title = %s link = %s is_rtl = %s",argv[0],argv[1], argv[2]);
 	return 0;
 }
 
 static int vectorofstring_callback(void * vp, int argc, char ** argv, char ** /* azColName */) {
-	std::vector<std::string> * vectorptr = (std::vector<std::string> *)vp;
+	std::vector<std::string> * vectorptr = static_cast<std::vector<std::string> *>(vp);
 	assert(argc == 1);
 	assert(argv[0] != NULL);
 	vectorptr->push_back(std::string(argv[0]));
@@ -60,7 +74,7 @@ static int vectorofstring_callback(void * vp, int argc, char ** argv, char ** /*
 }
 
 static int rssitem_callback(void * myfeed, int argc, char ** argv, char ** /* azColName */) {
-	rss_feed * feed = (rss_feed *)myfeed;
+	rss_feed * feed = static_cast<rss_feed *>(myfeed);
 	assert (argc == 12);
 	rss_item item(NULL);
 	item.set_guid(argv[0]);
@@ -89,7 +103,7 @@ static int rssitem_callback(void * myfeed, int argc, char ** argv, char ** /* az
 }
 
 static int rssitemvector_callback(void * vector, int argc, char ** argv, char ** /* azColName */) {
-	std::vector<rss_item> * items = (std::vector<rss_item> *)vector;
+	std::vector<rss_item> * items = static_cast<std::vector<rss_item> *>(vector);
 
 	assert (argc == 12);
 	rss_item item(NULL);
@@ -118,7 +132,7 @@ static int rssitemvector_callback(void * vector, int argc, char ** argv, char **
 }
 
 static int search_item_callback(void * myfeed, int argc, char ** argv, char ** /* azColName */) {
-	std::vector<rss_item> * items = (std::vector<rss_item> *)myfeed;
+	std::vector<rss_item> * items = static_cast<std::vector<rss_item> *>(myfeed);
 	assert (argc == 12);
 	rss_item item(NULL);
 	item.set_guid(argv[0]);
@@ -157,10 +171,8 @@ cache::cache(const std::string& cachefile, configcontainer * c) : db(0),cfg(c),
 	// TODO: this should be refactored into an exception
 	if (error != SQLITE_OK) {
 		GetLogger().log(LOG_ERROR,"couldn't sqlite3_open(%s): error = %d", cachefile.c_str(), error);
-		char buf[1024];
-		snprintf(buf, sizeof(buf), _("Error: opening the cache file `%s' failed: %s"), cachefile.c_str(), sqlite3_errmsg(db));
+		std::cout << utils::strprintf(_("Error: opening the cache file `%s' failed: %s"), cachefile.c_str(), sqlite3_errmsg(db)) << std::endl;
 		sqlite3_close(db);
-		std::cout << buf << std::endl;
 		::exit(EXIT_FAILURE);
 	}
 
@@ -247,12 +259,53 @@ void cache::populate_tables() {
 		GetLogger().log(LOG_DEBUG, "cache::populate_tables: ANALYZE indices (6) rc = %d", rc);
 	}
 
+	rc = sqlite3_exec(db, "ALTER TABLE rss_feed ADD lastmodified INTEGER(11);", NULL, NULL, NULL);
+	GetLogger().log(LOG_DEBUG, "cache::populate_tables: ALTER TABLE rss_feed ADD lastmodified: rc = %d", rc);
+
+	rc = sqlite3_exec(db, "CREATE INDEX IF NOT EXISTS idx_lastmodified ON rss_feed(lastmodified);", NULL, NULL, NULL);
+	GetLogger().log(LOG_DEBUG, "cache::populate_tables: CREATE INDEX ON rss_feed(lastmodified) rc = %d", rc);
+
+	rc = sqlite3_exec(db, "ALTER TABLE rss_item ADD deleted INTEGER(1) NOT NULL DEFAULT 0;", NULL, NULL, NULL);
+	GetLogger().log(LOG_DEBUG, "cache::populate_tables: ALTER TABLE rss_item (7) rc = %d", rc);
+
+	rc = sqlite3_exec(db, "CREATE INDEX IF NOT EXISTS idx_deleted ON rss_item(deleted);", NULL, NULL, NULL);
+	GetLogger().log(LOG_DEBUG, "cache::populate_tables: CREATE INDEX ON rss_item(deleted) rc = %d", rc);
+
+	rc = sqlite3_exec(db, "ALTER TABLE rss_feed ADD is_rtl INTEGER(1) NOT NULL DEFAULT 0;", NULL, NULL, NULL);
+	GetLogger().log(LOG_DEBUG, "cache::populate_tables: ALTER TABLE rss_feed (8) rc = %d", rc);
+
+}
+
+time_t cache::get_lastmodified(const std::string& feedurl) {
+	scope_mutex lock(mtx);
+	time_t result = 0;
+	std::string query = prepare_query("SELECT lastmodified FROM rss_feed WHERE rssurl = '%q';", feedurl.c_str());
+	GetLogger().log(LOG_DEBUG, "running: query: %s", query.c_str());
+	int rc = sqlite3_exec(db, query.c_str(), single_int_callback, &result, NULL);
+	if (rc != SQLITE_OK) {
+		GetLogger().log(LOG_CRITICAL, "query \"%s\" failed: error = %d", query.c_str(), rc);
+		throw dbexception(db);
+	}
+	return result;
+}
+
+void cache::set_lastmodified(const std::string& feedurl, time_t lastmod) {
+	if (lastmod > 0) {
+		std::string query = prepare_query("UPDATE rss_feed SET lastmodified = '%d' WHERE rssurl = '%q';", lastmod, feedurl.c_str());
+		int rc = sqlite3_exec(db, query.c_str(), NULL, NULL, NULL);
+		GetLogger().log(LOG_DEBUG, "ran SQL statement: %s result = %d", query.c_str(), rc);
+	}
+}
+
+void cache::mark_item_deleted(const std::string& guid, bool b) {
+	std::string query = prepare_query("UPDATE rss_item SET deleted = %u WHERE guid = '%q'", b ? 1 : 0, guid.c_str());
+	int rc = sqlite3_exec(db, query.c_str(), NULL, NULL, NULL);
+	GetLogger().log(LOG_DEBUG, "cache::mark_item_deleted ran SQL statement: %s result = %d", query.c_str(), rc);
 }
 
 
-
 std::vector<std::string> cache::get_feed_urls() {
-	mtx->lock();
+	scope_mutex lock(mtx);
 	std::string query = "SELECT rssurl FROM rss_feed;";
 
 	std::vector<std::string> urls;
@@ -260,36 +313,36 @@ std::vector<std::string> cache::get_feed_urls() {
 	int rc = sqlite3_exec(db, query.c_str(), vectorofstring_callback,&urls, NULL);
 	assert(rc == SQLITE_OK);
 
-	mtx->unlock();
-
 	return urls;
 }
 
 
 // this function writes an rss_feed including all rss_items to the database
 void cache::externalize_rssfeed(rss_feed& feed) {
+	scope_measure m1("cache::externalize_feed");
 	if (feed.rssurl().substr(0,6) == "query:")
 		return;
 
-	mtx->lock();
-	std::ostringstream query;
-	query << "SELECT count(*) FROM rss_feed WHERE rssurl = '" << feed.rssurl() << "';";
-	cb_handler count_cbh;
-	int rc = sqlite3_exec(db,query.str().c_str(),count_callback,&count_cbh,NULL);
-	int count = count_cbh.count();
-	GetLogger().log(LOG_DEBUG, "cache::externalize_rss_feed: rss_feeds with rssurl = '%s': found %d",feed.rssurl().c_str(), count);
-	if (count > 0) {
-		std::string updatequery = prepare_query("UPDATE rss_feed SET title = '%q', url = '%q' WHERE rssurl = '%q';",feed.title_raw().c_str(),feed.link().c_str(), feed.rssurl().c_str());
-		rc = sqlite3_exec(db,updatequery.c_str(),NULL,NULL,NULL);
-		GetLogger().log(LOG_DEBUG,"ran SQL statement: %s", updatequery.c_str());
-	} else {
-		std::string insertquery = prepare_query("INSERT INTO rss_feed (rssurl, url, title) VALUES ( '%q', '%q', '%q' );", feed.rssurl().c_str(), feed.link().c_str(), feed.title_raw().c_str());
-		rc = sqlite3_exec(db,insertquery.c_str(),NULL,NULL,NULL);
-		GetLogger().log(LOG_DEBUG,"ran SQL statement: %s", insertquery.c_str());
+	sqlite3_exec(db, "BEGIN;", NULL, NULL, NULL);
+
+	{
+		scope_mutex lock(mtx);
+
+		std::string countquery = prepare_query("SELECT count(*) FROM rss_feed WHERE rssurl = '%q';", feed.rssurl().c_str());
+		cb_handler count_cbh;
+		int rc = sqlite3_exec(db,countquery.c_str(),count_callback,&count_cbh,NULL);
+		int count = count_cbh.count();
+		GetLogger().log(LOG_DEBUG, "cache::externalize_rss_feed: rss_feeds with rssurl = '%s': found %d",feed.rssurl().c_str(), count);
+		if (count > 0) {
+			std::string updatequery = prepare_query("UPDATE rss_feed SET title = '%q', url = '%q', is_rtl = %u WHERE rssurl = '%q';",feed.title_raw().c_str(),feed.link().c_str(), feed.is_rtl() ? 1 : 0, feed.rssurl().c_str());
+			rc = sqlite3_exec(db,updatequery.c_str(),NULL,NULL,NULL);
+			GetLogger().log(LOG_DEBUG,"ran SQL statement: %s", updatequery.c_str());
+		} else {
+			std::string insertquery = prepare_query("INSERT INTO rss_feed (rssurl, url, title, is_rtl) VALUES ( '%q', '%q', '%q', %u );", feed.rssurl().c_str(), feed.link().c_str(), feed.title_raw().c_str(), feed.is_rtl() ? 1 : 0);
+			rc = sqlite3_exec(db,insertquery.c_str(),NULL,NULL,NULL);
+			GetLogger().log(LOG_DEBUG,"ran SQL statement: %s", insertquery.c_str());
+		}
 	}
-
-
-	mtx->unlock();
 	
 	unsigned int max_items = cfg->get_configvalue_as_int("max-items");
 
@@ -307,15 +360,17 @@ void cache::externalize_rssfeed(rss_feed& feed) {
 	for (std::vector<rss_item>::reverse_iterator it=feed.items().rbegin(); it != feed.items().rend(); ++it) {
 		update_rssitem(*it, feed.rssurl());
 	}
+	sqlite3_exec(db, "END;", NULL, NULL, NULL);
 }
 
 // this function reads an rss_feed including all of its rss_items.
 // the feed parameter needs to have the rssurl member set.
 void cache::internalize_rssfeed(rss_feed& feed) {
+	scope_measure m1("cache::internalize_rssfeed");
 	if (feed.rssurl().substr(0,6) == "query:")
 		return;
 
-	mtx->lock();
+	scope_mutex lock(mtx);
 
 	/* first, we check whether the feed is there at all */
 	std::string query = prepare_query("SELECT count(*) FROM rss_feed WHERE rssurl = '%q';",feed.rssurl().c_str());
@@ -324,22 +379,19 @@ void cache::internalize_rssfeed(rss_feed& feed) {
 	int rc = sqlite3_exec(db,query.c_str(),count_callback,&count_cbh,NULL);
 	if (rc != SQLITE_OK) {
 		GetLogger().log(LOG_CRITICAL,"query \"%s\" failed: error = %d", query.c_str(), rc);
-		mtx->unlock();
 		throw dbexception(db);
 	}
 
 	if (count_cbh.count() == 0) {
-		mtx->unlock();
 		return;
 	}
 
 	/* then we first read the feed from the database */
-	query = prepare_query("SELECT title, url FROM rss_feed WHERE rssurl = '%q';",feed.rssurl().c_str());
+	query = prepare_query("SELECT title, url, is_rtl FROM rss_feed WHERE rssurl = '%q';",feed.rssurl().c_str());
 	GetLogger().log(LOG_DEBUG,"running query: %s",query.c_str());
 	rc = sqlite3_exec(db,query.c_str(),rssfeed_callback,&feed,NULL);
 	if (rc != SQLITE_OK) {
 		GetLogger().log(LOG_CRITICAL,"query \"%s\" failed: error = %d", query.c_str(), rc);
-		mtx->unlock();
 		throw dbexception(db);
 	}
 
@@ -348,12 +400,11 @@ void cache::internalize_rssfeed(rss_feed& feed) {
 	}
 
 	/* ...and then the associated items */
-	query = prepare_query("SELECT guid,title,author,url,pubDate,content,unread,feedurl,enclosure_url,enclosure_type,enqueued,flags FROM rss_item WHERE feedurl = '%q' ORDER BY pubDate DESC, id DESC;",feed.rssurl().c_str());
+	query = prepare_query("SELECT guid,title,author,url,pubDate,content,unread,feedurl,enclosure_url,enclosure_type,enqueued,flags FROM rss_item WHERE feedurl = '%q' AND deleted = 0 ORDER BY pubDate DESC, id DESC;",feed.rssurl().c_str());
 	GetLogger().log(LOG_DEBUG,"running query: %s",query.c_str());
 	rc = sqlite3_exec(db,query.c_str(),rssitem_callback,&feed,NULL);
 	if (rc != SQLITE_OK) {
 		GetLogger().log(LOG_CRITICAL,"query \"%s\" failed: error = %d", query.c_str(), rc);
-		mtx->unlock();
 		throw dbexception(db);
 	}
 
@@ -366,30 +417,35 @@ void cache::internalize_rssfeed(rss_feed& feed) {
 	unsigned int max_items = cfg->get_configvalue_as_int("max-items");
 	
 	if (max_items > 0 && feed.items().size() > max_items) {
+		std::vector<rss_item> flagged_items;
 		std::vector<rss_item>::iterator it=feed.items().begin();
 		for (unsigned int i=0;i<max_items;++i)
 			++it;
 		for (unsigned int i=max_items;i<feed.items().size();++i) {
-			delete_item(feed.items()[i]);	
+			if (feed.items()[i].flags().length() == 0) {
+				delete_item(feed.items()[i]);
+			} else {
+				flagged_items.push_back(feed.items()[i]);
+			}
 		}	
 		feed.items().erase(it, feed.items().end()); // delete old entries
+		if (flagged_items.size() > 0) {
+			feed.items().insert(feed.items().end(), flagged_items.begin(), flagged_items.end()); // if some flagged articles were saved, append them
+		}
 	}
-	mtx->unlock();
+	feed.sort(cfg->get_configvalue("article-sort-order"));
 }
 
 void cache::get_latest_items(std::vector<rss_item>& items, unsigned int limit) {
-	mtx->lock();
+	scope_mutex lock(mtx);
 	std::string query = prepare_query("SELECT guid,title,author,url,pubDate,content,unread,feedurl,enclosure_url,enclosure_type,enqueued,flags "
-									"FROM rss_item ORDER BY pubDate DESC, id DESC LIMIT %d;", limit);
+									"FROM rss_item WHERE deleted = 0 ORDER BY pubDate DESC, id DESC LIMIT %d;", limit);
 	GetLogger().log(LOG_DEBUG, "running query: %s", query.c_str());
 	int rc = sqlite3_exec(db, query.c_str(), rssitemvector_callback, &items, NULL);
 	if (rc != SQLITE_OK) {
 		GetLogger().log(LOG_CRITICAL,"query \"%s\" failed: error = %d", query.c_str(), rc);
-		mtx->unlock();
 		throw dbexception(db);
 	}
-
-	mtx->unlock();
 }
 
 rss_feed cache::get_feed_by_url(const std::string& feedurl) {
@@ -397,20 +453,17 @@ rss_feed cache::get_feed_by_url(const std::string& feedurl) {
 	std::string query;
 	int rc;
 
-	mtx->lock();
-	
-	query = prepare_query("SELECT title, url FROM rss_feed WHERE rssurl = '%q';",feedurl.c_str());
+	scope_mutex lock(mtx);
+
+	query = prepare_query("SELECT title, url, is_rtl FROM rss_feed WHERE rssurl = '%q';",feedurl.c_str());
 	GetLogger().log(LOG_DEBUG,"running query: %s",query.c_str());
 
 	rc = sqlite3_exec(db,query.c_str(),rssfeed_callback,&feed,NULL);
 	if (rc != SQLITE_OK) {
 		GetLogger().log(LOG_CRITICAL,"query \"%s\" failed: error = %d", query.c_str(), rc);
-		mtx->unlock();
 		throw dbexception(db);
 	}
 
-	mtx->unlock();
-
 	return feed;
 }
 
@@ -419,12 +472,11 @@ std::vector<rss_item> cache::search_for_items(const std::string& querystr, const
 	std::vector<rss_item> items;
 	int rc;
 
-	mtx->lock();
-
+	scope_mutex lock(mtx);
 	if (feedurl.length() > 0) {
-		query = prepare_query("SELECT guid,title,author,url,pubDate,content,unread,feedurl,enclosure_url,enclosure_type,enqueued,flags FROM rss_item WHERE (title LIKE '%%%q%%' OR content LIKE '%%%q%%') AND feedurl = '%q' ORDER BY pubDate DESC, id DESC;",querystr.c_str(), querystr.c_str(), feedurl.c_str());
+		query = prepare_query("SELECT guid,title,author,url,pubDate,content,unread,feedurl,enclosure_url,enclosure_type,enqueued,flags FROM rss_item WHERE (title LIKE '%%%q%%' OR content LIKE '%%%q%%') AND feedurl = '%q' AND deleted = 0 ORDER BY pubDate DESC, id DESC;",querystr.c_str(), querystr.c_str(), feedurl.c_str());
 	} else {
-		query = prepare_query("SELECT guid,title,author,url,pubDate,content,unread,feedurl,enclosure_url,enclosure_type,enqueued,flags FROM rss_item WHERE (title LIKE '%%%q%%' OR content LIKE '%%%q%%') ORDER BY pubDate DESC, id DESC;",querystr.c_str(), querystr.c_str());
+		query = prepare_query("SELECT guid,title,author,url,pubDate,content,unread,feedurl,enclosure_url,enclosure_type,enqueued,flags FROM rss_item WHERE (title LIKE '%%%q%%' OR content LIKE '%%%q%%') AND deleted = 0 ORDER BY pubDate DESC, id DESC;",querystr.c_str(), querystr.c_str());
 	}
 
 	GetLogger().log(LOG_DEBUG,"running query: %s",query.c_str());
@@ -432,12 +484,9 @@ std::vector<rss_item> cache::search_for_items(const std::string& querystr, const
 	rc = sqlite3_exec(db,query.c_str(),search_item_callback,&items,NULL);
 	if (rc != SQLITE_OK) {
 		GetLogger().log(LOG_CRITICAL,"query \"%s\" failed: error = %d", query.c_str(), rc);
-		mtx->unlock();
 		throw dbexception(db);
 	}
 
-	mtx->unlock();
-
 	return items;
 }
 
@@ -452,17 +501,16 @@ void cache::delete_item(const rss_item& item) {
 }
 
 void cache::do_vacuum() {
-	mtx->lock();
+	scope_mutex lock(mtx);
 	const char * vacuum_query = "VACUUM;";
 	int rc = sqlite3_exec(db,vacuum_query,NULL,NULL,NULL);
 	if (rc != SQLITE_OK) {
 		GetLogger().log(LOG_CRITICAL,"query \"%s\" failed: error = %d", vacuum_query, rc);
 	}
-	mtx->unlock();
 }
 
 void cache::cleanup_cache(std::vector<rss_feed>& feeds) {
-	mtx->lock();
+	mtx->lock(); // we don't use the scope_mutex here... see comments below
 
 	/*
 	 * cache cleanup means that all entries in both the rss_feed and rss_item tables that are associated with
@@ -503,7 +551,7 @@ void cache::cleanup_cache(std::vector<rss_feed>& feeds) {
 		rc = sqlite3_exec(db,cleanup_rss_feeds_statement.c_str(),NULL,NULL,NULL);
 		if (rc != SQLITE_OK) {
 			GetLogger().log(LOG_CRITICAL,"query \"%s\" failed: error = %d", cleanup_rss_feeds_statement.c_str(), rc);
-			mtx->unlock();
+			mtx->unlock(); // see above
 			throw dbexception(db);
 		}
 
@@ -511,13 +559,10 @@ void cache::cleanup_cache(std::vector<rss_feed>& feeds) {
 		rc = sqlite3_exec(db,cleanup_rss_items_statement.c_str(),NULL,NULL,NULL);
 		if (rc != SQLITE_OK) {
 			GetLogger().log(LOG_CRITICAL,"query \"%s\" failed: error = %d", cleanup_rss_items_statement.c_str(), rc);
-			mtx->unlock();
+			mtx->unlock(); // see above
 			throw dbexception(db);
 		}
 
-		// rc = sqlite3_exec(db,"VACUUM;",NULL,NULL,NULL);
-		// assert(rc == SQLITE_OK);
-
 		// WARNING: THE MISSING UNLOCK OPERATION IS MISSING FOR A PURPOSE!
 		// It's missing so that no database operation can occur after the cache cleanup!
 		// mtx->unlock();
@@ -528,49 +573,44 @@ void cache::cleanup_cache(std::vector<rss_feed>& feeds) {
 
 /* this function writes an rss_item to the database, also checking whether this item already exists in the database */
 void cache::update_rssitem(rss_item& item, const std::string& feedurl) {
-	mtx->lock();
+	scope_mutex lock(mtx);
 	std::string query = prepare_query("SELECT count(*) FROM rss_item WHERE guid = '%q';",item.guid().c_str());
 	cb_handler count_cbh;
 	GetLogger().log(LOG_DEBUG,"running query: %s", query.c_str());
 	int rc = sqlite3_exec(db,query.c_str(),count_callback,&count_cbh,NULL);
 	if (rc != SQLITE_OK) {
 		GetLogger().log(LOG_CRITICAL,"query \"%s\" failed: error = %d", query.c_str(), rc);
-		mtx->unlock();
 		throw dbexception(db);
 	}
 	if (count_cbh.count() > 0) {
-		std::string update = prepare_query("UPDATE rss_item SET title = '%q', author = '%q', url = '%q', feedurl = '%q', content = '%q', enclosure_url = '%q', enclosure_type = '%q', flags = '%q' WHERE guid = '%q'",
+		std::string update = prepare_query("UPDATE rss_item SET title = '%q', author = '%q', url = '%q', feedurl = '%q', content = '%q', enclosure_url = '%q', enclosure_type = '%q' WHERE guid = '%q'",
 			item.title_raw().c_str(), item.author_raw().c_str(), item.link().c_str(), 
 			feedurl.c_str(), item.description_raw().c_str(), 
 			item.enclosure_url().c_str(), item.enclosure_type().c_str(),
-			item.flags().c_str(),
 			item.guid().c_str());
 		GetLogger().log(LOG_DEBUG,"running query: %s", update.c_str());
 		rc = sqlite3_exec(db,update.c_str(),NULL,NULL,NULL);
 		if (rc != SQLITE_OK) {
 			GetLogger().log(LOG_CRITICAL,"query \"%s\" failed: error = %d", update.c_str(), rc);
-			mtx->unlock();
 			throw dbexception(db);
 		}
 	} else {
-		std::string insert = prepare_query("INSERT INTO rss_item (guid,title,author,url,feedurl,pubDate,content,unread,enclosure_url,enclosure_type,enqueued,flags) "
-								"VALUES ('%q','%q','%q','%q','%q','%u','%q',1,'%q','%q',%d, '%q')",
+		std::string insert = prepare_query("INSERT INTO rss_item (guid,title,author,url,feedurl,pubDate,content,unread,enclosure_url,enclosure_type,enqueued) "
+								"VALUES ('%q','%q','%q','%q','%q','%u','%q',1,'%q','%q',%d)",
 								item.guid().c_str(), item.title_raw().c_str(), item.author_raw().c_str(), 
 								item.link().c_str(), feedurl.c_str(), item.pubDate_timestamp(), item.description_raw().c_str(),
-								item.enclosure_url().c_str(), item.enclosure_type().c_str(), item.enqueued() ? 1 : 0, item.flags().c_str());
+								item.enclosure_url().c_str(), item.enclosure_type().c_str(), item.enqueued() ? 1 : 0);
 		GetLogger().log(LOG_DEBUG,"running query: %s", insert.c_str());
 		rc = sqlite3_exec(db,insert.c_str(),NULL,NULL,NULL);
 		if (rc != SQLITE_OK) {
 			GetLogger().log(LOG_CRITICAL,"query \"%s\" failed: error = %d", insert.c_str(), rc);
-			mtx->unlock();
 			throw dbexception(db);
 		}
 	}
-	mtx->unlock();
 }
 
 void cache::catchup_all(rss_feed& feed) {
-	mtx->lock();
+	scope_mutex lock(mtx);
 	std::string query = "UPDATE rss_item SET unread = '0' WHERE unread != '0' AND guid IN (";
 
 	for (std::vector<rss_item>::iterator it=feed.items().begin();it!=feed.items().end();++it) {
@@ -582,16 +622,14 @@ void cache::catchup_all(rss_feed& feed) {
 	int rc = sqlite3_exec(db, query.c_str(), NULL, NULL, NULL);
 	if (rc != SQLITE_OK) {
 		GetLogger().log(LOG_CRITICAL,"query \"%s\" failed: error = %d", query.c_str(), rc);
-		mtx->unlock();
 		throw dbexception(db);
 	}
-
-	mtx->unlock();
 }
 
 /* this function marks all rss_items (optionally of a certain feed url) as read */
 void cache::catchup_all(const std::string& feedurl) {
-	mtx->lock();
+	scope_mutex lock(mtx);
+
 	std::string query;
 	if (feedurl.length() > 0) {
 		query = prepare_query("UPDATE rss_item SET unread = '0' WHERE unread != '0' AND feedurl = '%q';", feedurl.c_str());
@@ -602,22 +640,20 @@ void cache::catchup_all(const std::string& feedurl) {
 	int rc = sqlite3_exec(db, query.c_str(), NULL, NULL, NULL);
 	if (rc != SQLITE_OK) {
 		GetLogger().log(LOG_CRITICAL,"query \"%s\" failed: error = %d", query.c_str(), rc);
-		mtx->unlock();
 		throw dbexception(db);
 	}
-	mtx->unlock();
 }
 
 /* this function updates the unread and enqueued flags */
 void cache::update_rssitem_unread_and_enqueued(rss_item& item, const std::string& feedurl) {
-	mtx->lock();
+	scope_mutex lock(mtx);
+
 	std::string query = prepare_query("SELECT count(*) FROM rss_item WHERE guid = '%q';",item.guid().c_str());
 	cb_handler count_cbh;
 	GetLogger().log(LOG_DEBUG,"running query: %s", query.c_str());
 	int rc = sqlite3_exec(db,query.c_str(),count_callback,&count_cbh,NULL);
 	if (rc != SQLITE_OK) {
 		GetLogger().log(LOG_CRITICAL,"query \"%s\" failed: error = %d", query.c_str(), rc);
-		mtx->unlock();
 		throw dbexception(db);
 	}
 
@@ -628,24 +664,21 @@ void cache::update_rssitem_unread_and_enqueued(rss_item& item, const std::string
 		rc = sqlite3_exec(db,update.c_str(),NULL,NULL,NULL);
 		if (rc != SQLITE_OK) {
 			GetLogger().log(LOG_CRITICAL,"query \"%s\" failed: error = %d", update.c_str(), rc);
-			mtx->unlock();
 			throw dbexception(db);
 		}
 	} else {
 		std::string insert = prepare_query("INSERT INTO rss_item (guid,title,author,url,feedurl,pubDate,content,unread,enclosure_url,enclosure_type,enqueued,flags) "
-										"VALUES ('%q','%q','%q','%q','%q','%u','%q',1,'%q','%q',%d, '%q')",
+										"VALUES ('%q','%q','%q','%q','%q','%u','%q',1,'%q','%q',%d)",
 										item.guid().c_str(), item.title_raw().c_str(), item.author_raw().c_str(), 
 										item.link().c_str(), feedurl.c_str(), item.pubDate_timestamp(), item.description_raw().c_str(),
-										item.enclosure_url().c_str(), item.enclosure_type().c_str(), item.enqueued() ? 1 : 0, item.flags().c_str());
+										item.enclosure_url().c_str(), item.enclosure_type().c_str(), item.enqueued() ? 1 : 0);
 		GetLogger().log(LOG_DEBUG,"running query: %s", insert.c_str());
 		rc = sqlite3_exec(db,insert.c_str(),NULL,NULL,NULL);
 		if (rc != SQLITE_OK) {
 			GetLogger().log(LOG_CRITICAL,"query \"%s\" failed: error = %d", insert.c_str(), rc);
-			mtx->unlock();
 			throw dbexception(db);
 		}
 	}
-	mtx->unlock();
 }
 
 /* helper function to wrap std::string around the sqlite3_*mprintf function */
@@ -656,20 +689,48 @@ std::string cache::prepare_query(const char * format, ...) {
 	char * query = sqlite3_vmprintf(format, ap);
 	if (query) {
 		result = query;
-		free(query);
+		sqlite3_free(query);
 	}
+	va_end(ap);
 	return result;
 }
 
 void cache::update_rssitem_flags(rss_item& item) {
-	mtx->lock();
+	scope_mutex lock(mtx);
+
 	std::string update = prepare_query("UPDATE rss_item SET flags = '%q' WHERE guid = '%q';", item.flags().c_str(), item.guid().c_str());
 	GetLogger().log(LOG_DEBUG,"running query: %s", update.c_str());
 	int rc = sqlite3_exec(db,update.c_str(), NULL, NULL, NULL);
 	if (rc != SQLITE_OK) {
 		GetLogger().log(LOG_CRITICAL, "query \"%s\" failed: error = %d", update.c_str(), rc);
-		mtx->unlock();
 		throw dbexception(db);
 	}
-	mtx->unlock();
+}
+
+void cache::remove_old_deleted_items(const std::string& rssurl, const std::vector<std::string>& guids) {
+	scope_measure m1("cache::remove_old_deleted_items");
+	std::string guidset = "(";
+	for (std::vector<std::string>::const_iterator it=guids.begin();it!=guids.end();++it) {
+		guidset.append(prepare_query("'%q', ", it->c_str()));
+	}
+	guidset.append("'')");
+	std::string query = prepare_query("DELETE FROM rss_item WHERE feedurl = '%q' AND deleted = 1 AND guid NOT IN %s;", rssurl.c_str(), guidset.c_str());
+	int rc = sqlite3_exec(db, query.c_str(), NULL, NULL, NULL);
+	if (rc != SQLITE_OK) {
+		GetLogger().log(LOG_CRITICAL, "query \"%s\" failed: error = %d", query.c_str(), rc);
+		throw dbexception(db);
+	} else {
+		GetLogger().log(LOG_DEBUG, "cache::remove_old_deleted_items: executed query successfully: %s", query.c_str());
+	}
+}
+
+unsigned int cache::get_unread_count() {
+	scope_mutex lock(mtx);
+
+	std::string countquery = "SELECT count(id) FROM rss_item WHERE unread = 1;";
+	cb_handler count_cbh;
+	int rc = sqlite3_exec(db,countquery.c_str(),count_callback,&count_cbh,NULL);
+	unsigned int count = static_cast<unsigned int>(count_cbh.count());
+	GetLogger().log(LOG_DEBUG, "cache::get_unread_count: rc = %d count = %u", rc, count);
+	return count;
 }
