@@ -4,6 +4,7 @@
 #include <sstream>
 #include <iostream>
 #include <utils.h>
+#include <config.h>
 
 #include <sys/types.h>
 #include <pwd.h>
@@ -19,7 +20,7 @@ configcontainer::configcontainer()
 	config_data["browser"]         = configdata("lynx", configdata::PATH);
 	config_data["use-proxy"]       = configdata("no", configdata::BOOL);
 	config_data["auto-reload"]     = configdata("no", configdata::BOOL);
-	config_data["reload-time"]     = configdata("30", configdata::INT);
+	config_data["reload-time"]     = configdata("60", configdata::INT);
 	config_data["max-items"]       = configdata("0", configdata::INT);
 	config_data["save-path"]       = configdata("~/", configdata::PATH);
 	config_data["download-path"]   = configdata("~/", configdata::PATH);
@@ -38,6 +39,7 @@ configcontainer::configcontainer()
 	config_data["notify-screen"]   = configdata("no", configdata::BOOL);
 	config_data["notify-xterm"]    = configdata("no", configdata::BOOL);
 	config_data["notify-program"]  = configdata("", configdata::PATH);
+	config_data["notify-format"]   = configdata(_("newsbeuter: finished reload, %f unread feeds (%n unread articles total)"), configdata::STR);
 	config_data["datetime-format"] = configdata("%b %d", configdata::STR);
 	config_data["urls-source"]     = configdata("local", configdata::STR);
 	config_data["bloglines-auth"]  = configdata("", configdata::STR);
@@ -45,6 +47,24 @@ configcontainer::configcontainer()
 	config_data["bookmark-cmd"]    = configdata("", configdata::STR);
 	config_data["opml-url"]        = configdata("", configdata::STR);
 	config_data["html-renderer"]   = configdata("internal", configdata::PATH);
+	config_data["feedlist-format"] = configdata("%4i %n %11u %t", configdata::STR);
+	config_data["articlelist-format"] = configdata("%4i %f %D   %?T?|%-17T|  &?%t", configdata::STR);
+	config_data["text-width"]      = configdata("0", configdata::INT);
+	config_data["always-display-description"] = configdata("false", configdata::BOOL);
+	config_data["reload-only-visible-feeds"] = configdata("false", configdata::BOOL);
+	config_data["article-sort-order"] = configdata("date", configdata::STR);
+
+	/* undocumented: */
+	config_data["feedlist-title-format"] = configdata(_("%N %V - Your feeds (%u unread, %t total)%?T? - tag `%T'&?"), configdata::STR);
+	config_data["urlview-title-format"] = configdata(_("%N %V - URLs"), configdata::STR);
+	config_data["articlelist-title-format"] = configdata(_("%N %V - Articles in feed '%T' (%u unread, %t total) - %U"), configdata::STR);
+	config_data["searchresult-title-format"] = configdata(_("%N %V - Search result (%u unread, %t total)"), configdata::STR);
+	config_data["filebrowser-title-format"] = configdata(_("%N %V - %?O?Open File&Save File? - %f"), configdata::STR);
+	config_data["help-title-format"] = configdata(_("%N %V - Help"), configdata::STR);
+	config_data["selecttag-title-format"] = configdata(_("%N %V - Select Tag"), configdata::STR);
+	config_data["selectfilter-title-format"] = configdata(_("%N %V - Select Filter"), configdata::STR);
+	config_data["itemview-title-format"] = configdata(_("%N %V - Article '%T'"), configdata::STR);
+	config_data["urlview-title-format"] = configdata(_("%N %V - URLs"), configdata::STR);
 }
 
 configcontainer::~configcontainer()
