@@ -1,3 +1,4 @@
+#include <config.h>
 #include <view.h>
 #include <controller.h>
 #include <configparser.h>
@@ -8,8 +9,10 @@
 #include <logger.h>
 #include <utils.h>
 #include <stflpp.h>
-#include <sstream>
+#include <exception.h>
+#include <formatstring.h>
 #include <cstdlib>
+#include <cstring>
 #include <iostream>
 #include <fstream>
 #include <cerrno>
@@ -19,8 +22,9 @@
 #include <cassert>
 #include <signal.h>
 #include <sys/utsname.h>
+#include <langinfo.h>
 
-#include <nxml.h>
+#include <_nxml.h>
 
 #include <sys/types.h>
 #include <sys/wait.h>
@@ -28,11 +32,11 @@
 
 #include <ncursesw/ncurses.h>
 
-#include <config.h>
-
 using namespace newsbeuter;
 
-static std::string lock_file = "lock.pid";
+#define LOCK_SUFFIX ".lock"
+
+static std::string lock_file;
 
 void ctrl_c_action(int sig) {
 	GetLogger().log(LOG_DEBUG,"caugh signal %d",sig);
@@ -51,12 +55,10 @@ void ignore_signal(int sig) {
 void omg_a_child_died(int /* sig */) {
 	pid_t pid;
 	int stat;
-	while ((pid = waitpid(-1,&stat,WNOHANG)) > 0);
+	while ((pid = waitpid(-1,&stat,WNOHANG)) > 0) { }
 }
 
-controller::controller() : v(0), rsscache(0), url_file("urls"), cache_file("cache.db"), config_file("config"), queue_file("queue"), refresh_on_start(false), cfg(0) {
-	std::ostringstream cfgfile;
-
+controller::controller() : v(0), urlcfg(0), rsscache(0), url_file("urls"), cache_file("cache.db"), config_file("config"), queue_file("queue"), refresh_on_start(false), cfg(0) {
 	char * cfgdir;
 	if (!(cfgdir = ::getenv("HOME"))) {
 		struct passwd * spw = ::getpwuid(::getuid());
@@ -64,24 +66,16 @@ controller::controller() : v(0), rsscache(0), url_file("urls"), cache_file("cach
 			cfgdir = spw->pw_dir;
 		} else {
 			std::cout << _("Fatal error: couldn't determine home directory!") << std::endl;
-			char buf[1024];
-			snprintf(buf, sizeof(buf), _("Please set the HOME environment variable or add a valid user for UID %u!"), ::getuid());
-			std::cout << buf << std::endl;
+			std::cout << utils::strprintf(_("Please set the HOME environment variable or add a valid user for UID %u!"), ::getuid()) << std::endl;
 			::exit(EXIT_FAILURE);
 		}
 	}
 	config_dir = cfgdir;
 
-
 	config_dir.append(NEWSBEUTER_PATH_SEP);
 	config_dir.append(NEWSBEUTER_CONFIG_SUBDIR);
 	mkdir(config_dir.c_str(),0700); // create configuration directory if it doesn't exist
 
-	url_file = config_dir + std::string(NEWSBEUTER_PATH_SEP) + url_file;
-	cache_file = config_dir + std::string(NEWSBEUTER_PATH_SEP) + cache_file;
-	config_file = config_dir + std::string(NEWSBEUTER_PATH_SEP) + config_file;
-	lock_file = config_dir + std::string(NEWSBEUTER_PATH_SEP) + lock_file;
-	queue_file = config_dir + std::string(NEWSBEUTER_PATH_SEP) + queue_file;
 	reload_mutex = new mutex();
 }
 
@@ -89,6 +83,7 @@ controller::~controller() {
 	delete rsscache;
 	delete reload_mutex;
 	delete cfg;
+	delete urlcfg;
 }
 
 void controller::set_view(view * vv) {
@@ -97,7 +92,12 @@ void controller::set_view(view * vv) {
 
 void controller::run(int argc, char * argv[]) {
 	int c;
-	char msgbuf[1024];
+
+	url_file = config_dir + std::string(NEWSBEUTER_PATH_SEP) + url_file;
+	cache_file = config_dir + std::string(NEWSBEUTER_PATH_SEP) + cache_file;
+	lock_file = cache_file + LOCK_SUFFIX;
+	config_file = config_dir + std::string(NEWSBEUTER_PATH_SEP) + config_file;
+	queue_file = config_dir + std::string(NEWSBEUTER_PATH_SEP) + queue_file;
 
 	::signal(SIGINT, ctrl_c_action);
 #ifndef DEBUG
@@ -111,8 +111,11 @@ void controller::run(int argc, char * argv[]) {
 	bool offline_mode = false, real_offline_mode = false;
 	std::string importfile;
 
+	bool silent = false;
+	bool execute_cmds = false;
+
 	do {
-		if((c = ::getopt(argc,argv,"i:erhu:c:C:d:l:vVo"))<0)
+		if((c = ::getopt(argc,argv,"i:erhu:c:C:d:l:vVox"))<0)
 			continue;
 		switch (c) {
 			case ':': /* fall-through */
@@ -123,6 +126,7 @@ void controller::run(int argc, char * argv[]) {
 				if (do_export)
 					usage(argv[0]);
 				do_import = true;
+				silent = true;
 				importfile = optarg;
 				break;
 			case 'r':
@@ -132,6 +136,7 @@ void controller::run(int argc, char * argv[]) {
 				if (do_import)
 					usage(argv[0]);
 				do_export = true;
+				silent = true;
 				break;
 			case 'h':
 				usage(argv[0]);
@@ -141,6 +146,7 @@ void controller::run(int argc, char * argv[]) {
 				break;
 			case 'c':
 				cache_file = optarg;
+				lock_file = std::string(cache_file) + LOCK_SUFFIX;
 				cachefile_given_on_cmdline = true;
 				break;
 			case 'C':
@@ -155,6 +161,10 @@ void controller::run(int argc, char * argv[]) {
 			case 'o':
 				offline_mode = true;
 				break;
+			case 'x':
+				execute_cmds = true;
+				silent = true;
+				break;
 			case 'd': // this is an undocumented debug commandline option!
 				GetLogger().set_logfile(optarg);
 				break;
@@ -166,8 +176,7 @@ void controller::run(int argc, char * argv[]) {
 				}
 				break;
 			default:
-				snprintf(msgbuf, sizeof(msgbuf), _("%s: unknown option - %c"), argv[0], static_cast<char>(c));
-				std::cout << msgbuf << std::endl;
+				std::cout << utils::strprintf(_("%s: unknown option - %c"), argv[0], static_cast<char>(c)) << std::endl;
 				usage(argv[0]);
 				break;
 		}
@@ -181,10 +190,12 @@ void controller::run(int argc, char * argv[]) {
 		return;
 	}
 
+	GetLogger().log(LOG_INFO, "nl_langinfo(CODESET): %s", nl_langinfo(CODESET));
 
 	if (!do_export) {
-		snprintf(msgbuf, sizeof(msgbuf), _("Starting %s %s..."), PROGRAM_NAME, PROGRAM_VERSION);
-		std::cout << msgbuf << std::endl;
+
+		if (!silent)
+			std::cout << utils::strprintf(_("Starting %s %s..."), PROGRAM_NAME, PROGRAM_VERSION) << std::endl;
 
 		pid_t pid;
 		if (!utils::try_fs_lock(lock_file, pid)) {
@@ -193,13 +204,12 @@ void controller::run(int argc, char * argv[]) {
 			} else {
 				GetLogger().log(LOG_ERROR,"something went wrong with the lock: %s", strerror(errno));
 			}
-			snprintf(msgbuf, sizeof(msgbuf), _("Error: an instance of %s is already running (PID: %u)"), PROGRAM_NAME, pid);
-			std::cout << msgbuf << std::endl;
+			std::cout << utils::strprintf(_("Error: an instance of %s is already running (PID: %u)"), PROGRAM_NAME, pid) << std::endl;
 			return;
 		}
 	}
-	
-	if (!do_export)
+
+	if (!silent)
 		std::cout << _("Loading configuration...");
 	std::cout.flush();
 	
@@ -212,6 +222,7 @@ void controller::run(int argc, char * argv[]) {
 	keymap keys(KM_NEWSBEUTER);
 	cfgparser.register_handler("bind-key",&keys);
 	cfgparser.register_handler("unbind-key",&keys);
+	cfgparser.register_handler("macro", &keys);
 
 	cfgparser.register_handler("ignore-article",&ign);
 
@@ -227,6 +238,7 @@ void controller::run(int argc, char * argv[]) {
 		return;	
 	}
 
+
 	if (colorman->colors_loaded()) {
 		v->set_colors(*colorman);
 	}
@@ -236,25 +248,43 @@ void controller::run(int argc, char * argv[]) {
 		GetLogger().set_errorlogfile(cfg->get_configvalue("error-log").c_str());
 	}
 
-	if (!do_export)
+	if (!silent)
 		std::cout << _("done.") << std::endl;
 
 	// create cache object
 	std::string cachefilepath = cfg->get_configvalue("cache-file");
 	if (cachefilepath.length() > 0 && !cachefile_given_on_cmdline) {
 		cache_file = cachefilepath.c_str();
+
+		// ok, we got another cache file path via the configuration
+		// that means we need to remove the old lock file, assemble
+		// the new lock file's name, and then try to lock it.
+		utils::remove_fs_lock(lock_file);
+		lock_file = std::string(cache_file) + LOCK_SUFFIX;
+
+		pid_t pid;
+		if (!utils::try_fs_lock(lock_file, pid)) {
+			if (pid > 0) {
+				GetLogger().log(LOG_ERROR,"an instance is already running: pid = %u",pid);
+			} else {
+				GetLogger().log(LOG_ERROR,"something went wrong with the lock: %s", strerror(errno));
+			}
+			std::cout << utils::strprintf(_("Error: an instance of %s is already running (PID: %u)"), PROGRAM_NAME, pid) << std::endl;
+			return;
+		}
 	}
 
-	if (!do_export) {
+	if (!silent) {
 		std::cout << _("Opening cache...");
 		std::cout.flush();
 	}
 	rsscache = new cache(cache_file,cfg);
-	if (!do_export) {
+	if (!silent) {
 		std::cout << _("done.") << std::endl;
 	}
 
 
+
 	std::string type = cfg->get_configvalue("urls-source");
 	if (type == "local") {
 		urlcfg = new file_urlreader(url_file);
@@ -270,8 +300,7 @@ void controller::run(int argc, char * argv[]) {
 
 	if (real_offline_mode) {
 		if (!do_export) {
-			snprintf(msgbuf,sizeof(msgbuf), _("Loading URLs from local cache..."));
-			std::cout << msgbuf;
+			std::cout << _("Loading URLs from local cache...");
 			std::cout.flush();
 		}
 		urlcfg->set_offline(true);
@@ -280,33 +309,33 @@ void controller::run(int argc, char * argv[]) {
 			std::cout << _("done.") << std::endl;
 		}
 	} else {
-		if (!do_export) {
-			snprintf(msgbuf,sizeof(msgbuf), _("Loading URLs from %s..."), urlcfg->get_source().c_str());
-			std::cout << msgbuf;
+		if (!do_export && !silent) {
+			std::cout << utils::strprintf(_("Loading URLs from %s..."), urlcfg->get_source().c_str());
 			std::cout.flush();
 		}
 		urlcfg->reload();
-		if (!do_export) {
+		if (!do_export && !silent) {
 			std::cout << _("done.") << std::endl;
 		}
 	}
 
 	if (urlcfg->get_urls().size() == 0) {
 		GetLogger().log(LOG_ERROR,"no URLs configured.");
+		std::string msg;
 		if (type == "local") {
-			snprintf(msgbuf, sizeof(msgbuf), _("Error: no URLs configured. Please fill the file %s with RSS feed URLs or import an OPML file."), url_file.c_str());
+			msg = utils::strprintf(_("Error: no URLs configured. Please fill the file %s with RSS feed URLs or import an OPML file."), url_file.c_str());
 		} else if (type == "bloglines") {
-			snprintf(msgbuf, sizeof(msgbuf), _("It looks like you haven't configured any feeds in your bloglines account. Please do so, and try again."));
+			msg = utils::strprintf(_("It looks like you haven't configured any feeds in your bloglines account. Please do so, and try again."));
 		} else if (type == "opml") {
-			snprintf(msgbuf, sizeof(msgbuf), _("It looks like the OPML feed you subscribed contains no feeds. Please fill it with feeds, and try again."));
+			msg = utils::strprintf(_("It looks like the OPML feed you subscribed contains no feeds. Please fill it with feeds, and try again."));
 		} else {
 			assert(0); // shouldn't happen
 		}
-		std::cout << msgbuf << std::endl << std::endl;
+		std::cout << msg << std::endl << std::endl;
 		usage(argv[0]);
 	}
 
-	if (!do_export && !do_vacuum)
+	if (!do_export && !do_vacuum && !silent)
 		std::cout << _("Loading articles from cache...");
 	if (do_vacuum)
 		std::cout << _("Opening cache...");
@@ -339,7 +368,7 @@ void controller::run(int argc, char * argv[]) {
 
 	std::vector<std::string> tags = urlcfg->get_alltags();
 
-	if (!do_export)
+	if (!do_export && !silent)
 		std::cout << _("done.") << std::endl;
 
 	if (do_export) {
@@ -348,6 +377,12 @@ void controller::run(int argc, char * argv[]) {
 		return;
 	}
 
+	if (execute_cmds) {
+		execute_commands(argv, optind);
+		utils::remove_fs_lock(lock_file);
+		return;	
+	}
+
 	// if the user wants to refresh on startup via configuration file, then do so,
 	// but only if -r hasn't been supplied.
 	if (!refresh_on_start && cfg->get_configvalue_as_bool("refresh-on-startup")) {
@@ -357,7 +392,6 @@ void controller::run(int argc, char * argv[]) {
 	// hand over the important objects to the view
 	v->set_config_container(cfg);
 	v->set_keymap(&keys);
-	v->set_feedlist(feeds);
 	v->set_tags(tags);
 
 	// run the view
@@ -379,13 +413,15 @@ void controller::update_feedlist() {
 	v->set_feedlist(feeds);
 }
 
+void controller::update_visible_feeds() {
+	v->update_visible_feeds(feeds);
+}
+
 void controller::catchup_all() {
 	try {
 		rsscache->catchup_all();
 	} catch (const dbexception& e) {
-		char buf[1024];
-		snprintf(buf, sizeof(buf), _("Error: couldn't mark all feeds read: %s"), e.what());
-		v->show_error(buf);
+		v->show_error(utils::strprintf(_("Error: couldn't mark all feeds read: %s"), e.what()));
 		return;
 	}
 	for (std::vector<rss_feed>::iterator it=feeds.begin();it!=feeds.end();++it) {
@@ -413,71 +449,62 @@ void controller::mark_all_read(unsigned int pos) {
 	}
 }
 
-void controller::reload(unsigned int pos, unsigned int max) {
+void controller::reload(unsigned int pos, unsigned int max, bool unattended) {
 	GetLogger().log(LOG_DEBUG, "controller::reload: pos = %u max = %u", pos, max);
-	char msgbuf[1024];
 	if (pos < feeds.size()) {
 		rss_feed feed = feeds[pos];
 		std::string msg;
 		if (max > 0) {
-			msg.append("(");
-			std::ostringstream posstr;
-			posstr << (pos+1);
-			msg.append(posstr.str());
-			msg.append("/");
-			std::ostringstream maxstr;
-			maxstr << max;
-			msg.append(maxstr.str());
-			msg.append(") ");
+			msg = utils::strprintf("(%u/%u) ", pos+1, max);
 		}
-		snprintf(msgbuf, sizeof(msgbuf), _("%sLoading %s..."), msg.c_str(), feed.rssurl().c_str());
 		GetLogger().log(LOG_DEBUG, "controller::reload: before setting status");
-		v->set_status(msgbuf);
+		if (!unattended)
+			v->set_status(utils::strprintf(_("%sLoading %s..."), msg.c_str(), feed.rssurl().c_str()));
 		GetLogger().log(LOG_DEBUG, "controller::reload: after setting status");
 				
 		rss_parser parser(feed.rssurl().c_str(), rsscache, cfg, &ign);
 		GetLogger().log(LOG_DEBUG, "controller::reload: created parser");
 		try {
-			feed = parser.parse();
-			GetLogger().log(LOG_DEBUG, "controller::reload: after parser.parse");
-			if (!feed.is_empty()) {
-				GetLogger().log(LOG_DEBUG, "controller::reload: feed is nonempty, saving");
-				rsscache->externalize_rssfeed(feed);
-				GetLogger().log(LOG_DEBUG, "controller::reload: after externalize_rssfeed");
+			if (parser.check_and_update_lastmodified()) {
+				feed = parser.parse();
+				GetLogger().log(LOG_DEBUG, "controller::reload: after parser.parse");
+				if (!feed.is_empty()) {
+					GetLogger().log(LOG_DEBUG, "controller::reload: feed is nonempty, saving");
+					rsscache->externalize_rssfeed(feed);
+					GetLogger().log(LOG_DEBUG, "controller::reload: after externalize_rssfeed");
 
-				rsscache->internalize_rssfeed(feed);
-				GetLogger().log(LOG_DEBUG, "controller::reload: after internalize_rssfeed");
-				feed.set_tags(urlcfg->get_tags(feed.rssurl()));
-				feeds[pos] = feed;
-				v->notify_itemlist_change(feed);
-			} else {
-				GetLogger().log(LOG_DEBUG, "controller::reload: feed is empty, not saving");
-			}
+					rsscache->internalize_rssfeed(feed);
+					GetLogger().log(LOG_DEBUG, "controller::reload: after internalize_rssfeed");
+					feed.set_tags(urlcfg->get_tags(feed.rssurl()));
+					feeds[pos] = feed;
+					v->notify_itemlist_change(feed);
+				} else {
+					GetLogger().log(LOG_DEBUG, "controller::reload: feed is empty, not saving");
+				}
 
-			for (std::vector<rss_item>::iterator it=feed.items().begin();it!=feed.items().end();++it) {
-				if (cfg->get_configvalue_as_bool("podcast-auto-enqueue") && !it->enqueued() && it->enclosure_url().length() > 0) {
-					GetLogger().log(LOG_DEBUG, "controller::reload: enclosure_url = `%s' enclosure_type = `%s'", it->enclosure_url().c_str(), it->enclosure_type().c_str());
-					if (is_valid_podcast_type(it->enclosure_type())) {
-						GetLogger().log(LOG_INFO, "controller::reload: enqueuing `%s'", it->enclosure_url().c_str());
-						enqueue_url(it->enclosure_url());
-						it->set_enqueued(true);
-						rsscache->update_rssitem_unread_and_enqueued(*it, feed.rssurl());
+				for (std::vector<rss_item>::iterator it=feed.items().begin();it!=feed.items().end();++it) {
+					if (cfg->get_configvalue_as_bool("podcast-auto-enqueue") && !it->enqueued() && it->enclosure_url().length() > 0) {
+						GetLogger().log(LOG_DEBUG, "controller::reload: enclosure_url = `%s' enclosure_type = `%s'", it->enclosure_url().c_str(), it->enclosure_type().c_str());
+						if (is_valid_podcast_type(it->enclosure_type())) {
+							GetLogger().log(LOG_INFO, "controller::reload: enqueuing `%s'", it->enclosure_url().c_str());
+							enqueue_url(it->enclosure_url());
+							it->set_enqueued(true);
+							rsscache->update_rssitem_unread_and_enqueued(*it, feed.rssurl());
+						}
 					}
 				}
-			}
 
-			
-			v->set_feedlist(feeds);
-			GetLogger().log(LOG_DEBUG, "controller::reload: after set_feedlist");
+				
+				if (!unattended) {
+					v->set_feedlist(feeds);
+					GetLogger().log(LOG_DEBUG, "controller::reload: after set_feedlist");
+				}
+			}
 			v->set_status("");
 		} catch (const dbexception& e) {
-			char buf[1024];
-			snprintf(buf, sizeof(buf), _("Error while retrieving %s: %s"), feed.rssurl().c_str(), e.what());
-			v->set_status(buf);
+			v->set_status(utils::strprintf(_("Error while retrieving %s: %s"), feed.rssurl().c_str(), e.what()));
 		} catch (const std::string& errmsg) {
-			char buf[1024];
-			snprintf(buf, sizeof(buf), _("Error while retrieving %s: %s"), feed.rssurl().c_str(), errmsg.c_str());
-			v->set_status(buf);
+			v->set_status(utils::strprintf(_("Error while retrieving %s: %s"), feed.rssurl().c_str(), errmsg.c_str()));
 		}
 	} else {
 		v->show_error(_("Error: invalid feed!"));
@@ -491,7 +518,30 @@ rss_feed * controller::get_feed(unsigned int pos) {
 	return &(feeds[pos]);
 }
 
-void controller::reload_all() {
+void controller::reload_indexes(const std::vector<int>& indexes, bool unattended) {
+	scope_measure m1("controller::reload_indexes");
+	unsigned int unread_feeds, unread_articles;
+	compute_unread_numbers(unread_feeds, unread_articles);
+
+	for (std::vector<int>::const_iterator it=indexes.begin();it!=indexes.end();++it) {
+		this->reload(*it,feeds.size(), unattended);
+	}
+
+	unsigned int unread_feeds2, unread_articles2;
+	compute_unread_numbers(unread_feeds2, unread_articles2);
+	if (unread_feeds2 != unread_feeds || unread_articles2 != unread_articles) {
+		fmtstr_formatter fmt;
+		fmt.register_fmt('f', utils::to_s(unread_feeds2));
+		fmt.register_fmt('n', utils::to_s(unread_articles2));
+		fmt.register_fmt('d', utils::to_s(unread_articles2 - unread_articles));
+		fmt.register_fmt('D', utils::to_s(unread_feeds2 - unread_feeds));
+		this->notify(fmt.do_format(cfg->get_configvalue("notify-format")));
+	}
+	if (!unattended)
+		v->set_status("");
+}
+
+void controller::reload_all(bool unattended) {
 	GetLogger().log(LOG_DEBUG,"controller::reload_all: starting with reload all...");
 	unsigned int unread_feeds, unread_articles;
 	compute_unread_numbers(unread_feeds, unread_articles);
@@ -499,7 +549,7 @@ void controller::reload_all() {
 	t1 = time(NULL);
 	for (unsigned int i=0;i<feeds.size();++i) {
 		GetLogger().log(LOG_DEBUG, "controller::reload_all: reloading feed #%u", i);
-		this->reload(i,feeds.size());
+		this->reload(i,feeds.size(), unattended);
 	}
 	t2 = time(NULL);
 	dt = t2 - t1;
@@ -508,9 +558,12 @@ void controller::reload_all() {
 	unsigned int unread_feeds2, unread_articles2;
 	compute_unread_numbers(unread_feeds2, unread_articles2);
 	if (unread_feeds2 != unread_feeds || unread_articles2 != unread_articles) {
-		char buf[2048];
-		snprintf(buf,sizeof(buf),_("newsbeuter: finished reload, %u unread feeds (%u unread articles total)"), unread_feeds2, unread_articles2);
-		this->notify(buf);
+		fmtstr_formatter fmt;
+		fmt.register_fmt('f', utils::to_s(unread_feeds2));
+		fmt.register_fmt('n', utils::to_s(unread_articles2));
+		fmt.register_fmt('d', utils::to_s(unread_articles2 - unread_articles));
+		fmt.register_fmt('D', utils::to_s(unread_feeds2 - unread_feeds));
+		this->notify(fmt.do_format(cfg->get_configvalue("notify-format")));
 	}
 }
 
@@ -530,19 +583,6 @@ void controller::notify(const std::string& msg) {
 		GetLogger().log(LOG_DEBUG, "controller:notify: notifying external program `%s'", prog.c_str());
 		utils::run_command(prog, msg);
 	}
-	/* // TODO: implement Growl support
-	if (cfg->get_configvalue_as_bool("notify-growl")) {
-		std::vector<std::string> tokens = utils::tokenize(cfg->get_configvalue("growl-config"), ":");
-		if (tokens.size() >= 1) {
-			std::string hostname = tokens[0];
-			std::string password;
-			if (tokens.size() >= 2) {
-				password = tokens[1];
-			}
-			growlnotifier->send_notify(hostname, password, "newsbeuter", msg);
-		}
-	}
-	*/
 }
 
 void controller::compute_unread_numbers(unsigned int& unread_feeds, unsigned int& unread_articles) {
@@ -566,15 +606,15 @@ bool controller::trylock_reload_mutex() {
 	return false;
 }
 
-void controller::start_reload_all_thread() {
+void controller::start_reload_all_thread(std::vector<int> * indexes) {
 	GetLogger().log(LOG_INFO,"starting reload all thread");
-	thread * dlt = new downloadthread(this);
+	thread * dlt = new downloadthread(this, indexes);
 	dlt->start();
 }
 
 void controller::version_information() {
 	std::cout << PROGRAM_NAME << " " << PROGRAM_VERSION << " - " << PROGRAM_URL << std::endl;
-	std::cout << "Copyright (C) 2006-2007 Andreas Krennmair" << std::endl << std::endl;
+	std::cout << "Copyright (C) 2006-2008 Andreas Krennmair" << std::endl << std::endl;
 
 	struct utsname xuts;
 	uname(&xuts);
@@ -591,9 +631,7 @@ void controller::version_information() {
 }
 
 void controller::usage(char * argv0) {
-	char buf[2048];
-	snprintf(buf, sizeof(buf), 
-				_("%s %s\nusage: %s [-i <file>|-e] [-u <urlfile>] [-c <cachefile>] [-h]\n"
+	std::cout << utils::strprintf(_("%s %s\nusage: %s [-i <file>|-e] [-u <urlfile>] [-c <cachefile>] [-x <command> ...] [-h]\n"
 				"-e              export OPML feed to stdout\n"
 				"-r              refresh feeds on start\n"
 				"-i <file>       import OPML file\n"
@@ -601,10 +639,10 @@ void controller::usage(char * argv0) {
 				"-c <cachefile>  use <cachefile> as cache file\n"
 				"-C <configfile> read configuration from <configfile>\n"
 				"-v              clean up cache thoroughly\n"
+				"-x <command>... execute list of commands\n"
 				"-o              activate offline mode (only applies to bloglines synchronization mode)\n"
 				"-V              get version information\n"
 				"-h              this help\n"), PROGRAM_NAME, PROGRAM_VERSION, argv0);
-	std::cout << buf;
 	::exit(EXIT_FAILURE);
 }
 
@@ -637,9 +675,7 @@ void controller::import_opml(const char * filename) {
 	}
 
 	nxml_free(data);
-	char buf[1024];
-	snprintf(buf, sizeof(buf), _("Import of %s finished."), filename);
-	std::cout << buf << std::endl;
+	std::cout << utils::strprintf(_("Import of %s finished."), filename) << std::endl;
 }
 
 void controller::export_opml() {
@@ -665,7 +701,7 @@ void controller::export_opml() {
 
 void controller::rec_find_rss_outlines(nxml_data_t * node, std::string tag) {
 	while (node) {
-		char * url = nxmle_find_attribute(node, "xmlUrl", NULL);
+		const char * url = nxmle_find_attribute(node, "xmlUrl", NULL);
 		std::string newtag = tag;
 
 		if (!url) {
@@ -699,7 +735,7 @@ void controller::rec_find_rss_outlines(nxml_data_t * node, std::string tag) {
 					GetLogger().log(LOG_DEBUG,"OPML import: url = %s is already in list",url);
 				}
 			} else {
-				char * text = nxmle_find_attribute(node, "text", NULL);
+				const char * text = nxmle_find_attribute(node, "text", NULL);
 				if (!text)
 					text = nxmle_find_attribute(node, "title", NULL);
 				if (text) {
@@ -737,7 +773,6 @@ rss_feed * controller::get_feed_by_url(const std::string& feedurl) {
 
 bool controller::is_valid_podcast_type(const std::string& /* mimetype */) {
 	return true;
-	// return mimetype == "audio/mpeg" || mimetype == "video/x-m4v" || mimetype == "audio/x-mpeg";
 }
 
 void controller::enqueue_url(const std::string& url) {
@@ -773,6 +808,7 @@ void controller::reload_urls_file() {
 		for (std::vector<rss_feed>::iterator jt=feeds.begin();jt!=feeds.end();++jt) {
 			if (*it == jt->rssurl()) {
 				found = true;
+				jt->set_tags(urlcfg->get_tags(*it));
 				new_feeds.push_back(*jt);
 				break;
 			}
@@ -805,8 +841,8 @@ std::string controller::bookmark(const std::string& url, const std::string& titl
 	std::string bookmark_cmd = cfg->get_configvalue("bookmark-cmd");
 	if (bookmark_cmd.length() > 0) {
 		char * my_argv[4];
-		my_argv[0] = "/bin/sh";
-		my_argv[1] = "-c";
+		my_argv[0] = const_cast<char *>("/bin/sh");
+		my_argv[1] = const_cast<char *>("-c");
 
 		// wow. what an abuse.
 		std::string cmdline = bookmark_cmd + " " + stfl::quote(url) + " " + stfl::quote(title) + " " + stfl::quote(description);
@@ -820,3 +856,57 @@ std::string controller::bookmark(const std::string& url, const std::string& titl
 	}
 }
 
+void controller::execute_commands(char ** argv, unsigned int i) {
+	v->pop_current_formaction();
+	for (;argv[i];++i) {
+		GetLogger().log(LOG_DEBUG, "controller::execute_commands: executing `%s'", argv[i]);
+		std::string cmd(argv[i]);
+		if (cmd == "reload") {
+			reload_all(true);
+		} else if (cmd == "print-unread") {
+			std::cout << utils::strprintf(_("%u unread articles"), rsscache->get_unread_count()) << std::endl;
+		}
+	}
+}
+
+void controller::write_item(const rss_item& item, const std::string& filename) {
+	std::vector<std::string> lines;
+	std::vector<linkpair> links; // not used
+	
+	std::string title(_("Title: "));
+	title.append(item.title());
+	lines.push_back(title);
+	
+	std::string author(_("Author: "));
+	author.append(item.author());
+	lines.push_back(author);
+	
+	std::string date(_("Date: "));
+	date.append(item.pubDate());
+	lines.push_back(date);
+
+	std::string link(_("Link: "));
+	link.append(item.link());
+	lines.push_back(link);
+	
+	lines.push_back(std::string(""));
+	
+	unsigned int width = cfg->get_configvalue_as_int("text-width");
+	if (width == 0)
+		width = 80;
+	htmlrenderer rnd(width);
+	rnd.render(item.description(), lines, links, item.feedurl());
+
+	std::fstream f;
+	f.open(filename.c_str(),std::fstream::out);
+	if (!f.is_open())
+		throw exception(errno);
+		
+	for (std::vector<std::string>::iterator it=lines.begin();it!=lines.end();++it) {
+		f << *it << std::endl;	
+	}
+}
+
+void controller::mark_deleted(const std::string& guid, bool b) {
+	rsscache->mark_item_deleted(guid, b);
+}
