@@ -4,7 +4,9 @@
 namespace newsbeuter
 {
 
-downloadthread::downloadthread(controller * c) : ctrl(c) {
+downloadthread::downloadthread(controller * c, std::vector<int> * idxs) : ctrl(c) {
+	if (idxs)
+		indexes = *idxs;
 }
 
 downloadthread::~downloadthread() {
@@ -18,7 +20,11 @@ void downloadthread::run() {
 	 */
 	GetLogger().log(LOG_DEBUG, "downloadthread::run: inside downloadthread, reloading all feeds...");
 	if (ctrl->trylock_reload_mutex()) {
-		ctrl->reload_all();
+		if (indexes.size() == 0) {
+			ctrl->reload_all();
+		} else {
+			ctrl->reload_indexes(indexes);
+		}
 		ctrl->unlock_reload_mutex();
 	}
 	this->detach();
