@@ -3,6 +3,7 @@
 #include <config.h>
 #include <cerrno>
 #include <cstring>
+#include <utils.h>
 
 using namespace newsbeuter;
 
@@ -11,18 +12,18 @@ exception::exception(unsigned int error_code) : ecode(error_code) { }
 exception::~exception() throw() { }
 
 const char * exception::what() const throw() {
-	return std::strerror(ecode);
+	return ::strerror(ecode);
 }
 
 
 const char * matcherexception::what() const throw() {
-	static char errmsgbuf[2048];
+	static std::string errmsg;
 	switch (type) {
 		case ATTRIB_UNAVAIL:
-			snprintf(errmsgbuf, sizeof(errmsgbuf), _("attribute `%s' is not available."), addinfo.c_str());
+			errmsg = utils::strprintf(_("attribute `%s' is not available."), addinfo.c_str());
 			break;
 		default:
-			strcpy(errmsgbuf,"");
+			errmsg = "";
 	}
-	return errmsgbuf;
+	return errmsg.c_str();
 }
