@@ -6,37 +6,45 @@
 #include <reloadthread.h>
 #include <exceptions.h>
 #include <utils.h>
+#include <formatstring.h>
+
+#include <listformatter.h>
 
 #include <sstream>
-#include <cassert>
 #include <string>
 
+#include <langinfo.h>
+
 #define FILTER_UNREAD_FEEDS "unread_count != \"0\""
 
 namespace newsbeuter {
 
 feedlist_formaction::feedlist_formaction(view * vv, std::string formstr) 
 	: formaction(vv,formstr), zero_feedpos(false), feeds_shown(0),
-		auto_open(false), quit(false), apply_filter(false), search_dummy_feed(v->get_ctrl()->get_cache()) {
+		auto_open(false), quit(false), apply_filter(false), search_dummy_feed(v->get_ctrl()->get_cache()),
+		filterpos(0), set_filterpos(false) {
 	assert(true==m.parse(FILTER_UNREAD_FEEDS));
 }
 
 void feedlist_formaction::init() {
 	set_keymap_hints();
 
+	f->run(-3); // compute all widget dimensions
+
 	if(v->get_ctrl()->get_refresh_on_start()) {
-		f->run(-1);
+		f->run(-1); // FRUN
 		v->get_ctrl()->start_reload_all_thread();
 	}
+	v->get_ctrl()->update_feedlist();
 
 	/*
 	 * This is kind of a hack.
 	 * The feedlist_formaction is responsible for starting up the reloadthread, which is responsible
 	 * for regularly spawning downloadthreads.
 	 */
-	unsigned int reload_cycle = 60 * static_cast<unsigned int>(v->get_cfg()->get_configvalue_as_int("reload-time"));
+	time_t reload_cycle = 60 * v->get_cfg()->get_configvalue_as_int("reload-time");
 	if (v->get_cfg()->get_configvalue_as_bool("auto-reload") == true) {
-		f->run(-1);
+		f->run(-1); // FRUN
 		reloadthread  * rt = new reloadthread(v->get_ctrl(), reload_cycle, v->get_cfg());
 		rt->start();
 	}
@@ -47,17 +55,41 @@ void feedlist_formaction::init() {
 feedlist_formaction::~feedlist_formaction() { }
 
 void feedlist_formaction::prepare() {
+	static unsigned int old_width = 0;
+
+	std::string listwidth = f->get("items:w");
+	std::istringstream is(listwidth);
+	unsigned int width = 0;
+	is >> width;
+
+	if (old_width != width) {
+		do_redraw = true;
+		old_width = width;
+		GetLogger().log(LOG_DEBUG, "feedlist_formaction::prepare: apparent resize");
+	}
+
 	if (do_redraw) {
+		GetLogger().log(LOG_DEBUG, "feedlist_formaction::prepare: doing redraw");
 		do_redraw = false;
 		v->get_ctrl()->update_feedlist();
-		if (zero_feedpos) {
+		if (set_filterpos) {
+			set_filterpos = false;
+			unsigned int i = 0;
+			for (std::vector<feedptr_pos_pair>::iterator it=visible_feeds.begin();it!=visible_feeds.end();++it, ++i) {
+				if (it->second == filterpos) {
+					f->set("feedpos", utils::to_s(i));
+					return;
+				}
+			}
+			f->set("feedpos", "0");
+		} else if (zero_feedpos) {
 			f->set("feedpos","0");
 			zero_feedpos = false;
 		}
 	}
 }
 
-void feedlist_formaction::process_operation(operation op) {
+void feedlist_formaction::process_operation(operation op, bool automatic, std::vector<std::string> * args) {
 	std::string feedpos = f->get("feedposname");
 	std::istringstream posname(feedpos);
 	unsigned int pos = 0;
@@ -65,6 +97,10 @@ void feedlist_formaction::process_operation(operation op) {
 	switch (op) {
 		case OP_OPEN: {
 				if (f->get_focus() == "feeds") {
+					if (automatic && args->size() > 0) {
+						std::istringstream x((*args)[0]);
+						x >> pos;
+					}
 					GetLogger().log(LOG_INFO, "feedlist_formaction: opening feed at position `%s'",feedpos.c_str());
 					if (feeds_shown > 0 && feedpos.length() > 0) {
 						v->push_itemlist(pos);
@@ -83,12 +119,29 @@ void feedlist_formaction::process_operation(operation op) {
 				}
 			}
 			break;
+		case OP_INT_RESIZE:
+			do_redraw = true;
+			break;
 		case OP_RELOADURLS:
 			v->get_ctrl()->reload_urls_file();
 			break;
+		case OP_OPENINBROWSER: {
+				rss_feed * feed = v->get_ctrl()->get_feed(pos);
+				if (feed) {
+					v->open_in_browser(feed->link());
+				}
+			}
+			break;
 		case OP_RELOADALL:
 			GetLogger().log(LOG_INFO, "feedlist_formaction: reloading all feeds");
-			v->get_ctrl()->start_reload_all_thread();
+			{
+				bool reload_only_visible_feeds = v->get_cfg()->get_configvalue_as_bool("reload-only-visible-feeds");
+				std::vector<int> idxs;
+				for (std::vector<feedptr_pos_pair>::iterator it=visible_feeds.begin();it!=visible_feeds.end();++it) {
+					idxs.push_back(it->second);
+				}
+				v->get_ctrl()->start_reload_all_thread(reload_only_visible_feeds ? &idxs : NULL);
+			}
 			break;
 		case OP_MARKFEEDREAD: {
 				GetLogger().log(LOG_INFO, "feedlist_formaction: marking feed read at position `%s'",feedpos.c_str());
@@ -99,9 +152,7 @@ void feedlist_formaction::process_operation(operation op) {
 						do_redraw = true;
 						v->set_status("");
 					} catch (const dbexception& e) {
-						char buf[1024];
-						snprintf(buf, sizeof(buf), _("Error: couldn't mark feed read: %s"), e.what());
-						v->show_error(buf);
+						v->show_error(utils::strprintf(_("Error: couldn't mark feed read: %s"), e.what()));
 					}
 				} else {
 					v->show_error(_("No feed selected!")); // should not happen
@@ -118,6 +169,7 @@ void feedlist_formaction::process_operation(operation op) {
 				v->get_cfg()->set_configvalue("show-read-feeds","yes");
 				apply_filter = false;
 			}
+			save_filterpos();
 			do_redraw = true;
 			break;
 		case OP_NEXTUNREAD: {
@@ -150,7 +202,12 @@ void feedlist_formaction::process_operation(operation op) {
 			break;
 		case OP_SETTAG: 
 			if (tags.size() > 0) {
-				std::string newtag = v->select_tag(tags);
+				std::string newtag;
+				if (automatic && args->size() > 0) {
+					newtag = (*args)[0];
+				} else {
+					newtag = v->select_tag(tags);
+				}
 				if (newtag != "") {
 					tag = newtag;
 					do_redraw = true;
@@ -162,7 +219,12 @@ void feedlist_formaction::process_operation(operation op) {
 			break;
 		case OP_SELECTFILTER:
 			if (v->get_ctrl()->get_filters().size() > 0) {
-				std::string newfilter = v->select_filter(v->get_ctrl()->get_filters().get_filters());
+				std::string newfilter;
+				if (automatic && args->size() > 0) {
+					newfilter = (*args)[0];
+				} else {
+					newfilter = v->select_filter(v->get_ctrl()->get_filters().get_filters());
+				}
 				if (newfilter != "") {
 					filterhistory.add_line(newfilter);
 					if (newfilter.length() > 0) {
@@ -170,6 +232,7 @@ void feedlist_formaction::process_operation(operation op) {
 							v->show_error(_("Error: couldn't parse filter command!"));
 							m.parse(FILTER_UNREAD_FEEDS);
 						} else {
+							save_filterpos();
 							apply_filter = true;
 							do_redraw = true;
 						}
@@ -179,9 +242,17 @@ void feedlist_formaction::process_operation(operation op) {
 				v->show_error(_("No filters defined."));
 			}
 			break;
-		case OP_SEARCH: {
-				std::vector<std::pair<std::string, std::string> > qna;
-				qna.push_back(std::pair<std::string, std::string>(_("Search for: "), ""));
+		case OP_SEARCH:
+			if (automatic && args->size() > 0) {
+				qna_responses.erase(qna_responses.begin(), qna_responses.end());
+				// when in automatic mode, we manually fill the qna_responses vector from the arguments
+				// and then run the finished_qna() by ourselves to simulate a "Q&A" session that is
+				// in fact macro-driven.
+				qna_responses.push_back((*args)[0]);
+				finished_qna(OP_INT_START_SEARCH);
+			} else {
+				std::vector<qna_pair> qna;
+				qna.push_back(qna_pair(_("Search for: "), ""));
 				this->start_qna(qna, OP_INT_START_SEARCH, &searchhistory);
 			}
 			break;
@@ -189,16 +260,22 @@ void feedlist_formaction::process_operation(operation op) {
 			apply_filter = !(v->get_cfg()->get_configvalue_as_bool("show-read-feeds"));
 			m.parse(FILTER_UNREAD_FEEDS);
 			do_redraw = true;
+			save_filterpos();
 			break;
-		case OP_SETFILTER: {
-				std::vector<std::pair<std::string, std::string> > qna;
-				qna.push_back(std::pair<std::string,std::string>(_("Filter: "), ""));
+		case OP_SETFILTER:
+			if (automatic && args->size() > 0) {
+				qna_responses.erase(qna_responses.begin(), qna_responses.end());
+				qna_responses.push_back((*args)[0]);
+				finished_qna(OP_INT_END_SETFILTER);
+			} else {
+				std::vector<qna_pair> qna;
+				qna.push_back(qna_pair(_("Filter: "), ""));
 				this->start_qna(qna, OP_INT_END_SETFILTER, &filterhistory);
 			}
 			break;
 		case OP_QUIT:
 			GetLogger().log(LOG_INFO, "feedlist_formaction: quitting");
-			if (!v->get_cfg()->get_configvalue_as_bool("confirm-exit") || v->confirm(_("Do you really want to quit (y:Yes n:No)? "), _("yn")) == *_("y")) {
+			if (automatic || !v->get_cfg()->get_configvalue_as_bool("confirm-exit") || v->confirm(_("Do you really want to quit (y:Yes n:No)? "), _("yn")) == *_("y")) {
 				quit = true;
 			}
 			break;
@@ -213,79 +290,85 @@ void feedlist_formaction::process_operation(operation op) {
 	}
 }
 
-void feedlist_formaction::set_feedlist(std::vector<rss_feed>& feeds) {
-	std::string code = "{list";
-	char buf[1024];
-	
+void feedlist_formaction::update_visible_feeds(std::vector<rss_feed>& feeds) {
 	assert(v->get_cfg() != NULL); // must not happen
-	
-	feeds_shown = 0;
-	unsigned int i = 0;
-	unsigned short feedlist_number = 1;
-	unsigned int unread_feeds = 0;
 
 	if (visible_feeds.size() > 0)
 		visible_feeds.erase(visible_feeds.begin(), visible_feeds.end());
 
-	for (std::vector<rss_feed>::iterator it = feeds.begin(); it != feeds.end(); ++it, ++i, ++feedlist_number) {
-		rss_feed feed = *it;
-		std::string title = it->title();
+	unsigned int i = 0;
+
+	for (std::vector<rss_feed>::iterator it = feeds.begin(); it != feeds.end(); ++it, ++i) {
+		if ((tag == "" || it->matches_tag(tag)) && (!apply_filter || m.matches(&(*it)))) {
+			visible_feeds.push_back(feedptr_pos_pair(&(*it),i));
+		}
+	}
+
+	feeds_shown = visible_feeds.size();
+}
+
+void feedlist_formaction::set_feedlist(std::vector<rss_feed>& feeds) {
+	assert(v->get_cfg() != NULL); // must not happen
+
+	std::string listwidth = f->get("feeds:w");
+	std::istringstream is(listwidth);
+	unsigned int width;
+	is >> width;
+
+	unsigned int i = 0;
+	unsigned short feedlist_number = 1;
+	unsigned int unread_feeds = 0;
+
+	std::string feedlist_format = v->get_cfg()->get_configvalue("feedlist-format");
+
+	listformatter listfmt;
+
+	update_visible_feeds(feeds);
+
+	for (std::vector<feedptr_pos_pair>::iterator it = visible_feeds.begin(); it != visible_feeds.end(); ++it, ++i, ++feedlist_number) {
+		std::string title = it->first->title();
 		if (title.length()==0) {
-			title = it->rssurl(); // rssurl must always be present.
+			title = it->first->rssurl(); // rssurl must always be present.
 			if (title.length()==0) {
 				title = "<no title>"; // shouldn't happen
 			}
 		}
 
-		// TODO: refactor
-		char sbuf[20];
-		char sbuf2[20];
 		unsigned int unread_count = 0;
-		if (it->items().size() > 0) {
-			unread_count = it->unread_item_count();
+		if (it->first->items().size() > 0) {
+			unread_count = it->first->unread_item_count();
 		}
 		if (unread_count > 0)
 			++unread_feeds;
 
-		/*
-		 * we only display an entry in the feedlist if:
-		 *   - no tag is active, or the entry matches the currently selected tag
-		 *   - no filter shall be applied, or the entry matches the currently set filter
-		 */
-		if ((tag == "" || it->matches_tag(tag)) && (!apply_filter || m.matches(&(*it)))) {
-			visible_feeds.push_back(std::pair<rss_feed *, unsigned int>(&(*it),i));
+		fmtstr_formatter fmt;
 
-			snprintf(sbuf,sizeof(buf),"(%u/%u) ",unread_count,static_cast<unsigned int>(it->items().size()));
-			snprintf(sbuf2,sizeof(sbuf2),"%4u %c %11s",feedlist_number, unread_count > 0 ? 'N' : ' ',sbuf);
-			std::string newtitle(sbuf2);
-			newtitle.append(title);
-			title = newtitle;
+		fmt.register_fmt('i', utils::strprintf("%u", it->second + 1));
+		fmt.register_fmt('u', utils::strprintf("(%u/%u)",unread_count,static_cast<unsigned int>(it->first->items().size())));
+		fmt.register_fmt('n', unread_count > 0 ? "N" : " ");
+		fmt.register_fmt('t', title);
+		fmt.register_fmt('l', it->first->link());
+		fmt.register_fmt('L', it->first->rssurl());
+		fmt.register_fmt('d', it->first->description());
 
-			std::string line = "{listitem[";
-			std::ostringstream num;
-			num << i;
-			line.append(num.str());
-			line.append("] text:");
-			line.append(stfl::quote(title));
-			line.append("}");
+		std::string format = fmt.do_format(feedlist_format, width);
+		GetLogger().log(LOG_DEBUG, "feedlist_formaction::set_feedlist: format result = %s", format.c_str());
 
-			code.append(line);
-
-			++feeds_shown;
-		}
+		listfmt.add_line(format, it->second);
 	}
 
-	code.append("}");
+	f->modify("feeds","replace_inner",listfmt.format_list());
 
-	f->modify("feeds","replace_inner",code);
+	std::string title_format = v->get_cfg()->get_configvalue("feedlist-title-format");
 
-	if (tag.length() > 0) {
-		snprintf(buf, sizeof(buf), _("%s %s - Your feeds (%u unread, %u total) - tag `%s'"), PROGRAM_NAME, PROGRAM_VERSION, unread_feeds, i, tag.c_str());
-	} else {
-		snprintf(buf, sizeof(buf), _("%s %s - Your feeds (%u unread, %u total)"), PROGRAM_NAME, PROGRAM_VERSION, unread_feeds, i);
-	}
+	fmtstr_formatter fmt;
+	fmt.register_fmt('T', tag);
+	fmt.register_fmt('N', PROGRAM_NAME);
+	fmt.register_fmt('V', PROGRAM_VERSION);
+	fmt.register_fmt('u', utils::to_s(unread_feeds));
+	fmt.register_fmt('t', utils::to_s(i));
 
-	f->set("head", buf);
+	f->set("head", fmt.do_format(title_format, width));
 }
 
 void feedlist_formaction::set_tags(const std::vector<std::string>& t) {
@@ -318,9 +401,7 @@ bool feedlist_formaction::jump_to_previous_unread_feed(unsigned int& feedpos) {
 		GetLogger().log(LOG_DEBUG, "feedlist_formaction::jump_to_previous_unread_feed: visible_feeds[%u] unread items: %u", i, visible_feeds[i].first->unread_item_count());
 		if (visible_feeds[i].first->unread_item_count() > 0) {
 			GetLogger().log(LOG_DEBUG, "feedlist_formaction::jump_to_previous_unread_feed: hit");
-			std::ostringstream os;
-			os << i;
-			f->set("feedpos", os.str());
+			f->set("feedpos", utils::to_s(i));
 			feedpos = visible_feeds[i].second;
 			return true;
 		}
@@ -329,15 +410,33 @@ bool feedlist_formaction::jump_to_previous_unread_feed(unsigned int& feedpos) {
 		GetLogger().log(LOG_DEBUG, "feedlist_formaction::jump_to_previous_unread_feed: visible_feeds[%u] unread items: %u", i, visible_feeds[i].first->unread_item_count());
 		if (visible_feeds[i].first->unread_item_count() > 0) {
 			GetLogger().log(LOG_DEBUG, "feedlist_formaction::jump_to_previous_unread_feed: hit");
-			std::ostringstream os;
-			os << i;
-			f->set("feedpos", os.str());
+			f->set("feedpos", utils::to_s(i));
+			feedpos = visible_feeds[i].second;
 			return true;
 		}
 	}
 	return false;
 }
 
+void feedlist_formaction::goto_feed(const std::string& str) {
+	unsigned int curpos;
+	std::istringstream is(f->get("feedpos"));
+	is >> curpos;
+	GetLogger().log(LOG_DEBUG, "feedlist_formaction::goto_feed: curpos = %u str = `%s'", curpos, str.c_str());
+	for (unsigned int i=curpos+1;i<visible_feeds.size();++i) {
+		if (strcasestr(visible_feeds[i].first->title().c_str(), str.c_str()) != NULL) {
+			f->set("feedpos", utils::to_s(i));
+			return;
+		}
+	}
+	for (unsigned int i=0;i<=curpos;++i) {
+		if (strcasestr(visible_feeds[i].first->title().c_str(), str.c_str()) != NULL) {
+			f->set("feedpos", utils::to_s(i));
+			return;
+		}
+	}
+}
+
 bool feedlist_formaction::jump_to_next_unread_feed(unsigned int& feedpos) {
 	unsigned int curpos;
 	std::istringstream is(f->get("feedpos"));
@@ -348,9 +447,7 @@ bool feedlist_formaction::jump_to_next_unread_feed(unsigned int& feedpos) {
 		GetLogger().log(LOG_DEBUG, "feedlist_formaction::jump_to_next_unread_feed: visible_feeds[%u] unread items: %u", i, visible_feeds[i].first->unread_item_count());
 		if (visible_feeds[i].first->unread_item_count() > 0) {
 			GetLogger().log(LOG_DEBUG, "feedlist_formaction::jump_to_next_unread_feed: hit");
-			std::ostringstream os;
-			os << i;
-			f->set("feedpos", os.str());
+			f->set("feedpos", utils::to_s(i));
 			feedpos = visible_feeds[i].second;
 			return true;
 		}
@@ -359,9 +456,8 @@ bool feedlist_formaction::jump_to_next_unread_feed(unsigned int& feedpos) {
 		GetLogger().log(LOG_DEBUG, "feedlist_formaction::jump_to_next_unread_feed: visible_feeds[%u] unread items: %u", i, visible_feeds[i].first->unread_item_count());
 		if (visible_feeds[i].first->unread_item_count() > 0) {
 			GetLogger().log(LOG_DEBUG, "feedlist_formaction::jump_to_next_unread_feed: hit");
-			std::ostringstream os;
-			os << i;
-			f->set("feedpos", os.str());
+			f->set("feedpos", utils::to_s(i));
+			feedpos = visible_feeds[i].second;
 			return true;
 		}
 	}
@@ -398,9 +494,7 @@ void feedlist_formaction::handle_cmdline(const std::string& cmd) {
 			if (i == -1) {
 				v->show_error(_("Position not visible!"));
 			} else {
-				std::ostringstream idxstr;
-				idxstr << i;
-				f->set("feedpos", idxstr.str());
+				f->set("feedpos", utils::to_s(i));
 			}
 		} else {
 			v->show_error(_("Invalid position!"));
@@ -415,6 +509,10 @@ void feedlist_formaction::handle_cmdline(const std::string& cmd) {
 					do_redraw = true;
 					zero_feedpos = true;
 				}
+			} else if (tokens[0] == "goto") {
+				if (tokens.size() >= 2 && tokens[1] != "") {
+					goto_feed(tokens[1]);
+				}
 			} else {
 				formaction::handle_cmdline(cmd);
 			}
@@ -434,6 +532,7 @@ void feedlist_formaction::finished_qna(operation op) {
 						v->show_error(_("Error: couldn't parse filter command!"));
 						m.parse(FILTER_UNREAD_FEEDS);
 					} else {
+						save_filterpos();
 						apply_filter = true;
 						do_redraw = true;
 					}
@@ -447,11 +546,10 @@ void feedlist_formaction::finished_qna(operation op) {
 					searchhistory.add_line(searchphrase);
 					std::vector<rss_item> items;
 					try {
-						items = v->get_ctrl()->search_for_items(searchphrase, "");
+						std::string utf8searchphrase = utils::convert_text(searchphrase, "utf-8", nl_langinfo(CODESET));
+						items = v->get_ctrl()->search_for_items(utf8searchphrase, "");
 					} catch (const dbexception& e) {
-						char buf[1024];
-						snprintf(buf, sizeof(buf), _("Error while searching for `%s': %s"), searchphrase.c_str(), e.what());
-						v->show_error(buf);
+						v->show_error(utils::strprintf(_("Error while searching for `%s': %s"), searchphrase.c_str(), e.what()));
 						return;
 					}
 					if (items.size() > 0) {
@@ -468,4 +566,27 @@ void feedlist_formaction::finished_qna(operation op) {
 	}
 }
 
+void feedlist_formaction::mark_pos_if_visible(unsigned int pos) {
+	scope_measure m1("feedlist_formaction::mark_pos_if_visible");
+	unsigned int vpos = 0;
+	v->get_ctrl()->update_visible_feeds();
+	for (std::vector<feedptr_pos_pair>::iterator it=visible_feeds.begin();it!=visible_feeds.end();++it, ++vpos) {
+		if (it->second == pos) {
+			GetLogger().log(LOG_DEBUG, "feedlist_formaction::mark_pos_if_visible: match, setting position to %u", vpos);
+			f->set("feedpos", utils::to_s(vpos));
+			break;
+		}
+	}
+}
+
+void feedlist_formaction::save_filterpos() {
+	std::istringstream is(f->get("feedpos"));
+	unsigned int i;
+	is >> i;
+	if (i<visible_feeds.size()) {
+		filterpos = visible_feeds[i].second;
+		set_filterpos = true;
+	}
+}
+
 }
