@@ -1,17 +1,19 @@
 #include <filebrowser_formaction.h>
+#include <formatstring.h>
 #include <logger.h>
 #include <config.h>
 #include <view.h>
+#include <utils.h>
 
 #include <iostream>
 #include <iomanip>
 #include <sstream>
+#include <cstring>
 
 #include <unistd.h>
 #include <sys/types.h>
 #include <dirent.h>
 #include <sys/param.h>
-#include <sys/types.h>
 #include <pwd.h>
 #include <grp.h>
 
@@ -23,8 +25,7 @@ filebrowser_formaction::filebrowser_formaction(view * vv, std::string formstr)
 
 filebrowser_formaction::~filebrowser_formaction() { }
 
-void filebrowser_formaction::process_operation(operation op) {
-	char buf[1024];
+void filebrowser_formaction::process_operation(operation op, bool /* automatic */, std::vector<std::string> * /* args */) {
 	switch (op) {
 		case OP_OPEN: 
 			{
@@ -42,13 +43,20 @@ void filebrowser_formaction::process_operation(operation op) {
 						selection.erase(0,1);
 						std::string filename(selection);
 						switch (filetype) {
-							case 'd':
-								if (type == FBT_OPEN) {
-									snprintf(buf, sizeof(buf), _("%s %s - Open File - %s"), PROGRAM_NAME, PROGRAM_VERSION, filename.c_str());
-								} else {
-									snprintf(buf, sizeof(buf), _("%s %s - Save File - %s"), PROGRAM_NAME, PROGRAM_VERSION, filename.c_str());
+							case 'd': {
+									std::string fileswidth = f->get("files:w");
+									std::istringstream is(fileswidth);
+									unsigned int width;
+									is >> width;
+
+									fmtstr_formatter fmt;
+									fmt.register_fmt('N', PROGRAM_NAME);
+									fmt.register_fmt('V', PROGRAM_VERSION);
+									// we use O only to distinguish between "open file" and "save file" for the %? format:
+									fmt.register_fmt('O', (type == FBT_OPEN) ? "dummy" : ""); 
+									fmt.register_fmt('f', filename);
+									f->set("head", fmt.do_format(v->get_cfg()->get_configvalue("filebrowser-title-format"), width));
 								}
-								f->set("head", buf);
 								::chdir(filename.c_str());
 								f->set("listpos","0");
 								if (type == FBT_SAVE) {
@@ -89,10 +97,8 @@ void filebrowser_formaction::process_operation(operation op) {
 						 * with no further warning...
 						 */
 						if (::stat(fn.c_str(), &sbuf)!=-1 && type == FBT_SAVE) {
-							char lbuf[2048];
-							snprintf(lbuf,sizeof(lbuf), _("Do you really want to overwrite `%s' (y:Yes n:No)? "), fn.c_str());
 							f->set_focus("files");
-							if (v->confirm(lbuf, _("yn")) == *_("n")) {
+							if (v->confirm(utils::strprintf(_("Do you really want to overwrite `%s' (y:Yes n:No)? "), fn.c_str()), _("yn")) == *_("n")) {
 								do_pop = false;
 							}
 							f->set_focus("filenametext");
@@ -136,7 +142,6 @@ void filebrowser_formaction::prepare() {
 		
 		code.append("}");
 		
-		// GetLogger().log(LOG_DEBUG, "filebrowser: code = %s", code.c_str());
 		f->modify("files", "replace_inner", code);
 		do_redraw = false;
 	}
@@ -166,11 +171,11 @@ void filebrowser_formaction::init() {
 	
 	f->set("filenametext", default_filename);
 	
-	char buf[1024];
+	std::string buf;
 	if (type == FBT_OPEN) {
-		snprintf(buf, sizeof(buf), _("%s %s - Open File - %s"), PROGRAM_NAME, PROGRAM_VERSION, cwdtmp);
+		buf = utils::strprintf(_("%s %s - Open File - %s"), PROGRAM_NAME, PROGRAM_VERSION, cwdtmp);
 	} else {
-		snprintf(buf, sizeof(buf), _("%s %s - Save File - %s"), PROGRAM_NAME, PROGRAM_VERSION, cwdtmp);
+		buf = utils::strprintf(_("%s %s - Save File - %s"), PROGRAM_NAME, PROGRAM_VERSION, cwdtmp);
 	}
 	f->set("head", buf);
 }
@@ -220,9 +225,7 @@ std::string filebrowser_formaction::add_file(std::string filename) {
 			}
 		}
 		
-		std::ostringstream os;
-		os << std::setw(12) << sb.st_size;
-		std::string sizestr = os.str();
+		std::string sizestr = utils::strprintf("%12u", sb.st_size);
 		
 		std::string line;
 		line.append(1,ftype);
