@@ -18,6 +18,10 @@ void formaction::set_keymap_hints() {
 	f->set("help", prepare_keymap_hint(this->get_keymap_hint()));
 }
 
+void formaction::recalculate_form() {
+	f->run(-3);
+}
+
 formaction::~formaction() { 
 	delete f;
 }
@@ -49,13 +53,13 @@ std::string formaction::get_value(const std::string& value) {
 
 
 void formaction::start_cmdline() {
-	std::vector<std::pair<std::string, std::string> > qna;
-	qna.push_back(std::pair<std::string,std::string>(":", ""));
+	std::vector<qna_pair> qna;
+	qna.push_back(qna_pair(":", ""));
 	this->start_qna(qna, OP_INT_END_CMDLINE, &formaction::cmdlinehistory);
 }
 
 
-void formaction::process_op(operation op) {
+void formaction::process_op(operation op, bool automatic, std::vector<std::string> * args) {
 	switch (op) {
 		case OP_CMDLINE: 
 			start_cmdline();
@@ -81,7 +85,7 @@ void formaction::process_op(operation op) {
 			start_next_question();
 			break;
 		default:
-			this->process_operation(op);
+			this->process_operation(op, automatic, args);
 	}
 }
 
@@ -96,7 +100,6 @@ void formaction::handle_cmdline(const std::string& cmdline) {
 	 * is tokenized, and then the tokens are looked at.
 	 */
 	std::vector<std::string> tokens = utils::tokenize_quoted(cmdline, " \t=");
-	char buf[1024];
 	configcontainer * cfg = v->get_cfg();
 	assert(cfg != NULL);
 	if (tokens.size() > 0) {
@@ -106,8 +109,7 @@ void formaction::handle_cmdline(const std::string& cmdline) {
 			if (tokens.size()==0) {
 				v->show_error(_("usage: set <variable>[=<value>]"));
 			} else if (tokens.size()==1) {
-				snprintf(buf,sizeof(buf), "  %s=%s", tokens[0].c_str(), cfg->get_configvalue(tokens[0]).c_str());
-				v->set_status(buf);
+				v->set_status(utils::strprintf("  %s=%s", tokens[0].c_str(), cfg->get_configvalue(tokens[0]).c_str()));
 			} else if (tokens.size()==2) {
 				cfg->set_configvalue(tokens[0], tokens[1]);
 				set_redraw(true); // because some configuration value might have changed something UI-related
@@ -122,7 +124,7 @@ void formaction::handle_cmdline(const std::string& cmdline) {
 	}
 }
 
-void formaction::start_qna(const std::vector<std::pair<std::string, std::string> >& prompts, operation finish_op, history * h) {
+void formaction::start_qna(const std::vector<qna_pair>& prompts, operation finish_op, history * h) {
 	/*
 	 * the formaction base class contains a "Q&A" mechanism that makes it possible for all formaction-derived classes to
 	 * query the user for 1 or more values, optionally with a history.
@@ -180,11 +182,11 @@ void formaction::finished_qna(operation op) {
 
 void formaction::start_bookmark_qna(const std::string& default_title, const std::string& default_url, const std::string& default_desc) {
 	GetLogger().log(LOG_DEBUG, "formaction::start_bookmark_qna: OK, starting bookmark Q&A...");
-	std::vector<std::pair<std::string,std::string> > prompts;
+	std::vector<qna_pair> prompts;
 
-	prompts.push_back(std::pair<std::string,std::string>(_("URL: "), default_url));
-	prompts.push_back(std::pair<std::string,std::string>(_("Title: "), default_title));
-	prompts.push_back(std::pair<std::string,std::string>(_("Description: "), default_desc));
+	prompts.push_back(qna_pair(_("URL: "), default_url));
+	prompts.push_back(qna_pair(_("Title: "), default_title));
+	prompts.push_back(qna_pair(_("Description: "), default_desc));
 
 	start_qna(prompts, OP_INT_BM_END);
 }
