@@ -1,6 +1,10 @@
+#include <sstream>
 #include <config.h>
 #include <help_formaction.h>
+#include <formatstring.h>
 #include <view.h>
+#include <listformatter.h>
+#include <utils.h>
 
 namespace newsbeuter {
 
@@ -10,7 +14,7 @@ help_formaction::help_formaction(view * vv, std::string formstr)
 
 help_formaction::~help_formaction() { }
 
-void help_formaction::process_operation(operation op) {
+void help_formaction::process_operation(operation op, bool /* automatic */, std::vector<std::string> * /* args */) {
 	switch (op) {
 		case OP_QUIT:
 			quit = true;
@@ -25,36 +29,29 @@ void help_formaction::process_operation(operation op) {
 
 void help_formaction::prepare() {
 	if (do_redraw) {
+		std::string listwidth = f->get("helptext:w");
+		std::istringstream is(listwidth);
+		unsigned int width;
+		is >> width;
 
-		char buf[1024];
-		snprintf(buf,sizeof(buf),_("%s %s - Help"), PROGRAM_NAME, PROGRAM_VERSION);
-		f->set("head",buf);
+		fmtstr_formatter fmt;
+		fmt.register_fmt('N', PROGRAM_NAME);
+		fmt.register_fmt('V', PROGRAM_VERSION);
+		f->set("head",fmt.do_format(v->get_cfg()->get_configvalue("help-title-format"), width));
 		
 		std::vector<keymap_desc> descs;
 		v->get_keys()->get_keymap_descriptions(descs, KM_NEWSBEUTER);
-		
-		std::string code = "{list";
+
+		listformatter listfmt;
 		
 		for (std::vector<keymap_desc>::iterator it=descs.begin();it!=descs.end();++it) {
-			std::string line = "{listitem text:";
-
-			std::string descline;
-			descline.append(it->key);
-			descline.append(1,'\t');
-			descline.append(it->cmd);
 			unsigned int how_often = 3 - (it->cmd.length() / 8);
-			descline.append(how_often,'\t');
-			descline.append(it->desc);
-
-			line.append(stfl::quote(descline));
-			line.append("}");
-			
-			code.append(line);
+			char tabs[] = "\t\t\t";
+			tabs[how_often] = '\0';
+			listfmt.add_line(utils::strprintf("%s\t%s%s%s", it->key.c_str(), it->cmd.c_str(), tabs, it->desc.c_str()));
 		}
-		
-		code.append("}");
-		
-		f->modify("helptext","replace_inner",code);
+
+		f->modify("helptext","replace_inner", listfmt.format_list());
 
 		do_redraw = false;
 	}
