@@ -1,9 +1,12 @@
 #include <htmlrenderer.h>
-#include <xmlpullparser.h>
+#include <tagsouppullparser.h>
 #include <utils.h>
 #include <sstream>
+#include <cstdio>
 #include <iostream>
+#include <cstring>
 #include <stdexcept>
+#include <algorithm>
 #include <logger.h>
 #include <libgen.h>
 #include <config.h>
@@ -35,34 +38,6 @@ void htmlrenderer::render(const std::string& source, std::vector<std::string>& l
 	render(input, lines, links, url);
 }
 
-std::string htmlrenderer::absolute_url(const std::string& url, const std::string& link) {
-	if (link.substr(0,7)=="http://" || link.substr(0,8)=="https://" || link.substr(0,6)=="ftp://" || link.substr(0,7) == "mailto:"){
-		return link;
-	}
-	char u[1024];
-	snprintf(u, sizeof(u), "%s", url.c_str());
-	if (link[0] == '/') {
-		// this is probably the worst code in the whole program
-		char * foo = strstr(u, "//");
-		if (foo) {
-			if (strlen(foo)>=2) {
-				foo += 2;
-				foo = strchr(foo,'/');
-				char u2[1024];
-				strcpy(u2, u);
-				snprintf(u2 + (foo - u), sizeof(u2) - (foo - u), "%s", link.c_str());
-				return u2;
-			}
-		}
-		return link;
-	} else {
-		char * base = dirname(u);
-		std::string retval(base);
-		retval.append(1,'/');
-		retval.append(link);
-		return retval;
-	}
-}
 
 unsigned int htmlrenderer::add_link(std::vector<linkpair>& links, const std::string& link, link_type type) {
 	bool found = false;
@@ -96,14 +71,18 @@ void htmlrenderer::render(std::istream& input, std::vector<std::string>& lines,
 	 *   - we then gather an iterator
 	 *   - we then can iterate over all continuous elements, such as start tag, close tag, text element, ...
 	 */
-	xmlpullparser xpp;
+	tagsouppullparser xpp;
 	xpp.setInput(input);
 	
-	for (xmlpullparser::event e = xpp.next(); e != xmlpullparser::END_DOCUMENT; e = xpp.next()) {	
+	for (tagsouppullparser::event e = xpp.next(); e != tagsouppullparser::END_DOCUMENT; e = xpp.next()) {	
+		std::string tagname;
 		switch (e) {
-			case xmlpullparser::START_TAG:
-				current_tag = tags[xpp.getText()];
-				GetLogger().log(LOG_DEBUG,"htmlrenderer::render: found start tag %s (id = %u)",xpp.getText().c_str(), current_tag);
+			case tagsouppullparser::START_TAG:
+				tagname = xpp.getText();
+				std::transform(tagname.begin(), tagname.end(), tagname.begin(), ::tolower);
+				current_tag = tags[tagname];
+
+				GetLogger().log(LOG_DEBUG,"htmlrenderer::render: found start tag %s (id = %u)",tagname.c_str(), current_tag);
 				switch (current_tag) {
 					case TAG_A: {
 							std::string link;
@@ -114,10 +93,8 @@ void htmlrenderer::render(std::istream& input, std::vector<std::string>& lines,
 								link = "";
 							}
 							if (link.length() > 0) {
-								unsigned int link_num = add_link(links,absolute_url(url,link), LINK_HREF);
-								std::ostringstream ref;
-								ref << "[" << link_num << "]";
-								curline.append(ref.str());
+								unsigned int link_num = add_link(links,utils::absolute_url(url,link), LINK_HREF);
+								curline.append(utils::strprintf("[%u]", link_num));
 							}
 						}
 						break;
@@ -139,10 +116,8 @@ void htmlrenderer::render(std::istream& input, std::vector<std::string>& lines,
 									link = "";
 								}
 								if (link.length() > 0) {
-									unsigned int link_num = add_link(links,absolute_url(url,link), LINK_EMBED);
-									std::ostringstream ref;
-									ref << "[" << _("embedded flash:") << " " << link_num  << "]";
-									curline.append(ref.str());
+									unsigned int link_num = add_link(links,utils::absolute_url(url,link), LINK_EMBED);
+									curline.append(utils::strprintf("[%s %u]", _("embedded flash:"), link_num));
 								}
 							}
 						}
@@ -174,11 +149,9 @@ void htmlrenderer::render(std::istream& input, std::vector<std::string>& lines,
 								imgurl = "";
 							}
 							if (imgurl.length() > 0) {
-								unsigned int link_num = add_link(links,absolute_url(url,imgurl), LINK_IMG);
-								std::ostringstream ref;
-								ref << "[" << _("image") << " " << link_num << "]";
+								unsigned int link_num = add_link(links,utils::absolute_url(url,imgurl), LINK_IMG);
+								curline.append(utils::strprintf("[%s %u]", _("image"), link_num));
 								image_count++;
-								curline.append(ref.str());
 							}
 						}
 						break;
@@ -231,12 +204,7 @@ void htmlrenderer::render(std::istream& input, std::vector<std::string>& lines,
 						prepare_newline(curline, indent_level);
 						indent_level+=2;
 						if (is_ol) {
-							std::ostringstream num;
-							num << ol_count;
-							if (ol_count < 10)
-								curline.append(" ");
-							curline.append(num.str());
-							curline.append(". ");
+							curline.append(utils::strprintf("%2u.", ol_count));
 							++ol_count;
 						} else {
 							curline.append("  * ");
@@ -279,9 +247,12 @@ void htmlrenderer::render(std::istream& input, std::vector<std::string>& lines,
 				}
 				break;
 
-			case xmlpullparser::END_TAG:
-				current_tag = tags[xpp.getText()];
-				GetLogger().log(LOG_DEBUG, "htmlrenderer::render: found end tag %s (id = %u)",xpp.getText().c_str(), current_tag);
+			case tagsouppullparser::END_TAG:
+				tagname = xpp.getText();
+				std::transform(tagname.begin(), tagname.end(), tagname.begin(), ::tolower);
+				current_tag = tags[tagname];
+
+				GetLogger().log(LOG_DEBUG, "htmlrenderer::render: found end tag %s (id = %u)",tagname.c_str(), current_tag);
 
 				switch (current_tag) {
 					case TAG_BLOCKQUOTE:
@@ -368,7 +339,7 @@ void htmlrenderer::render(std::istream& input, std::vector<std::string>& lines,
 				}
 				break;
 
-			case xmlpullparser::TEXT:
+			case tagsouppullparser::TEXT:
 				{
 					GetLogger().log(LOG_DEBUG,"htmlrenderer::render: found text `%s'",xpp.getText().c_str());
 					if (itunes_hack) {
@@ -415,8 +386,7 @@ void htmlrenderer::render(std::istream& input, std::vector<std::string>& lines,
 						while (s.length() > 0 && s[0] == '\n')
 							s.erase(0, 1);
 						std::vector<std::string> words = utils::tokenize_spaced(s);
-						//if (!line_is_nonempty(words[0]))
-						//	words.erase(words.begin());
+
 						unsigned int i=0;
 						bool new_line = false;
 						GetLogger().log(LOG_DEBUG, "htmlrenderer::render: tokenized `%s'", xpp.getText().c_str());
@@ -451,9 +421,7 @@ void htmlrenderer::render(std::istream& input, std::vector<std::string>& lines,
 		lines.push_back("");
 		lines.push_back(_("Links: "));
 		for (unsigned int i=0;i<links.size();++i) {
-			std::ostringstream line;
-			line << "[" << (i+1) << "]: " << links[i].first << " (" << type2str(links[i].second) << ")";
-			lines.push_back(line.str());
+			lines.push_back(utils::strprintf("[%u]: %s (%s)", i+1, links[i].first.c_str(), type2str(links[i].second).c_str()));
 		}
 	}
 }
