@@ -4,19 +4,24 @@
 #include <logger.h>
 #include <exceptions.h>
 #include <utils.h>
-#include <cassert>
+#include <formatstring.h>
+#include <listformatter.h>
 
+#include <cassert>
 #include <sstream>
 
+#include <langinfo.h>
+
 namespace newsbeuter {
 
 itemlist_formaction::itemlist_formaction(view * vv, std::string formstr)
-	: formaction(vv,formstr), feed(0), apply_filter(false), update_visible_items(true), search_dummy_feed(v->get_ctrl()->get_cache()) { 
+	: formaction(vv,formstr), feed(0), apply_filter(false), update_visible_items(true), search_dummy_feed(v->get_ctrl()->get_cache()),
+		set_filterpos(false), filterpos(0) {
 }
 
 itemlist_formaction::~itemlist_formaction() { }
 
-void itemlist_formaction::process_operation(operation op) {
+void itemlist_formaction::process_operation(operation op, bool automatic, std::vector<std::string> * args) {
 	bool quit = false;
 
 	/*
@@ -40,6 +45,26 @@ void itemlist_formaction::process_operation(operation op) {
 				}
 			}
 			break;
+		case OP_DELETE: {
+				scope_measure m1("OP_DELETE");
+				if (itemposname.length() > 0) {
+					visible_items[itempos].first->set_deleted(!visible_items[itempos].first->deleted());
+					v->get_ctrl()->mark_deleted(visible_items[itempos].first->guid(), visible_items[itempos].first->deleted());
+					if (itempos < visible_items.size()-1)
+						f->set("itempos", utils::strprintf("%u", itempos + 1));
+					do_redraw = true;
+				} else {
+					v->show_error(_("No item selected!")); // should not happen
+				}
+			}
+			break;
+		case OP_PURGE_DELETED: {
+				scope_measure m1("OP_PURGE_DELETED");
+				feed->purge_deleted_items();
+				update_visible_items = true;
+				do_redraw = true;
+			}
+			break;
 		case OP_OPENINBROWSER: {
 				GetLogger().log(LOG_INFO, "itemlist_formaction: opening item at pos `%s'", itemposname.c_str());
 				if (itemposname.length() > 0) {
@@ -56,7 +81,15 @@ void itemlist_formaction::process_operation(operation op) {
 				GetLogger().log(LOG_INFO, "itemlist_formaction: bookmarking item at pos `%s'", itemposname.c_str());
 				if (itemposname.length() > 0) {
 					if (itempos < visible_items.size()) {
-						this->start_bookmark_qna(visible_items[itempos].first->title(), visible_items[itempos].first->link(), "");
+						if (automatic) {
+							qna_responses.erase(qna_responses.begin(), qna_responses.end());
+							qna_responses.push_back(visible_items[itempos].first->title());
+							qna_responses.push_back(visible_items[itempos].first->link());
+							qna_responses.push_back(args->size() > 0 ? (*args)[0] : "");
+							this->finished_qna(OP_INT_BM_END);
+						} else {
+							this->start_bookmark_qna(visible_items[itempos].first->title(), visible_items[itempos].first->link(), "");
+						}
 					}
 				} else {
 					v->show_error(_("No item selected!")); // should not happen
@@ -66,9 +99,17 @@ void itemlist_formaction::process_operation(operation op) {
 		case OP_EDITFLAGS: {
 				if (itemposname.length() > 0) {
 					if (itempos < visible_items.size()) {
-						std::vector<std::pair<std::string, std::string> > qna;
-						qna.push_back(std::pair<std::string,std::string>(_("Flags: "), visible_items[itempos].first->flags()));
-						this->start_qna(qna, OP_INT_EDITFLAGS_END);
+						if (automatic) {
+							if (args->size() > 0) {
+								qna_responses.erase(qna_responses.begin(), qna_responses.end());
+								qna_responses.push_back((*args)[0]);
+								finished_qna(OP_INT_EDITFLAGS_END);
+							}
+						} else {
+							std::vector<qna_pair> qna;
+							qna.push_back(qna_pair(_("Flags: "), visible_items[itempos].first->flags()));
+							this->start_qna(qna, OP_INT_EDITFLAGS_END);
+						}
 					}
 				} else {
 					v->show_error(_("No item selected!")); // should not happen
@@ -77,23 +118,17 @@ void itemlist_formaction::process_operation(operation op) {
 			break;
 		case OP_SAVE: 
 			{
-				char buf[1024];
 				GetLogger().log(LOG_INFO, "itemlist_formaction: saving item at pos `%s'", itemposname.c_str());
 				if (itemposname.length() > 0) {
-					std::string filename = v->run_filebrowser(FBT_SAVE,v->get_filename_suggestion(visible_items[itempos].first->title()));
-					if (filename == "") {
-						v->show_error(_("Aborted saving."));
-					} else {
-						try {
-							v->write_item(*visible_items[itempos].first, filename);
-							snprintf(buf, sizeof(buf), _("Saved article to %s"), filename.c_str());
-							v->show_error(buf);
-						
-						} catch (...) {
-							snprintf(buf, sizeof(buf), _("Error: couldn't save article to %s"), filename.c_str());
-							v->show_error(buf);
+					std::string filename ;
+					if (automatic) {
+						if (args->size() > 0) {
+							filename = (*args)[0];
 						}
+					} else {
+						filename = v->run_filebrowser(FBT_SAVE,v->get_filename_suggestion(visible_items[itempos].first->title()));
 					}
+					save_article(filename, *visible_items[itempos].first);
 				} else {
 					v->show_error(_("Error: no item selected!"));
 				}
@@ -106,7 +141,6 @@ void itemlist_formaction::process_operation(operation op) {
 			if (!show_searchresult) {
 				GetLogger().log(LOG_INFO, "itemlist_formaction: reloading current feed");
 				v->get_ctrl()->reload(pos);
-				// feed = v->get_ctrl()->get_feed(pos);
 				update_visible_items = true;
 				do_redraw = true;
 			} else {
@@ -115,6 +149,8 @@ void itemlist_formaction::process_operation(operation op) {
 			break;
 		case OP_QUIT:
 			GetLogger().log(LOG_INFO, "itemlist_formaction: quitting");
+			v->feedlist_mark_pos_if_visible(pos);
+			feed->purge_deleted_items();
 			quit = true;
 			break;
 		case OP_NEXTUNREAD:
@@ -147,19 +183,33 @@ void itemlist_formaction::process_operation(operation op) {
 			GetLogger().log(LOG_INFO, "itemlist_formaction: marking feed read");
 			v->set_status(_("Marking feed read..."));
 			try {
-				v->get_ctrl()->mark_all_read(pos);
+				if (feed->rssurl() != "") {
+					v->get_ctrl()->mark_all_read(pos);
+				} else {
+					GetLogger().log(LOG_DEBUG, "itemlist_formaction: oh, it looks like I'm in a pseudo-feed (search result, query feed)");
+					for (std::vector<rss_item>::iterator it=feed->items().begin();it!=feed->items().end();++it) {
+						it->set_unread_nowrite_notify(false);
+					}
+					v->get_ctrl()->catchup_all(*feed);
+				}
 				do_redraw = true;
 				v->set_status("");
 			} catch (const dbexception& e) {
-				char buf[1024];
-				snprintf(buf, sizeof(buf), _("Error: couldn't mark feed read: %s"), e.what());
-				v->show_error(buf);
+				v->show_error(utils::strprintf(_("Error: couldn't mark feed read: %s"), e.what()));
 			}
 			break;
 		case OP_SEARCH: {
-				std::vector<std::pair<std::string, std::string> > qna;
-				qna.push_back(std::pair<std::string, std::string>(_("Search for: "), ""));
-				this->start_qna(qna, OP_INT_START_SEARCH, &searchhistory);
+				std::vector<qna_pair> qna;
+				if (automatic) {
+					if (args->size() > 0) {
+						qna_responses.erase(qna_responses.begin(), qna_responses.end());
+						qna_responses.push_back((*args)[0]);
+						finished_qna(OP_INT_START_SEARCH);
+					}
+				} else {
+					qna.push_back(qna_pair(_("Search for: "), ""));
+					this->start_qna(qna, OP_INT_START_SEARCH, &searchhistory);
+				}
 			}
 			break;
 		case OP_TOGGLEITEMREAD: {
@@ -167,20 +217,35 @@ void itemlist_formaction::process_operation(operation op) {
 				if (itemposname.length() > 0) {
 					v->set_status(_("Toggling read flag for article..."));
 					try {
-						visible_items[itempos].first->set_unread(!visible_items[itempos].first->unread());
-						v->set_status("");
+						if (automatic && args->size() > 0) {
+							if ((*args)[0] == "read") {
+								visible_items[itempos].first->set_unread(false);
+							} else if ((*args)[0] == "unread") {
+								visible_items[itempos].first->set_unread(true);
+							}
+							v->set_status("");
+						} else {
+							visible_items[itempos].first->set_unread(!visible_items[itempos].first->unread());
+							v->set_status("");
+						}
 					} catch (const dbexception& e) {
-						char buf[1024];
-						snprintf(buf, sizeof(buf), _("Error while toggling read flag: %s"), e.what());
-						v->set_status(buf);
+						v->set_status(utils::strprintf(_("Error while toggling read flag: %s"), e.what()));
 					}
+					if (itempos < visible_items.size()-1)
+						f->set("itempos", utils::strprintf("%u", itempos + 1));
 					do_redraw = true;
 				}
 			}
 			break;
 		case OP_SELECTFILTER:
 			if (v->get_ctrl()->get_filters().size() > 0) {
-				std::string newfilter = v->select_filter(v->get_ctrl()->get_filters().get_filters());
+				std::string newfilter;
+				if (automatic) {
+					if (args->size() > 0)
+						newfilter = (*args)[0];
+				} else {
+					newfilter = v->select_filter(v->get_ctrl()->get_filters().get_filters());
+				}
 				if (newfilter != "") {
 					filterhistory.add_line(newfilter);
 					if (newfilter.length() > 0) {
@@ -190,6 +255,7 @@ void itemlist_formaction::process_operation(operation op) {
 							apply_filter = true;
 							update_visible_items = true;
 							do_redraw = true;
+							save_filterpos();
 						}
 					}
 				}
@@ -198,15 +264,26 @@ void itemlist_formaction::process_operation(operation op) {
 			}
 
 			break;
-		case OP_SETFILTER: {
-				std::vector<std::pair<std::string, std::string> > qna;
-				qna.push_back(std::pair<std::string,std::string>(_("Filter: "), ""));
+		case OP_SETFILTER: 
+			if (automatic) {
+				if (args->size() > 0) {
+					qna_responses.erase(qna_responses.begin(), qna_responses.end());
+					qna_responses.push_back((*args)[0]);
+					this->finished_qna(OP_INT_END_SETFILTER);
+				}
+			} else {
+				std::vector<qna_pair> qna;
+				qna.push_back(qna_pair(_("Filter: "), ""));
 				this->start_qna(qna, OP_INT_END_SETFILTER, &filterhistory);
 			}
 			break;
 		case OP_CLEARFILTER:
 			apply_filter = false;
 			update_visible_items = true;
+			do_redraw = true;
+			save_filterpos();
+			break;
+		case OP_INT_RESIZE:
 			do_redraw = true;
 			break;
 		default:
@@ -231,6 +308,7 @@ void itemlist_formaction::finished_qna(operation op) {
 						apply_filter = true;
 						update_visible_items = true;
 						do_redraw = true;
+						save_filterpos();
 					}
 				}
 			}
@@ -260,15 +338,14 @@ void itemlist_formaction::finished_qna(operation op) {
 					searchhistory.add_line(searchphrase);
 					std::vector<rss_item> items;
 					try {
+						std::string utf8searchphrase = utils::convert_text(searchphrase, "utf-8", nl_langinfo(CODESET));
 						if (show_searchresult) {
-							items = v->get_ctrl()->search_for_items(searchphrase, "");
+							items = v->get_ctrl()->search_for_items(utf8searchphrase, "");
 						} else {
-							items = v->get_ctrl()->search_for_items(searchphrase, feed->rssurl());
+							items = v->get_ctrl()->search_for_items(utf8searchphrase, feed->rssurl());
 						}
 					} catch (const dbexception& e) {
-						char buf[1024];
-						snprintf(buf, sizeof(buf), _("Error while searching for `%s': %s"), searchphrase.c_str(), e.what());
-						v->show_error(buf);
+						v->show_error(utils::strprintf(_("Error while searching for `%s': %s"), searchphrase.c_str(), e.what()));
 						return;
 					}
 					if (items.size() > 0) {
@@ -303,80 +380,97 @@ void itemlist_formaction::do_update_visible_items() {
 	unsigned int i=0;
 	for (std::vector<rss_item>::iterator it = items.begin(); it != items.end(); ++it, ++i) {
 		if (!apply_filter || m.matches(&(*it))) {
-			visible_items.push_back(std::pair<rss_item *, unsigned int>(&(*it), i));
+			visible_items.push_back(itemptr_pos_pair(&(*it), i));
 		}
 	}
+
+	do_redraw = true;
 }
 
 void itemlist_formaction::prepare() {
+	scope_mutex mtx(&redraw_mtx);
 	if (update_visible_items) {
 		do_update_visible_items();
 		update_visible_items = false;
 	}
 
+	static unsigned int old_width = 0;
+
+	std::string listwidth = f->get("items:w");
+	std::istringstream is(listwidth);
+	unsigned int width;
+	is >> width;
+
+	if (old_width != width) {
+		do_redraw = true;
+		old_width = width;
+	}
+
 	if (do_redraw) {
+		do_redraw = false;
+
 		GetLogger().log(LOG_DEBUG, "itemlist_formaction::prepare: redrawing");
-		std::string code = "{list";
+		listformatter listfmt;
 
 		std::string datetimeformat = v->get_cfg()->get_configvalue("datetime-format");
 		if (datetimeformat.length() == 0)
 			datetimeformat = "%b %d";
 
-		for (std::vector<std::pair<rss_item *, unsigned int> >::iterator it = visible_items.begin(); it != visible_items.end(); ++it) {
-			std::string line = "{listitem[";
-			std::ostringstream x;
-			x << it->second;
-			line.append(x.str());
-			line.append("] text:");
-			std::string title;
-			char buf[20];
-			snprintf(buf,sizeof(buf),"%4u ",it->second + 1);
-			title.append(buf);
-			if (it->first->unread()) {
-				title.append("N");
+		std::string itemlist_format = v->get_cfg()->get_configvalue("articlelist-format");
+
+		for (std::vector<itemptr_pos_pair>::iterator it = visible_items.begin(); it != visible_items.end(); ++it) {
+			fmtstr_formatter fmt;
+
+			fmt.register_fmt('i', utils::strprintf("%u",it->second + 1));
+
+			std::string flags;
+			if (it->first->deleted()) {
+				flags.append("D");
+			} else if (it->first->unread()) {
+				flags.append("N");
 			} else {
-				title.append(" ");
+				flags.append(" ");
 			}
 			if (it->first->flags().length() > 0) {
-				title.append("! ");
+				flags.append("!");
 			} else {
-				title.append("  ");
+				flags.append(" ");
 			}
+
+			fmt.register_fmt('f', flags);
+
 			char datebuf[64];
 			time_t t = it->first->pubDate_timestamp();
 			struct tm * stm = localtime(&t);
 			strftime(datebuf,sizeof(datebuf), datetimeformat.c_str(), stm);
-			title.append(datebuf);
-			title.append("   ");
+
+			fmt.register_fmt('D', datebuf);
+
 			if (feed->rssurl() != it->first->feedurl()) {
-				char buf[20];
-				std::string feedtitle = it->first->get_feedptr()->title();
-				snprintf(buf,sizeof(buf),"|%-17s|",feedtitle.substr(0,17).c_str());
-				title.append(buf);
-				title.append("  ");
+				fmt.register_fmt('T', it->first->get_feedptr()->title());
 			}
-			title.append(it->first->title());
-			std::string quoted_title = stfl::quote(title);
-			GetLogger().log(LOG_DEBUG, "itemlist_formaction: XXXTITLE it->first->title = `%s' title = `%s' quoted title = `%s'", 
-				it->first->title().c_str(), title.c_str(), quoted_title.c_str());
-			// assert(quoted_title.substr(1,1) != "??");
-			line.append(quoted_title);
-			line.append("}");
-			GetLogger().log(LOG_INFO, "prepare: title  = %s", title.c_str());
-			GetLogger().log(LOG_INFO, "prepare: qtitle = %s", quoted_title.c_str());
-			GetLogger().log(LOG_INFO, "prepare: line = %s", line.c_str());
-			code.append(line);
+			fmt.register_fmt('t', it->first->title());
+			fmt.register_fmt('a', it->first->author());
+
+			listfmt.add_line(fmt.do_format(itemlist_format, width), it->second);
 		}
 
-		code.append("}");
-
-		GetLogger().log(LOG_INFO, "prepare: code = `%s'", code.c_str());
-
-		f->modify("items","replace_inner",code);
+		f->modify("items","replace_inner", listfmt.format_list());
 		
 		set_head(feed->title(),feed->unread_item_count(),feed->items().size(), feed->rssurl());
 
-		do_redraw = false;
+		if (set_filterpos) {
+			set_filterpos = false;
+			unsigned int i=0;
+			for (std::vector<itemptr_pos_pair>::iterator it=visible_items.begin();it!=visible_items.end();++it, ++i) {
+				if (it->second == filterpos) {
+					f->set("itempos", utils::to_s(i));
+					return;
+				}
+			}
+			f->set("itempos", "0");
+		}
+
 	}
 }
 
@@ -385,19 +479,36 @@ void itemlist_formaction::init() {
 	f->set("msg","");
 	do_redraw = true;
 	set_keymap_hints();
+	f->run(-3); // FRUN - compute all widget dimensions
 }
 
 void itemlist_formaction::set_head(const std::string& s, unsigned int unread, unsigned int total, const std::string &url) {
-	char buf[1024];
 	/*
 	 * Since the itemlist_formaction is also used to display search results, we always need to set the right title
 	 */
+	std::string title;
+	fmtstr_formatter fmt;
+
+	std::string listwidth = f->get("items:w");
+	std::istringstream is(listwidth);
+	unsigned int width;
+	is >> width;
+
+	fmt.register_fmt('N', PROGRAM_NAME);
+	fmt.register_fmt('V', PROGRAM_VERSION);
+
+	fmt.register_fmt('u', utils::to_s(unread));
+	fmt.register_fmt('t', utils::to_s(total));
+
+	fmt.register_fmt('T', s);
+	fmt.register_fmt('U', url);
+
 	if (!show_searchresult) {
-		snprintf(buf, sizeof(buf), _("%s %s - Articles in feed '%s' (%u unread, %u total) - %s"), PROGRAM_NAME, PROGRAM_VERSION, s.c_str(), unread, total, url.c_str());
+		title = fmt.do_format(v->get_cfg()->get_configvalue("articlelist-title-format"));
 	} else {
-		snprintf(buf, sizeof(buf), _("%s %s - Search results (%u unread, %u total)"), PROGRAM_NAME, PROGRAM_VERSION, unread, total);
+		title = fmt.do_format(v->get_cfg()->get_configvalue("searchresult-title-format"));
 	}
-	f->set("head", buf);
+	f->set("head", title);
 }
 
 bool itemlist_formaction::jump_to_previous_unread_item(bool start_with_last) {
@@ -407,17 +518,13 @@ bool itemlist_formaction::jump_to_previous_unread_item(bool start_with_last) {
 	for (int i=(start_with_last?itempos:(itempos-1));i>=0;--i) {
 		GetLogger().log(LOG_DEBUG, "itemlist_formaction::jump_to_previous_unread_item: visible_items[%u] unread = %s", i, visible_items[i].first->unread() ? "true" : "false");
 		if (visible_items[i].first->unread()) {
-			std::ostringstream os;
-			os << i;
-			f->set("itempos", os.str());
+			f->set("itempos", utils::to_s(i));
 			return true;
 		}
 	}
 	for (int i=visible_items.size()-1;i>=itempos;--i) {
 		if (visible_items[i].first->unread()) {
-			std::ostringstream os;
-			os << i;
-			f->set("itempos", os.str());
+			f->set("itempos", utils::to_s(i));
 			return true;
 		}
 	}
@@ -432,17 +539,13 @@ bool itemlist_formaction::jump_to_next_unread_item(bool start_with_first) {
 	for (unsigned int i=(start_with_first?itempos:(itempos+1));i<visible_items.size();++i) {
 		GetLogger().log(LOG_DEBUG, "itemlist_formaction::jump_to_next_unread_item: visible_items[%u] unread = %s", i, visible_items[i].first->unread() ? "true" : "false");
 		if (visible_items[i].first->unread()) {
-			std::ostringstream os;
-			os << i;
-			f->set("itempos", os.str());
+			f->set("itempos", utils::to_s(i));
 			return true;
 		}
 	}
 	for (unsigned int i=0;i<=itempos;++i) {
 		if (visible_items[i].first->unread()) {
-			std::ostringstream os;
-			os << i;
-			f->set("itempos", os.str());
+			f->set("itempos", utils::to_s(i));
 			return true;
 		}
 	}
@@ -479,9 +582,7 @@ void itemlist_formaction::handle_cmdline(const std::string& cmd) {
 			if (i == -1) {
 				v->show_error(_("Position not visible!"));
 			} else {
-				std::ostringstream idxstr;
-				idxstr << i;
-				f->set("itempos", idxstr.str());
+				f->set("itempos", utils::to_s(i));
 			}
 		} else {
 			v->show_error(_("Invalid position!"));
@@ -491,8 +592,6 @@ void itemlist_formaction::handle_cmdline(const std::string& cmd) {
 		if (tokens.size() > 0) {
 			if (tokens[0] == "save" && tokens.size() >= 2) {
 				std::string filename = utils::resolve_tilde(tokens[1]);
-				char buf[1024];
-
 				std::string itemposname = f->get("itempos");
 				GetLogger().log(LOG_INFO, "itemlist_formaction::handle_cmdline: saving item at pos `%s' to `%s'", itemposname.c_str(), filename.c_str());
 				if (itemposname.length() > 0) {
@@ -500,18 +599,7 @@ void itemlist_formaction::handle_cmdline(const std::string& cmd) {
 					unsigned int itempos = 0;
 					posname >> itempos;
 
-					if (filename == "") {
-						v->show_error(_("Aborted saving."));
-					} else {
-						try {
-							v->write_item(*visible_items[itempos].first, filename);
-							snprintf(buf, sizeof(buf), _("Saved article to %s"), filename.c_str());
-							v->show_error(buf);
-						} catch (...) {
-							snprintf(buf, sizeof(buf), _("Error: couldn't save article to %s"), filename.c_str());
-							v->show_error(buf);
-						}
-					}
+					save_article(filename, *visible_items[itempos].first);
 				} else {
 					v->show_error(_("Error: no item selected!"));
 				}
@@ -530,5 +618,33 @@ int itemlist_formaction::get_pos(unsigned int realidx) {
 	return -1;
 }
 
+void itemlist_formaction::recalculate_form() {
+	formaction::recalculate_form();
+	set_update_visible_items(true);
+}
+
+void itemlist_formaction::save_article(const std::string& filename, const rss_item& item) {
+	if (filename == "") {
+		v->show_error(_("Aborted saving."));
+	} else {
+		try {
+			v->get_ctrl()->write_item(item, filename);
+			v->show_error(utils::strprintf(_("Saved article to %s"), filename.c_str()));
+		} catch (...) {
+			v->show_error(utils::strprintf(_("Error: couldn't save article to %s"), filename.c_str()));
+		}
+	}
+}
+
+void itemlist_formaction::save_filterpos() {
+	std::istringstream is(f->get("itempos"));
+	unsigned int i;
+	is >> i;
+	if (i<visible_items.size()) {
+		filterpos = visible_items[i].second;
+		set_filterpos = true;
+	}
+}
+
 
 }
