@@ -4,6 +4,8 @@
 #include <logger.h>
 #include <exceptions.h>
 #include <utils.h>
+#include <formatstring.h>
+#include <listformatter.h>
 
 #include <sstream>
 
@@ -35,75 +37,55 @@ void itemview_formaction::prepare() {
 	 */
 	if (do_redraw) {
 		rss_item& item = feed->get_item_by_guid(guid);
-		std::string code = "{list";
+		listformatter listfmt;
 
 		rss_feed * feedptr = item.get_feedptr();
 
-		code.append("{listitem text:");
-		std::ostringstream feedtitle;
-		feedtitle << _("Feed: ");
+		std::string title, feedtitle;
 		if (feedptr->title().length() > 0) {
-			feedtitle << feedptr->title();
+			title = feedptr->title();
 		} else if (feedptr->link().length() > 0) {
-			feedtitle << feedptr->link();
+			title = feedptr->link();
 		} else if (feedptr->rssurl().length() > 0) {
-			feedtitle << feedptr->rssurl();
+			title = feedptr->rssurl();
 		}
-		code.append(stfl::quote(feedtitle.str().c_str()));
-		code.append("}");
+		feedtitle = utils::strprintf("%s%s", _("Feed: "), title.c_str());
+		listfmt.add_line(feedtitle);
 
-		code.append("{listitem text:");
-		std::ostringstream title;
-		title << _("Title: ");
-		title << item.title();
-		code.append(stfl::quote(title.str()));
-		code.append("}");
+		if (item.title().length() > 0) {
+			title = utils::strprintf("%s%s", _("Title: "), item.title().c_str());
+			listfmt.add_line(title);
+		}
 
-		code.append("{listitem text:");
-		std::ostringstream author;
-		author << _("Author: ");
-		author << item.author();
-		code.append(stfl::quote(author.str()));
-		code.append("}");
+		if (item.author().length() > 0) {
+			std::string author = utils::strprintf("%s%s", _("Author: "), item.author().c_str());
+			listfmt.add_line(author);
+		}
 
-		code.append("{listitem text:");
-		std::ostringstream link;
-		link << _("Link: ");
-		link << item.link();
-		code.append(stfl::quote(link.str()));
-		code.append("}");
-		
-		code.append("{listitem text:");
-		std::ostringstream date;
-		date << _("Date: ");
-		date << item.pubDate();
-		code.append(stfl::quote(date.str()));
-		code.append("}");
+		if (item.link().length() > 0) {
+			std::string link = utils::strprintf("%s%s", _("Link: "), item.link().c_str());
+			listfmt.add_line(link);
+		}
+
+		std::string date = utils::strprintf("%s%s", _("Date: "), item.pubDate().c_str());
+		listfmt.add_line(date);
 
 		if (item.flags().length() > 0) {
-			code.append("{listitem text:");
-			std::ostringstream flags;
-			flags << _("Flags: ");
-			flags << item.flags();
-			code.append(stfl::quote(flags.str()));
-			code.append("}");
+			std::string flags = utils::strprintf("%s%s", _("Flags: "), item.flags().c_str());
+			listfmt.add_line(flags);
 		}
 
 		if (item.enclosure_url().length() > 0) {
-			code.append("{listitem text:");
-			std::ostringstream enc_url;
-			enc_url << _("Podcast Download URL: ");
-			enc_url << item.enclosure_url() << " (" << _("type: ") << item.enclosure_type() << ")";
-			code.append(stfl::quote(enc_url.str()));
-			code.append("}");
+			std::string enc_url = utils::strprintf("%s%s (%s%s)", _("Podcast Download URL: "), item.enclosure_url().c_str(), _("type: "), item.enclosure_type().c_str());
+			listfmt.add_line(enc_url);
 		}
 
-		code.append("{listitem text:\"\"}");
-		
+		listfmt.add_line("");
+
 		set_head(item.title());
 
 		if (!render_hack) {
-			f->run(-1); // XXX HACK: render once so that we get a proper widget width
+			f->run(-3); // XXX HACK: render once so that we get a proper widget width
 			render_hack = true;
 		}
 
@@ -117,22 +99,20 @@ void itemview_formaction::prepare() {
 				render_width -= 5; 	
 		}
 
+		unsigned int textwidth = v->get_cfg()->get_configvalue_as_int("text-width");
+		if (textwidth > 0) {
+			render_width = textwidth;
+		}
+
 		if (show_source) {
 			render_source(lines, item.description(), render_width);
 		} else {
 			lines = render_html(item.description(), links, item.feedurl(), render_width);
 		}
 
-		for (std::vector<std::string>::iterator it = lines.begin(); it != lines.end(); ++it) {
-			std::string line("{listitem text:");
-			line.append(stfl::quote(*it));
-			line.append(1,'}');
-			code.append(line);
-		}
+		listfmt.add_lines(lines);
 
-		code.append("}");
-
-		f->modify("article","replace_inner",code);
+		f->modify("article","replace_inner",listfmt.format_list());
 		f->set("articleoffset","0");
 
 		do_redraw = false;
@@ -140,7 +120,7 @@ void itemview_formaction::prepare() {
 
 }
 
-void itemview_formaction::process_operation(operation op) {
+void itemview_formaction::process_operation(operation op, bool automatic, std::vector<std::string> * args) {
 	rss_item& item = feed->get_item_by_guid(guid);
 
 	/*
@@ -153,9 +133,7 @@ void itemview_formaction::process_operation(operation op) {
 	try {
 		item.set_unread(false);
 	} catch (const dbexception& e) {
-		char buf[1024];
-		snprintf(buf, sizeof(buf), _("Error while marking article as read: %s"), e.what());
-		v->show_error(buf);
+		v->show_error(utils::strprintf(_("Error while marking article as read: %s"), e.what()));
 	}
 
 	switch (op) {
@@ -168,29 +146,30 @@ void itemview_formaction::process_operation(operation op) {
 			do_redraw = true;
 			break;
 		case OP_ENQUEUE: {
-				char buf[1024];
 				if (item.enclosure_url().length() > 0) {
-					snprintf(buf, sizeof(buf), _("Added %s to download queue."), item.enclosure_url().c_str());
 					v->get_ctrl()->enqueue_url(item.enclosure_url());
-					v->set_status(buf);
+					v->set_status(utils::strprintf(_("Added %s to download queue."), item.enclosure_url().c_str()));
 				}
 			}
 			break;
 		case OP_SAVE:
 			{
-				char buf[1024];
 				GetLogger().log(LOG_INFO, "view::run_itemview: saving article");
-				std::string filename = v->run_filebrowser(FBT_SAVE,v->get_filename_suggestion(item.title()));
+				std::string filename;
+				if (automatic) {
+					if (args->size() > 0)
+						filename = (*args)[0];
+				} else {
+					filename = v->run_filebrowser(FBT_SAVE,v->get_filename_suggestion(item.title()));
+				}
 				if (filename == "") {
 					v->show_error(_("Aborted saving."));
 				} else {
 					try {
-						v->write_item(item, filename);
-						snprintf(buf, sizeof(buf), _("Saved article to %s."), filename.c_str());
-						v->show_error(buf);
+						v->get_ctrl()->write_item(item, filename);
+						v->show_error(utils::strprintf(_("Saved article to %s."), filename.c_str()));
 					} catch (...) {
-						snprintf(buf, sizeof(buf), _("Error: couldn't write article to file %s"), filename.c_str());
-						v->show_error(buf);
+						v->show_error(utils::strprintf(_("Error: couldn't write article to file %s"), filename.c_str()));
 					}
 				}
 			}
@@ -202,11 +181,25 @@ void itemview_formaction::process_operation(operation op) {
 			v->set_status("");
 			break;
 		case OP_BOOKMARK:
-			this->start_bookmark_qna(item.title(), item.link(), "");
+			if (automatic) {
+				qna_responses.erase(qna_responses.begin(), qna_responses.end());
+				qna_responses.push_back(item.title());
+				qna_responses.push_back(item.link());
+				qna_responses.push_back(args->size() > 0 ? (*args)[0] : "");
+			} else {
+				this->start_bookmark_qna(item.title(), item.link(), "");
+			}
 			break;
-		case OP_EDITFLAGS: {
-				std::vector<std::pair<std::string, std::string> > qna;
-				qna.push_back(std::pair<std::string,std::string>(_("Flags: "), item.flags()));
+		case OP_EDITFLAGS: 
+			if (automatic) {
+				qna_responses.erase(qna_responses.begin(), qna_responses.end());
+				if (args->size() > 0) {
+					qna_responses.push_back((*args)[0]);
+					this->finished_qna(OP_INT_EDITFLAGS_END);
+				}
+			} else {
+				std::vector<qna_pair> qna;
+				qna.push_back(qna_pair(_("Flags: "), item.flags()));
 				this->start_qna(qna, OP_INT_EDITFLAGS_END);
 			}
 			break;
@@ -268,9 +261,17 @@ keymap_hint_entry * itemview_formaction::get_keymap_hint() {
 }
 
 void itemview_formaction::set_head(const std::string& s) {
-	char buf[1024];
-	snprintf(buf, sizeof(buf), _("%s %s - Article '%s'"), PROGRAM_NAME, PROGRAM_VERSION, s.c_str());
-	f->set("head",buf);
+	fmtstr_formatter fmt;
+	fmt.register_fmt('N', PROGRAM_NAME);
+	fmt.register_fmt('V', PROGRAM_VERSION);
+	fmt.register_fmt('T', s);
+
+	std::string listwidth = f->get("article:w");
+	std::istringstream is(listwidth);
+	unsigned int width;
+	is >> width;
+
+	f->set("head",fmt.do_format(v->get_cfg()->get_configvalue("itemview-title-format"), width));
 }
 
 void itemview_formaction::render_source(std::vector<std::string>& lines, std::string desc, unsigned int width) {
@@ -312,18 +313,15 @@ void itemview_formaction::handle_cmdline(const std::string& cmd) {
 		if (tokens[0] == "save" && tokens.size() >= 2) {
 			std::string filename = utils::resolve_tilde(tokens[1]);
 			rss_item& item = feed->get_item_by_guid(guid);
-			char buf[1024];
 
 			if (filename == "") {
 				v->show_error(_("Aborted saving."));
 			} else {
 				try {
-					v->write_item(item, filename);
-					snprintf(buf, sizeof(buf), _("Saved article to %s"), filename.c_str());
-					v->show_error(buf);
+					v->get_ctrl()->write_item(item, filename);
+					v->show_error(utils::strprintf(_("Saved article to %s"), filename.c_str()));
 				} catch (...) {
-					snprintf(buf, sizeof(buf), _("Error: couldn't save article to %s"), filename.c_str());
-					v->show_error(buf);
+					v->show_error(utils::strprintf(_("Error: couldn't save article to %s"), filename.c_str()));
 				}
 			}
 
