@@ -4,12 +4,13 @@
 #include <config.h>
 #include <utils.h>
 #include <iostream>
-#include <sstream>
 #include <cstdio>
+#include <cstdlib>
 
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <pwd.h>
+#include <cstdlib>
 #include <signal.h>
 
 #include <keymap.h>
@@ -36,8 +37,6 @@ void ctrl_c_action(int sig) {
 namespace podbeuter {
 
 pb_controller::pb_controller() : v(0), config_file("config"), queue_file("queue"), cfg(0), view_update_(true),  max_dls(1), ql(0) { 
-	std::ostringstream cfgfile;
-
 	char * cfgdir;
 	if (!(cfgdir = ::getenv("HOME"))) {
 		struct passwd * spw = ::getpwuid(::getuid());
@@ -45,9 +44,7 @@ pb_controller::pb_controller() : v(0), config_file("config"), queue_file("queue"
 			cfgdir = spw->pw_dir;
 		} else {
 			std::cout << _("Fatal error: couldn't determine home directory!") << std::endl;
-			char buf[1024];
-			snprintf(buf, sizeof(buf), _("Please set the HOME environment variable or add a valid user for UID %u!"), ::getuid());
-			std::cout << buf << std::endl;
+			std::cout << utils::strprintf(_("Please set the HOME environment variable or add a valid user for UID %u!"), ::getuid()) << std::endl;
 			::exit(EXIT_FAILURE);
 		}
 	}
@@ -69,13 +66,13 @@ pb_controller::~pb_controller() {
 
 void pb_controller::run(int argc, char * argv[]) {
 	int c;
-	char msgbuf[1024];
+	bool automatic_dl = false;
 
 	::signal(SIGINT, ctrl_c_action);
 	::signal(SIGSEGV, ctrl_c_action);
 
 	do {
-		if ((c = ::getopt(argc, argv, "C:q:d:l:h")) < 0)
+		if ((c = ::getopt(argc, argv, "C:q:d:l:ha")) < 0)
 			continue;
 
 		switch (c) {
@@ -89,6 +86,9 @@ void pb_controller::run(int argc, char * argv[]) {
 			case 'q':
 				queue_file = optarg;
 				break;
+			case 'a':
+				automatic_dl = true;
+				break;
 			case 'd': // this is an undocumented debug commandline option!
 				GetLogger().set_logfile(optarg);
 				break;
@@ -103,20 +103,17 @@ void pb_controller::run(int argc, char * argv[]) {
 				usage(argv[0]);
 				break;
 			default:
-				snprintf(msgbuf, sizeof(msgbuf), _("%s: unknown option - %c"), argv[0], static_cast<char>(c));
-				std::cout << msgbuf << std::endl;
+				std::cout << utils::strprintf(_("%s: unknown option - %c"), argv[0], static_cast<char>(c)) << std::endl;
 				usage(argv[0]);
 				break;
 		}
 	} while (c != -1);
 
-	snprintf(msgbuf, sizeof(msgbuf), _("Starting %s %s..."), "podbeuter", PROGRAM_VERSION);
-	std::cout << msgbuf << std::endl;
+	std::cout << utils::strprintf(_("Starting %s %s..."), "podbeuter", PROGRAM_VERSION) << std::endl;
 
 	pid_t pid;
 	if (!utils::try_fs_lock(lock_file, pid)) {
-		snprintf(msgbuf, sizeof(msgbuf), _("Error: an instance of %s is already running (PID: %u)"), "podbeuter", pid);
-		std::cout << msgbuf << std::endl;
+		std::cout << utils::strprintf(_("Error: an instance of %s is already running (PID: %u)"), "podbeuter", pid) << std::endl;
 		return;
 	}
 
@@ -133,6 +130,14 @@ void pb_controller::run(int argc, char * argv[]) {
 	cfgparser.register_handler("bind-key", &keys);
 	cfgparser.register_handler("unbind-key", &keys);
 
+	null_config_action_handler null_cah;
+	cfgparser.register_handler("macro", &null_cah);
+	cfgparser.register_handler("ignore-article", &null_cah);
+	cfgparser.register_handler("define-filter", &null_cah);
+#if HAVE_RUBY
+	cfgparser.register_handler("load", &null_cah);
+#endif
+
 	try {
 		cfgparser.parse("/etc/newsbeuter/config");
 		cfgparser.parse(config_file);
@@ -154,7 +159,7 @@ void pb_controller::run(int argc, char * argv[]) {
 
 	v->set_keymap(&keys);
 	
-	v->run();
+	v->run(automatic_dl);
 
 	stfl::reset();
 
@@ -171,10 +176,10 @@ void pb_controller::run(int argc, char * argv[]) {
 
 void pb_controller::usage(const char * argv0) {
 	char buf[2048];
-	snprintf(buf, sizeof(buf),
-				_("%s %s\nusage %s [-C <file>] [-q <file>] [-h]\n"
+	std::cout << utils::strprintf(_("%s %s\nusage %s [-C <file>] [-q <file>] [-h]\n"
 				"-C <configfile> read configuration from <configfile>\n"
 				"-q <queuefile>  use <queuefile> as queue file\n"
+				"-a              start download on startup\n"
 				"-h              this help\n"), "podbeuter", PROGRAM_VERSION, argv0);
 	std::cout << buf;
 	::exit(EXIT_FAILURE);
