@@ -7,9 +7,11 @@
 
 #include <dllist.h>
 #include <help.h>
+#include <utils.h>
 
 #include <sstream>
 #include <iostream>
+#include <cstring>
 
 using namespace newsbeuter;
 
@@ -22,9 +24,8 @@ pb_view::~pb_view() {
 	stfl::reset();
 }
 
-void pb_view::run() {
+void pb_view::run(bool auto_download) {
 	bool quit = false;
-	bool auto_download = false;
 
 	set_dllist_keymap_hint();
 
@@ -55,10 +56,8 @@ void pb_view::run() {
 				unsigned int i = 0;
 				for (std::vector<download>::iterator it=ctrl->downloads().begin();it!=ctrl->downloads().end();++it,++i) {
 					char lbuf[1024];
-					std::ostringstream os;
 					snprintf(lbuf, sizeof(lbuf), " %4u [%6.1fMB/%6.1fMB] [%5.1f %%] [%7.2f kb/s] %-20s %s -> %s", i+1, it->current_size()/(1024*1024), it->total_size()/(1024*1024), it->percents_finished(), it->kbps(), it->status_text(), it->url(), it->filename());
-					os << "{listitem[" << i << "] text:" << stfl::quote(lbuf) << "}";
-					code.append(os.str());
+					code.append(utils::strprintf("{listitem[%u] text:%s}", i, stfl::quote(lbuf).c_str()));
 				}
 
 				code.append("}");
