@@ -1,20 +1,25 @@
 #include <rss.h>
 #include <config.h>
 #include <cache.h>
-#include <xmlpullparser.h>
+#include <tagsouppullparser.h>
 #include <utils.h>
 #include <logger.h>
 #include <exceptions.h>
 #include <sstream>
 #include <iostream>
 #include <configcontainer.h>
+#include <cstring>
+#include <algorithm>
 #include <curl/curl.h>
 #include <sys/utsname.h>
+#include <htmlrenderer.h>
 
 #include <langinfo.h>
 
 #include <cerrno>
 
+#include <functional>
+
 using namespace newsbeuter;
 
 rss_parser::rss_parser(const char * uri, cache * c, configcontainer * cfg, rss_ignores * ii) 
@@ -36,18 +41,7 @@ rss_feed rss_parser::parse() {
 		proxy_auth = const_cast<char *>(cfgcont->get_configvalue("proxy-auth").c_str());
 	}
 
-	/*
-	 * First we construct a fancy User-Agent header (which can be overridden by the user, BTW)
-	 */
-	char user_agent[1024];
-	std::string ua_pref = cfgcont->get_configvalue("user-agent");
-	if (ua_pref.length() == 0) {
-		struct utsname buf;
-		uname(&buf);
-		snprintf(user_agent, sizeof(user_agent), "%s/%s (%s %s; %s; %s) %s", PROGRAM_NAME, PROGRAM_VERSION, buf.sysname, buf.release, buf.machine, PROGRAM_URL, curl_version());
-	} else {
-		snprintf(user_agent, sizeof(user_agent), "%s", ua_pref.c_str());
-	}
+	std::string user_agent = utils::get_useragent(cfgcont);
 
 	/*
 	 * This is a bit messy.
@@ -60,8 +54,11 @@ rss_feed rss_parser::parse() {
 	int my_errno = 0;
 	CURLcode ccode = CURLE_OK;
 	if (my_uri.substr(0,5) == "http:" || my_uri.substr(0,6) == "https:") {
-		mrss_options_t * options = mrss_options_new(30, proxy, proxy_auth, NULL, NULL, NULL, 0, NULL, user_agent);
-		err = mrss_parse_url_with_options_and_error(const_cast<char *>(my_uri.c_str()), &mrss, options, &ccode);
+		mrss_options_t * options = mrss_options_new(30, proxy, proxy_auth, NULL, NULL, NULL, 0, NULL, user_agent.c_str());
+		{
+			scope_measure m1("mrss_parse_url_with_options_and_error");
+			err = mrss_parse_url_with_options_and_error(const_cast<char *>(my_uri.c_str()), &mrss, options, &ccode);
+		}
 		my_errno = errno;
 		GetLogger().log(LOG_DEBUG, "rss_parser::parse: http URL, err = %u errno = %u (%s)", err, my_errno, strerror(my_errno));
 		mrss_options_free(options);
@@ -74,7 +71,7 @@ rss_feed rss_parser::parse() {
 	} else if (my_uri.substr(0,7) == "filter:") {
 		std::string filter, url;
 		utils::extract_filter(my_uri, filter, url);
-		std::string buf = utils::retrieve_url(url, user_agent);
+		std::string buf = utils::retrieve_url(url, user_agent.c_str());
 		std::string result = utils::run_filter(filter, buf);
 		GetLogger().log(LOG_DEBUG, "rss_parser::parse: output of `%s' is: %s", filter.c_str(), result.c_str());
 		err = mrss_parse_buffer(const_cast<char *>(result.c_str()), result.length(), &mrss);
@@ -83,9 +80,7 @@ rss_feed rss_parser::parse() {
 		skip_parsing = true;
 		err = MRSS_OK;
 	} else {
-		char buf[1024];
-		snprintf(buf, sizeof(buf), _("Error: unsupported URL: %s"), my_uri.c_str());
-		throw std::string(buf);
+		throw utils::strprintf(_("Error: unsupported URL: %s"), my_uri.c_str());
 	}
 
 	if (!skip_parsing) {
@@ -126,12 +121,37 @@ rss_feed rss_parser::parse() {
 			feed.set_description(utils::convert_text(mrss->description, "utf-8", encoding));
 		}
 
-		if (mrss->link) feed.set_link(mrss->link);
+		if (mrss->link) {
+			feed.set_link(utils::absolute_url(my_uri, mrss->link));
+		}
+
 		if (mrss->pubDate) 
 			feed.set_pubDate(parse_date(mrss->pubDate));
 		else
 			feed.set_pubDate(::time(NULL));
 
+		// we implement right-to-left support for the languages listed in
+		// http://blogs.msdn.com/rssteam/archive/2007/05/17/reading-feeds-in-right-to-left-order.aspx
+		if (mrss->language) {
+			static const char * rtl_langprefix[] = { 
+				"ar",  // Arabic
+				"fa",  // Farsi
+				"ur",  // Urdu
+				"ps",  // Pashtu
+				"syr", // Syriac
+				"dv",  // Divehi
+				"he",  // Hebrew
+				"yi",  // Yiddish
+				NULL };
+			for (unsigned int i=0;rtl_langprefix[i]!=NULL;++i) {
+				if (strncmp(mrss->language,rtl_langprefix[i],strlen(rtl_langprefix[i]))==0) {
+					GetLogger().log(LOG_DEBUG, "rss_parser::parse: detected right-to-left order, language code = %s", rtl_langprefix[i]);
+					feed.set_rtl(true);
+					break;
+				}
+			}
+		}
+
 		GetLogger().log(LOG_DEBUG, "rss_parser::parse: feed title = `%s' link = `%s'", feed.title().c_str(), feed.link().c_str());
 
 		/*
@@ -141,12 +161,39 @@ rss_feed rss_parser::parse() {
 		for (mrss_item_t * item = mrss->item; item != NULL; item = item->next ) {
 			rss_item x(ch);
 			if (item->title) {
-				std::string title = utils::convert_text(item->title, "utf-8", encoding);
-				x.set_title(title);
-				GetLogger().log(LOG_DEBUG, "rss_parser::parse: converted title `%s' to `%s'", item->title, title.c_str());
+				if (item->title_type && (strcmp(item->title_type,"xhtml")==0 || strcmp(item->title_type,"html")==0)) {
+					std::string xhtmltitle = utils::convert_text(item->title, "utf-8", encoding);
+					htmlrenderer rnd(1 << 16); // a huge number
+					std::vector<std::string> lines;
+					std::vector<linkpair> links; // not needed
+					rnd.render(xhtmltitle, lines, links, feed.link());
+					if (lines.size() > 0) {
+						x.set_title(lines[0]);
+					}
+				} else {
+					std::string title = utils::convert_text(item->title, "utf-8", encoding);
+					GetLogger().log(LOG_DEBUG, "rss_parser::parse: before replace_newline_characters: `%s'", title.c_str());
+					replace_newline_characters(title);
+					GetLogger().log(LOG_DEBUG, "rss_parser::parse: after replace_newline_characters: `%s'", title.c_str());
+					x.set_title(title);
+					GetLogger().log(LOG_DEBUG, "rss_parser::parse: converted title `%s' to `%s'", item->title, title.c_str());
+				}
 			}
-			if (item->link) x.set_link(item->link);
-			if (item->author) {
+			if (item->link) {
+				x.set_link(utils::absolute_url(my_uri, item->link));
+			}
+			if (!item->author || strcmp(item->author,"")==0) {
+				if (mrss->managingeditor)
+					x.set_author(utils::convert_text(mrss->managingeditor, "utf-8", encoding));
+				else {
+					mrss_tag_t * creator;
+					if (mrss_search_tag(item, "creator", "http://purl.org/dc/elements/1.1/", &creator) == MRSS_OK && creator) {
+						if (creator->value) {
+							x.set_author(utils::convert_text(creator->value, "utf-8", encoding));
+						}
+					}
+				}
+			} else {
 				x.set_author(utils::convert_text(item->author, "utf-8", encoding));
 			}
 
@@ -208,8 +255,12 @@ rss_feed rss_parser::parse() {
 				GetLogger().log(LOG_DEBUG, "rss_parser::parse: no luck with itunes:summary");
 			}
 
-			if (x.description().length() == 0 && item->description) {
-				x.set_description(utils::convert_text(item->description, "utf-8", encoding));
+			if (x.description().length() == 0) {
+				if (item->description)
+					x.set_description(utils::convert_text(item->description, "utf-8", encoding));
+			} else {
+				if (cfgcont->get_configvalue_as_bool("always-display-description") && item->description)
+					x.set_description(x.description() + "<hr>" + utils::convert_text(item->description, "utf-8", encoding));
 			}
 
 			if (item->pubDate) 
@@ -255,6 +306,8 @@ rss_feed rss_parser::parse() {
 			}
 		}
 
+		feed.remove_old_deleted_items();
+
 		mrss_free(mrss);
 
 	}
@@ -264,6 +317,50 @@ rss_feed rss_parser::parse() {
 	return feed;
 }
 
+bool rss_parser::check_and_update_lastmodified() {
+	if (my_uri.substr(0,5) != "http:" && my_uri.substr(0,6) != "https:")
+		return true;
+
+	time_t oldlm = ch->get_lastmodified(my_uri);
+	time_t newlm = 0;
+	mrss_error_t err;
+
+	char * proxy = NULL;
+	char * proxy_auth = NULL;
+
+	if (cfgcont->get_configvalue_as_bool("use-proxy") == true) {
+		proxy = const_cast<char *>(cfgcont->get_configvalue("proxy").c_str());
+		proxy_auth = const_cast<char *>(cfgcont->get_configvalue("proxy-auth").c_str());
+	}
+
+	mrss_options_t * options = mrss_options_new(30, proxy, proxy_auth, NULL, NULL, NULL, 0, NULL, utils::get_useragent(cfgcont).c_str());
+
+	err = mrss_get_last_modified_with_options(const_cast<char *>(my_uri.c_str()), &newlm, options);
+
+	mrss_options_free(options);
+
+	GetLogger().log(LOG_DEBUG, "rss_parser::check_and_update_lastmodified: err = %u oldlm = %d newlm = %d", err, oldlm, newlm);
+
+	if (err != MRSS_OK) {
+		GetLogger().log(LOG_DEBUG, "rss_parser::check_and_update_lastmodified: no, don't download, due to error");
+		return false;
+	}
+
+	if (newlm == 0) {
+		GetLogger().log(LOG_DEBUG, "rss_parser::check_and_update_lastmodified: yes, download (no Last-Modified header)");
+		return true;
+	}
+
+	if (newlm > oldlm) {
+		ch->set_lastmodified(my_uri, newlm);
+		GetLogger().log(LOG_DEBUG, "rss_parser::check_and_update_lastmodified: yes, download");
+		return true;
+	}
+
+	GetLogger().log(LOG_DEBUG, "rss_parser::check_and_update_lastmodified: no, don't download");
+	return false;
+}
+
 
 // rss_item setters
 
@@ -334,7 +431,6 @@ unsigned int rss_feed::unread_item_count() const {
 }
 
 time_t rss_parser::parse_date(const std::string& datestr) {
-	// TODO: refactor	
 	std::istringstream is(datestr);
 	std::string monthstr, time, tmp;
 	struct tm stm;
@@ -377,6 +473,8 @@ time_t rss_parser::parse_date(const std::string& datestr) {
 	
 	int year;
 	is >> year;
+	if (year < 100)
+		year += 2000;
 	stm.tm_year = year - 1900;
 	
 	is >> time;
@@ -412,8 +510,10 @@ bool rss_feed::matches_tag(const std::string& tag) {
 std::string rss_feed::get_tags() {
 	std::string tags;
 	for (std::vector<std::string>::iterator it=tags_.begin();it!=tags_.end();++it) {
-		tags.append(*it);
-		tags.append(" ");
+		if (it->substr(0,1) == "~") {
+			tags.append(*it);
+			tags.append(" ");
+		}
 	}
 	return tags;
 }
@@ -452,7 +552,16 @@ std::string rss_item::description() const {
 }
 
 std::string rss_feed::title() const {
-	return utils::convert_text(title_, nl_langinfo(CODESET), "utf-8");
+	bool found_title = false;
+	std::string alt_title;
+	for (std::vector<std::string>::const_iterator it=tags_.begin();it!=tags_.end();it++) {
+		if (it->substr(0,1) == "~") {
+			found_title = true;
+			alt_title = it->substr(1, it->length()-1);
+			break;
+		}
+	}
+	return found_title ? alt_title : utils::convert_text(title_, nl_langinfo(CODESET), "utf-8");
 }
 
 std::string rss_feed::description() const {
@@ -493,7 +602,6 @@ bool rss_item::has_attribute(const std::string& attribname) {
 }
 
 std::string rss_item::get_attribute(const std::string& attribname) {
-	// GetLogger().log(LOG_DEBUG, "rss_item::get_attribute(%s) called", attribname.c_str());
 	if (attribname == "title")
 		return title();
 	else if (attribname == "link")
@@ -549,7 +657,6 @@ void rss_item::sort_flags() {
 }
 
 bool rss_feed::has_attribute(const std::string& attribname) {
-	// GetLogger().log(LOG_DEBUG, "rss_feed::has_attribute(%s) called", attribname.c_str());
 	if (attribname == "feedtitle" ||
 		attribname == "description" ||
 		attribname == "feedlink" ||
@@ -563,7 +670,6 @@ bool rss_feed::has_attribute(const std::string& attribname) {
 }
 
 std::string rss_feed::get_attribute(const std::string& attribname) {
-	// GetLogger().log(LOG_DEBUG, "rss_feed::get_attribute(%s) called", attribname.c_str());
 	if (attribname == "feedtitle")
 		return title();
 	else if (attribname == "description")
@@ -575,15 +681,10 @@ std::string rss_feed::get_attribute(const std::string& attribname) {
 	else if (attribname == "rssurl")
 		return rssurl();
 	else if (attribname == "unread_count") {
-		std::ostringstream os;
-		os << unread_item_count();
-		return os.str();
+		return utils::to_s(unread_item_count());
 	} else if (attribname == "total_count") {
-		std::ostringstream os;
-		os << items_.size();
-		return os.str();
+		return utils::to_s(items_.size());
 	} else if (attribname == "tags") {
-		// abort();
 		return get_tags();
 	}
 	return "";
@@ -596,7 +697,7 @@ action_handler_status rss_ignores::handle_action(const std::string& action, cons
 			std::string ignore_expr = params[1];
 			matcher m;
 			if (m.parse(ignore_expr)) {
-				ignores.push_back(std::pair<std::string,matcher *>(ignore_rssurl, new matcher(ignore_expr)));
+				ignores.push_back(feedurl_expr_pair(ignore_rssurl, new matcher(ignore_expr)));
 				return AHS_OK;
 			} else {
 				return AHS_INVALID_PARAMS;
@@ -609,13 +710,13 @@ action_handler_status rss_ignores::handle_action(const std::string& action, cons
 }
 
 rss_ignores::~rss_ignores() {
-	for (std::vector<std::pair<std::string, matcher *> >::iterator it=ignores.begin();it!=ignores.end();++it) {
+	for (std::vector<feedurl_expr_pair>::iterator it=ignores.begin();it!=ignores.end();++it) {
 		delete it->second;
 	}
 }
 
 bool rss_ignores::matches(rss_item* item) {
-	for (std::vector<std::pair<std::string, matcher *> >::iterator it=ignores.begin();it!=ignores.end();++it) {
+	for (std::vector<feedurl_expr_pair>::iterator it=ignores.begin();it!=ignores.end();++it) {
 		GetLogger().log(LOG_DEBUG, "rss_ignores::matches: it->first = `%s' item->feedurl = `%s'", it->first.c_str(), item->feedurl().c_str());
 		if (it->first == "*" || item->feedurl() == it->first) {
 			if (it->second->matches(item)) {
@@ -657,7 +758,7 @@ void rss_feed::update_items(std::vector<rss_feed>& feeds) {
 
 	gettimeofday(&tvx, NULL);
 
-	sort(items_.begin(), items_.end());
+	std::sort(items_.begin(), items_.end());
 
 	gettimeofday(&tv2, NULL);
 	unsigned long diff = (((tv2.tv_sec - tv1.tv_sec) * 1000000) + tv2.tv_usec) - tv1.tv_usec;
@@ -676,8 +777,95 @@ void rss_feed::set_rssurl(const std::string& u) {
 	}
 }
 
+struct sort_item_by_title : public std::binary_function<const rss_item&, const rss_item&, bool> {
+	bool operator()(const rss_item& a, const rss_item& b) {
+		return strcasecmp(a.title().c_str(), b.title().c_str()) < 0;
+	}
+};
+
+struct sort_item_by_flags : public std::binary_function<const rss_item&, const rss_item&, bool> {
+	bool operator()(const rss_item& a, const rss_item& b) {
+		return strcmp(a.flags().c_str(), b.flags().c_str()) < 0;
+	}
+};
+
+struct sort_item_by_author : public std::binary_function<const rss_item&, const rss_item&, bool> {
+	bool operator()(const rss_item& a, const rss_item& b) {
+		return strcmp(a.author().c_str(), b.author().c_str()) < 0;
+	}
+};
+
+struct sort_item_by_link : public std::binary_function<const rss_item&, const rss_item&, bool> {
+	bool operator()(const rss_item& a, const rss_item& b) {
+		return strcmp(a.link().c_str(), b.link().c_str()) < 0;
+	}
+};
+
+struct sort_item_by_guid : public std::binary_function<const rss_item&, const rss_item&, bool> {
+	bool operator()(const rss_item& a, const rss_item& b) {
+		return strcmp(a.guid().c_str(), b.guid().c_str()) < 0;
+	}
+};
+
+void rss_feed::sort(const std::string& method) {
+	std::vector<std::string> methods = utils::tokenize(method,"-");
+	bool reverse = false;
+
+	if (methods.size() > 0 && methods[0] == "date") { // date is descending by default
+		if (methods.size() > 1 && methods[1] == "asc") {
+			reverse = true;
+		}
+	} else { // all other sort methods are ascending by default
+		if (methods.size() > 1 && methods[1] == "desc") {
+			reverse = true;
+		}
+	}
+
+	if (methods.size() > 0 && methods[0] != "date") {
+		if (methods[0] == "title") {
+			std::stable_sort(items_.begin(), items_.end(), sort_item_by_title());
+		} else if (methods[0] == "flags") {
+			std::stable_sort(items_.begin(), items_.end(), sort_item_by_flags());
+		} else if (methods[0] == "author") {
+			std::stable_sort(items_.begin(), items_.end(), sort_item_by_author());
+		} else if (methods[0] == "link") {
+			std::stable_sort(items_.begin(), items_.end(), sort_item_by_link());
+		} else if (methods[0] == "guid") {
+			std::stable_sort(items_.begin(), items_.end(), sort_item_by_guid());
+		} // add new sorting methods here
+	}
+
+	if (reverse) {
+		std::reverse(items_.begin(), items_.end());
+	}
+}
+
+void rss_feed::remove_old_deleted_items() {
+	std::vector<std::string> guids;
+	for (std::vector<rss_item>::iterator it=items_.begin();it!=items_.end();++it) {
+		guids.push_back(it->guid());
+	}
+	ch->remove_old_deleted_items(rssurl_, guids);
+}
+
+void rss_feed::purge_deleted_items() {
+	scope_measure m1("rss_feed::purge_deleted_items");
+	std::vector<rss_item>::iterator it=items_.begin();
+	while (it!=items_.end()) {
+		if (it->deleted()) {
+			items_.erase(it);
+			it = items_.begin(); // items_ modified -> iterator invalidated
+		} else {
+			++it;
+		}
+	}
+}
+
 void rss_item::set_feedptr(rss_feed * ptr) {
-	// GetLogger().log(LOG_DEBUG, "rss_item::set_feedptr: setting feed pointer from %p to %p", feedptr, ptr);
 	feedptr = ptr;
 }
 
+void rss_parser::replace_newline_characters(std::string& str) {
+	str = utils::replace_all(str, "\r", " ");
+	str = utils::replace_all(str, "\n", " ");
+}
