@@ -1,4 +1,5 @@
 #include <fstream>
+#include <cstring>
 
 #include <urlreader.h>
 #include <utils.h>
@@ -22,7 +23,8 @@ std::vector<std::string>& urlreader::get_tags(const std::string& url) {
 std::vector<std::string> urlreader::get_alltags() {
 	std::vector<std::string> tmptags;
 	for (std::set<std::string>::iterator it=alltags.begin();it!=alltags.end();++it) {
-		tmptags.push_back(*it);
+		if (it->substr(0,1) != "~")
+			tmptags.push_back(*it);
 	}
 	return tmptags;
 }
@@ -129,17 +131,9 @@ void opml_urlreader::reload() {
 		alltags.erase(alltags.begin(), alltags.end());
 	}
 
-	char user_agent[1024];
-	std::string ua_pref = cfg->get_configvalue("user-agent");
-	if (ua_pref.length() == 0) {
-		struct utsname buf;
-		uname(&buf);
-		snprintf(user_agent, sizeof(user_agent), "%s/%s (%s %s; %s; %s) %s", PROGRAM_NAME, PROGRAM_VERSION, buf.sysname, buf.release, buf.machine, PROGRAM_URL, curl_version());
-	} else {
-		snprintf(user_agent, sizeof(user_agent), "%s", ua_pref.c_str());
-	}
+	std::string user_agent = utils::get_useragent(cfg);
 
-	std::string urlcontent = utils::retrieve_url(this->get_source(), user_agent, this->get_auth());
+	std::string urlcontent = utils::retrieve_url(this->get_source(), user_agent.c_str(), this->get_auth());
 	// GetLogger().log(LOG_DEBUG, "bloglines_urlreader::reload: return OPML content is `%s'", urlcontent.c_str());
 
 	nxml_t *data;
@@ -175,7 +169,7 @@ void opml_urlreader::reload() {
 
 void opml_urlreader::handle_node(nxml_data_t * node, const std::string& tag) {
 	if (node) {
-		char * rssurl = nxmle_find_attribute(node, "xmlUrl", NULL);
+		const char * rssurl = nxmle_find_attribute(node, "xmlUrl", NULL);
 		if (rssurl && strlen(rssurl) > 0) {
 			std::string theurl(rssurl);
 			urls.push_back(theurl);
@@ -191,7 +185,7 @@ void opml_urlreader::handle_node(nxml_data_t * node, const std::string& tag) {
 
 void bloglines_urlreader::handle_node(nxml_data_t * node, const std::string& tag) {
 	if (node) {
-		char * sub_id = nxmle_find_attribute(node, "BloglinesSubId", NULL);
+		const char * sub_id = nxmle_find_attribute(node, "BloglinesSubId", NULL);
 		if (sub_id) {
 			std::string theurl = getitems_url;
 			theurl.append("?s=");
@@ -224,7 +218,7 @@ void bloglines_urlreader::handle_node(nxml_data_t * node, const std::string& tag
 
 void opml_urlreader::rec_find_rss_outlines(nxml_data_t * node, std::string tag) {
 	while (node) {
-		char * type = nxmle_find_attribute(node, "type", NULL);
+		const char * type = nxmle_find_attribute(node, "type", NULL);
 
 		std::string newtag = tag;
 
@@ -232,7 +226,7 @@ void opml_urlreader::rec_find_rss_outlines(nxml_data_t * node, std::string tag)
 			if (type && strcmp(type,"rss")==0) {
 				handle_node(node, tag);
 			} else {
-				char * text = nxmle_find_attribute(node, "title", NULL);
+				const char * text = nxmle_find_attribute(node, "title", NULL);
 				if (text) {
 					if (newtag.length() > 0) {
 						newtag.append("/");
