@@ -1,17 +1,28 @@
 #include <utils.h>
 #include <logger.h>
+#include <config.h>
 
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <iconv.h>
 #include <errno.h>
-#include <sys/types.h>
 #include <pwd.h>
+#include <libgen.h>
+#include <sys/utsname.h>
 
+#include <sstream>
+#include <locale>
+#include <cwchar>
+#include <cstring>
+#include <cstdlib>
+#include <cstdarg>
 
 #include <curl/curl.h>
 
+#include <langinfo.h>
+#include <stfl.h>
+
 namespace newsbeuter {
 
 std::vector<std::string> utils::tokenize_quoted(const std::string& str, std::string delimiters) {
@@ -126,7 +137,7 @@ std::vector<std::string> utils::tokenize_quoted(const std::string& str, std::str
 	return tokens;
 }
 
-	
+
 std::vector<std::string> utils::tokenize(const std::string& str, std::string delimiters) {
 	/*
 	 * This function tokenizes a string by the delimiters. Plain and simple.
@@ -143,6 +154,22 @@ std::vector<std::string> utils::tokenize(const std::string& str, std::string del
 	return tokens;
 }
 
+std::vector<std::wstring> utils::wtokenize(const std::wstring& str, std::wstring delimiters) {
+	/*
+	 * This function tokenizes a string by the delimiters. Plain and simple.
+	 */
+	std::vector<std::wstring> tokens;
+	std::wstring::size_type last_pos = str.find_first_not_of(delimiters, 0);
+	std::wstring::size_type pos = str.find_first_of(delimiters, last_pos);
+
+	while (std::string::npos != pos || std::string::npos != last_pos) {
+		tokens.push_back(str.substr(last_pos, pos - last_pos));
+		last_pos = str.find_first_not_of(delimiters, pos);
+		pos = str.find_first_of(delimiters, last_pos);
+	}
+	return tokens;
+}
+
 std::vector<std::string> utils::tokenize_spaced(const std::string& str, std::string delimiters) {
 	std::vector<std::string> tokens;
 	std::string::size_type last_pos = str.find_first_not_of(delimiters, 0);
@@ -170,8 +197,10 @@ std::vector<std::string> utils::tokenize_nl(const std::string& str, std::string
 	unsigned int i;
 
 	GetLogger().log(LOG_DEBUG,"utils::tokenize_nl: last_pos = %u",last_pos);
-	for (i=0;i<last_pos;++i) {
-		tokens.push_back(std::string("\n"));
+	if (last_pos != std::string::npos) {
+		for (i=0;i<last_pos;++i) {
+			tokens.push_back(std::string("\n"));
+		}
 	}
 
 	while (std::string::npos != pos || std::string::npos != last_pos) {
@@ -189,7 +218,7 @@ std::vector<std::string> utils::tokenize_nl(const std::string& str, std::string
 }
 
 void utils::remove_fs_lock(const std::string& lock_file) {
-	GetLogger().log(LOG_DEBUG, "removed lockfile %s", lock_file.c_str());
+	GetLogger().log(LOG_DEBUG, "utils::remove_fs_lock: removed lockfile %s", lock_file.c_str());
 	::unlink(lock_file.c_str());
 }
 
@@ -198,6 +227,8 @@ bool utils::try_fs_lock(const std::string& lock_file, pid_t & pid) {
 	// pid == 0 indicates that something went majorly wrong during locking
 	pid = 0;
 
+	GetLogger().log(LOG_DEBUG, "utils::try_fs_lock: trying to lock %s", lock_file.c_str());
+
 	// first, we open (and possibly create) the lock file
 	fd = ::open(lock_file.c_str(), O_RDWR | O_CREAT, 0600);
 	if (fd < 0)
@@ -218,8 +249,10 @@ bool utils::try_fs_lock(const std::string& lock_file, pid_t & pid) {
 	if (fd >= 0) {
 		char buf[32];
 		int len = read(fd, buf, sizeof(buf)-1);
+		unsigned int upid;
 		buf[len] = '\0';
-		sscanf(buf, "%u", &pid);
+		sscanf(buf, "%u", &upid);
+		pid = upid;
 		close(fd);
 	}
 	return false;
@@ -234,7 +267,7 @@ std::string utils::convert_text(const std::string& text, const std::string& toco
 
 	iconv_t cd = ::iconv_open((tocode + "//TRANSLIT").c_str(), fromcode.c_str());
 
-	if (cd == (iconv_t)-1)
+	if (cd == reinterpret_cast<iconv_t>(-1))
 		return result;
 
 	size_t inbytesleft;
@@ -272,9 +305,7 @@ std::string utils::convert_text(const std::string& text, const std::string& toco
 				case EINVAL:
 					result.append(old_outbufp, outbufp - old_outbufp);
 					result.append("?");
-					GetLogger().log(LOG_DEBUG, "utils::convert_text: hit EILSEQ/EINVAL: result = `%s'", result.c_str());
 					inbufp += strlen(inbufp) - inbytesleft + 1;
-					GetLogger().log(LOG_DEBUG, "utils::convert_text: new inbufp: `%s'", inbufp);
 					inbytesleft = strlen(inbufp);
 					break;
 			}
@@ -283,9 +314,6 @@ std::string utils::convert_text(const std::string& text, const std::string& toco
 		}
 	} while (inbytesleft > 0);
 
-	GetLogger().log(LOG_DEBUG, "utils::convert_text: before: %s", text.c_str());
-	GetLogger().log(LOG_DEBUG, "utils::convert_text: after:  %s", result.c_str());
-
 	iconv_close(cd);
 
 	return result;
@@ -315,8 +343,8 @@ void utils::extract_filter(const std::string& line, std::string& filter, std::st
 }
 
 static size_t my_write_data(void *buffer, size_t size, size_t nmemb, void *userp) {
-	std::string * pbuf = (std::string *)userp;
-	pbuf->append((const char *)buffer, size * nmemb);
+	std::string * pbuf = static_cast<std::string *>(userp);
+	pbuf->append(static_cast<const char *>(buffer), size * nmemb);
 	return size * nmemb;
 }
 
@@ -330,6 +358,8 @@ std::string utils::retrieve_url(const std::string& url, const char * user_agent,
 	curl_easy_setopt(easyhandle, CURLOPT_URL, url.c_str());
 	curl_easy_setopt(easyhandle, CURLOPT_WRITEFUNCTION, my_write_data);
 	curl_easy_setopt(easyhandle, CURLOPT_WRITEDATA, &buf);
+	curl_easy_setopt(easyhandle, CURLOPT_NOSIGNAL, 1);
+	curl_easy_setopt(easyhandle, CURLOPT_ENCODING, "gzip, deflate");
 
 	if (auth) {
 		curl_easy_setopt(easyhandle, CURLOPT_USERPWD, auth);
@@ -337,6 +367,7 @@ std::string utils::retrieve_url(const std::string& url, const char * user_agent,
 	}
 
 	curl_easy_perform(easyhandle);
+	curl_easy_cleanup(easyhandle);
 
 	GetLogger().log(LOG_DEBUG, "utils::retrieve_url(%s): %s", url.c_str(), buf.c_str());
 
@@ -360,6 +391,8 @@ void utils::run_command(const std::string& cmd, const std::string& input) {
 			GetLogger().log(LOG_DEBUG, "utils::run_command: execlp of %s failed: %s", cmd.c_str(), strerror(errno));
 			exit(1);
 		}
+		default:
+			break;
 	}
 }
 
@@ -378,6 +411,7 @@ std::string utils::run_filter(const std::string& cmd, const std::string& input)
 				close(opipe[0]);
 				dup2(ipipe[0], 0);
 				dup2(opipe[1], 1);
+				GetLogger().log(LOG_DEBUG, "utils::run_filter: ipipe[0] = %d ipipe[1] = %d opipe[0] = %d opipe[1] = %d", ipipe[0], ipipe[1], opipe[0], opipe[1]);
 				execl("/bin/sh", "/bin/sh", "-c", cmd.c_str(), NULL);
 				exit(1);
 			}
@@ -483,4 +517,113 @@ std::string utils::replace_all(std::string str, const std::string& from, const s
 	return str;
 }
 
+std::wstring utils::utf8str2wstr(const std::string& utf8str) {
+	stfl_ipool * pool = stfl_ipool_create("utf-8");
+	std::wstring wstr = stfl_ipool_towc(pool, utf8str.c_str());
+	stfl_ipool_destroy(pool);
+	return wstr;
+}
+
+std::wstring utils::str2wstr(const std::string& str) {
+	const char * codeset = nl_langinfo(CODESET);
+	struct stfl_ipool * ipool = stfl_ipool_create(codeset);
+	std::wstring result = stfl_ipool_towc(ipool, str.c_str());
+	stfl_ipool_destroy(ipool);
+	return result;
+}
+
+std::string utils::wstr2str(const std::wstring& wstr) {
+	const char * codeset = nl_langinfo(CODESET);
+	struct stfl_ipool * ipool = stfl_ipool_create(codeset);
+	std::string result = stfl_ipool_fromwc(ipool, wstr.c_str());
+	stfl_ipool_destroy(ipool);
+	return result;
+}
+
+std::string utils::to_s(unsigned int u) {
+	std::ostringstream os;
+	os << u;
+	return os.str();
+}
+
+std::string utils::absolute_url(const std::string& url, const std::string& link) {
+	if (link.substr(0,7)=="http://" || link.substr(0,8)=="https://" || link.substr(0,6)=="ftp://" || link.substr(0,7) == "mailto:"){
+		return link;
+	}
+	char u[1024];
+	snprintf(u, sizeof(u), "%s", url.c_str());
+	if (link[0] == '/') {
+		// this is probably the worst code in the whole program
+		char * foo = strstr(u, "//");
+		if (foo) {
+			if (strlen(foo)>=2) {
+				foo += 2;
+				foo = strchr(foo,'/');
+				char u2[1024];
+				strcpy(u2, u);
+				snprintf(u2 + (foo - u), sizeof(u2) - (foo - u), "%s", link.c_str());
+				return u2;
+			}
+		}
+		return link;
+	} else {
+		char * base = dirname(u);
+		std::string retval(base);
+		retval.append(1,'/');
+		retval.append(link);
+		return retval;
+	}
+}
+
+
+std::string utils::strprintf(const char * format, ...) {
+	if (!format)
+		return std::string("");
+
+	va_list ap;
+	va_start(ap, format);
+
+	unsigned int len = vsnprintf(NULL, 0, format, ap);
+
+	va_end(ap);
+	va_start(ap, format);
+
+	char * buf = new char[len + 1];
+	vsnprintf(buf, len + 1, format, ap);
+	va_end(ap);
+
+	std::string ret(buf);
+	delete[] buf;
+
+	return ret;
+}
+
+std::string utils::get_useragent(configcontainer * cfgcont) {
+	std::string ua_pref = cfgcont->get_configvalue("user-agent");
+	if (ua_pref.length() == 0) {
+		struct utsname buf;
+		uname(&buf);
+		return utils::strprintf("%s/%s (%s %s; %s; %s) %s", PROGRAM_NAME, PROGRAM_VERSION, buf.sysname, buf.release, buf.machine, PROGRAM_URL, curl_version());
+	}
+	return ua_pref;
+}
+
+
+scope_measure::scope_measure(const std::string& func, loglevel ll) : lvl(ll) {
+	funcname = func;
+	gettimeofday(&tv1, NULL);
+}
+
+void scope_measure::stopover(const std::string& son) {
+	gettimeofday(&tv2, NULL);
+	unsigned long diff = (((tv2.tv_sec - tv1.tv_sec) * 1000000) + tv2.tv_usec) - tv1.tv_usec;
+	GetLogger().log(lvl, "scope_measure: function `%s' (stop over `%s') took %lu.%06lu s so far", funcname.c_str(), son.c_str(), diff / 1000000, diff % 1000000);
+}
+
+scope_measure::~scope_measure() {
+	gettimeofday(&tv2, NULL);
+	unsigned long diff = (((tv2.tv_sec - tv1.tv_sec) * 1000000) + tv2.tv_usec) - tv1.tv_usec;
+	GetLogger().log(LOG_INFO, "scope_measure: function `%s' took %lu.%06lu s", funcname.c_str(), diff / 1000000, diff % 1000000);
+}
+
 }
