@@ -19,6 +19,7 @@
 #include <exception.h>
 #include <exceptions.h>
 #include <keymap.h>
+#include <utils.h>
 
 #include <iostream>
 #include <iomanip>
@@ -45,8 +46,6 @@ extern "C" {
 #include <htmlrenderer.h>
 #include <cstring>
 #include <cstdio>
-#include <iostream>
-#include <sstream>
 
 namespace newsbeuter {
 
@@ -117,35 +116,31 @@ void view::set_bindings() {
 	GetLogger().log(LOG_DEBUG, "view::set_bindings: itemview bind_page_up = %s bind_page_down = %s", itemview->get_form()->get("bind_page_up").c_str(), itemview->get_form()->get("bind_page_down").c_str());
 }
 
-void view::set_status_unlocked(const char * msg) {
+void view::set_status_unlocked(const std::string& msg) {
 	if (formaction_stack.size() > 0 && (*formaction_stack.begin()) != NULL) {
 		stfl::form * form = (*formaction_stack.begin())->get_form();
-		// GetLogger().log(LOG_DEBUG, "view::set_status: form = %p", form);
 		form->set("msg",msg);
-		// GetLogger().log(LOG_DEBUG, "view::set_status: after form.set");
 		form->run(-1);
-		// GetLogger().log(LOG_DEBUG, "view::set_status: after form.run");
 	}
 }
 
-void view::set_status(const char * msg) {
-	mtx->lock();
-	GetLogger().log(LOG_DEBUG, "view::set_status: after mtx->lock; formaction_stack.size = %u", formaction_stack.size());
+void view::set_status(const std::string& msg) {
+	scope_mutex lock(mtx);
 	set_status_unlocked(msg);
-	mtx->unlock();
 }
 
-void view::show_error(const char * msg) {
+void view::show_error(const std::string& msg) {
 	set_status(msg);
 }
 
 void view::run() {
+	bool have_macroprefix = false;
+	std::vector<macrocmd> macrocmds;
 	/*
 	 * This is the main "event" loop of newsbeuter.
 	 */
 
 	feedlist->init();
-	itemlist->init();
 
 	while (formaction_stack.size() > 0) {
 		// first, we take the current formaction.
@@ -154,28 +149,55 @@ void view::run() {
 		// we signal "oh, you will receive an operation soon"
 		fa->prepare();
 
-		// we then receive the event and ignore timeouts.
-		const char * event = fa->get_form()->run(1000);
-		if (!event || strcmp(event,"TIMEOUT")==0) continue;
+		if (macrocmds.size() > 0) {
+			// if there is any macro command left to process, we do so
 
-		// retrieve operation code through the keymap
-		operation op = keys->get_operation(event);
+			fa->get_form()->run(-1);
+			fa->process_op(macrocmds[0].op, true, &macrocmds[0].args);
 
-		GetLogger().log(LOG_DEBUG, "view::run: event = %s op = %u", event, op);
+			macrocmds.erase(macrocmds.begin()); // remove first macro command, since it has already been processed
 
-		// the redraw keybinding is handled globally so
-		// that it doesn't need to be handled by all
-		// formactions. We simply reset the screen, the
-		// next time stfl_run() is called, it will be
-		// reinitialized, anyway, and thus we can secure
-		// that everything is redrawn.
-		if (OP_REDRAW == op) {
-			stfl::reset();
-			continue;
+		} else {
+
+			// we then receive the event and ignore timeouts.
+			const char * event = fa->get_form()->run(0);
+			if (!event || strcmp(event,"TIMEOUT")==0) continue;
+
+			GetLogger().log(LOG_DEBUG, "view::run: event = %s", event);
+
+			// retrieve operation code through the keymap
+			operation op;
+			
+			if (have_macroprefix) {
+				have_macroprefix = false;
+				GetLogger().log(LOG_DEBUG, "view::run: running macro `%s'", event);
+				macrocmds = keys->get_macro(event);
+				set_status("");
+			} else {
+				op = keys->get_operation(event);
+
+				GetLogger().log(LOG_DEBUG, "view::run: event = %s op = %u", event, op);
+
+				// the redraw keybinding is handled globally so
+				// that it doesn't need to be handled by all
+				// formactions. We simply reset the screen, the
+				// next time stfl_run() is called, it will be
+				// reinitialized, anyway, and thus we can secure
+				// that everything is redrawn.
+				if (OP_REDRAW == op) {
+					stfl::reset();
+					continue;
+				}
+
+				if (OP_MACROPREFIX == op) {
+					have_macroprefix = true;
+					set_status("macro-");
+				}
+
+				// now we handle the operation to the formaction.
+				fa->process_op(op);
+			}
 		}
-
-		// now we handle the operation to the formaction.
-		fa->process_op(op);
 	}
 
 	stfl::reset();
@@ -232,41 +254,6 @@ std::string view::get_filename_suggestion(const std::string& s) {
 	return retval;	
 }
 
-void view::write_item(const rss_item& item, const std::string& filename) {
-	// TODO: move this to the controller.
-	std::vector<std::string> lines;
-	std::vector<linkpair> links; // not used
-	
-	std::string title(_("Title: "));
-	title.append(item.title());
-	lines.push_back(title);
-	
-	std::string author(_("Author: "));
-	author.append(item.author());
-	lines.push_back(author);
-	
-	std::string date(_("Date: "));
-	date.append(item.pubDate());
-	lines.push_back(date);
-
-	std::string link(_("Link: "));
-	link.append(item.link());
-	lines.push_back(link);
-	
-	lines.push_back(std::string(""));
-	
-	htmlrenderer rnd(80);
-	rnd.render(item.description(), lines, links, item.feedurl());
-
-	std::fstream f;
-	f.open(filename.c_str(),std::fstream::out);
-	if (!f.is_open())
-		throw exception(errno);
-		
-	for (std::vector<std::string>::iterator it=lines.begin();it!=lines.end();++it) {
-		f << *it << std::endl;	
-	}
-}
 
 void view::open_in_browser(const std::string& url) {
 	formaction_stack.push_front(NULL); // we don't want a thread to write over the browser
@@ -285,8 +272,18 @@ void view::open_in_browser(const std::string& url) {
 	formaction_stack.pop_front();
 }
 
+void view::update_visible_feeds(std::vector<rss_feed>& feeds) {
+	scope_mutex lock(mtx);
+	try {
+		feedlist->update_visible_feeds(feeds);
+	} catch (matcherexception e) {
+		set_status_unlocked(utils::strprintf(_("Error: applying the filter failed: %s"), e.what()));
+		GetLogger().log(LOG_DEBUG, "view::update_visible_feeds: inside catch: %s", e.what());
+	}
+}
+
 void view::set_feedlist(std::vector<rss_feed>& feeds) {
-	mtx->lock();
+	scope_mutex lock(mtx);
 
 	for (std::vector<rss_feed>::iterator it=feeds.begin();it!=feeds.end();++it) {
 		if (it->rssurl().substr(0,6) != "query:") {
@@ -297,12 +294,9 @@ void view::set_feedlist(std::vector<rss_feed>& feeds) {
 	try {
 		feedlist->set_feedlist(feeds);
 	} catch (matcherexception e) {
-		char buf[1024];
-		snprintf(buf,sizeof(buf), _("Error: applying the filter failed: %s"), e.what());
-		set_status_unlocked(buf);
-		GetLogger().log(LOG_DEBUG, "view::set_feedlist: inside catch: %s", buf);
+		set_status_unlocked(utils::strprintf(_("Error: applying the filter failed: %s"), e.what()));
+		GetLogger().log(LOG_DEBUG, "view::set_feedlist: inside catch: %s", e.what());
 	}
-	mtx->unlock();
 }
 
 
@@ -359,6 +353,7 @@ void view::push_itemview(rss_feed * f, const std::string& guid) {
 
 void view::push_help() {
 	formaction_stack.push_front(helpview);
+	helpview->init();
 }
 
 void view::push_urlview(const std::vector<linkpair>& links) {
@@ -378,14 +373,14 @@ std::string view::select_tag(const std::vector<std::string>& tags) {
 	selecttag->set_type(select_formaction::SELECTTAG);
 	selecttag->set_tags(tags);
 	run_modal(selecttag, "");
-	return selecttag->get_value();
+	return selecttag->get_selected_value();
 }
 
-std::string view::select_filter(const std::vector<std::pair<std::string, std::string> >& filters) {
+std::string view::select_filter(const std::vector<filter_name_expr_pair>& filters) {
 	selecttag->set_type(select_formaction::SELECTFILTER);
 	selecttag->set_filters(filters);
 	run_modal(selecttag, "");
-	return selecttag->get_value();
+	return selecttag->get_selected_value();
 }
 
 char view::confirm(const std::string& prompt, const std::string& charset) {
@@ -498,9 +493,7 @@ void view::pop_current_formaction() {
 		if (f) {
 			f->set_redraw(true);
 			f->get_form()->set("msg","");
-			if (f == itemlist) { // TODO: improve this
-				itemlist->set_update_visible_items(true);
-			}
+			f->recalculate_form();
 		}
 	}
 }
@@ -558,5 +551,23 @@ void view::set_colors(const colormanager& colorman) {
 	}
 }
 
+std::string view::id() const {
+	if (formaction_stack.size() > 0) {
+		return (*formaction_stack.begin())->id();
+	}
+	return "";
+}
+
+std::string view::ask_user(const std::string& prompt) {
+	std::vector<qna_pair> qna;
+	qna.push_back(qna_pair(prompt, ""));
+	(*formaction_stack.begin())->start_qna(qna, OP_NIL);
+	return (*formaction_stack.begin())->get_qna_response(0);
+}
+
+void view::feedlist_mark_pos_if_visible(unsigned int pos) {
+	feedlist->mark_pos_if_visible(pos);
+}
+
 
 }
