@@ -1,6 +1,8 @@
 /* test driver for newsbeuter */
 
 #define BOOST_AUTO_TEST_MAIN
+#define BOOST_TEST_DYN_LINK
+#include <climits>
 #include <boost/test/auto_unit_test.hpp>
 
 #include <unistd.h>
@@ -10,16 +12,25 @@
 #include <rss.h>
 #include <configcontainer.h>
 #include <keymap.h>
-#include <xmlpullparser.h>
+#include <tagsouppullparser.h>
 #include <urlreader.h>
 #include <utils.h>
 #include <matcher.h>
 #include <history.h>
+#include <formatstring.h>
+#include <exceptions.h>
 
 #include <stdlib.h>
 
 using namespace newsbeuter;
 
+BOOST_AUTO_TEST_CASE(InitTests) {
+	setlocale(LC_CTYPE, "");
+	setlocale(LC_MESSAGES, "");
+	GetLogger().set_logfile("testlog.txt");
+	GetLogger().set_loglevel(LOG_DEBUG);
+}
+
 BOOST_AUTO_TEST_CASE(TestNewsbeuterReload) {
 	configcontainer * cfg = new configcontainer();
 	cache * rsscache = new cache("test-cache.db", cfg);
@@ -49,19 +60,40 @@ BOOST_AUTO_TEST_CASE(TestNewsbeuterReload) {
 	BOOST_CHECK_EQUAL(feed2.items()[0].title(), "Another Title");
 	BOOST_CHECK_EQUAL(feed2.items()[7].title(), "Handy als IR-Detektor");
 
+	rsscache->set_lastmodified("http://bereshit.synflood.at/~ak/rss.xml", 1000);
+	BOOST_CHECK_EQUAL(rsscache->get_lastmodified("http://bereshit.synflood.at/~ak/rss.xml"), 1000);
+	rsscache->set_lastmodified("http://bereshit.synflood.at/~ak/rss.xml", 0);
+	BOOST_CHECK_EQUAL(rsscache->get_lastmodified("http://bereshit.synflood.at/~ak/rss.xml"), 1000);
+
+	std::vector<std::string> feedurls = rsscache->get_feed_urls();
+	BOOST_CHECK_EQUAL(feedurls.size(), 1u);
+	BOOST_CHECK_EQUAL(feedurls[0], "http://bereshit.synflood.at/~ak/rss.xml");
+
+	std::vector<rss_feed> feedv;
+	feedv.push_back(feed);
+
+	cfg->set_configvalue("cleanup-on-quit", "true");
+	rsscache->cleanup_cache(feedv);
+
+
 	delete rsscache;
 	delete cfg;
 
 	::unlink("test-cache.db");
 }
 
-BOOST_AUTO_TEST_CASE(TestConfigParserAndContainer) {
+BOOST_AUTO_TEST_CASE(TestConfigParserContainerAndKeymap) {
 	configcontainer * cfg = new configcontainer();
 	configparser cfgparser;
 	cfg->register_commands(cfgparser);
+	keymap k(KM_NEWSBEUTER);
+	cfgparser.register_handler("macro", &k);
 
 	try {
 		cfgparser.parse("test-config.txt");
+	}  catch (const configexception& ex) {
+		GetLogger().log(LOG_ERROR,"an exception occured while parsing the configuration file: %s", ex.what());
+		BOOST_CHECK(false);
 	} catch (...) {
 		BOOST_CHECK(false);
 	}
@@ -83,13 +115,11 @@ BOOST_AUTO_TEST_CASE(TestConfigParserAndContainer) {
 	BOOST_CHECK(cfg->get_configvalue("cache-file") == cachefilecomp);
 
 	delete cfg;
-}
 
-BOOST_AUTO_TEST_CASE(TestKeymap) {
-	keymap k(KM_NEWSBEUTER);
 	BOOST_CHECK_EQUAL(k.get_operation("ENTER"), OP_OPEN);
 	BOOST_CHECK_EQUAL(k.get_operation("u"), OP_SHOWURLS);
 	BOOST_CHECK_EQUAL(k.get_operation("X"), OP_NIL);
+	BOOST_CHECK_EQUAL(k.get_operation(""), OP_NIL);
 
 	k.unset_key("ENTER");
 	BOOST_CHECK_EQUAL(k.get_operation("ENTER"), OP_NIL);
@@ -107,53 +137,67 @@ BOOST_AUTO_TEST_CASE(TestKeymap) {
 	BOOST_CHECK_EQUAL(k.get_key("U"), 'U');
 	BOOST_CHECK_EQUAL(k.get_key("~"), '~');
 	BOOST_CHECK_EQUAL(k.get_key("INVALID"), 0);
+	BOOST_CHECK_EQUAL(k.get_key("ENTER"), '\n');
+	BOOST_CHECK_EQUAL(k.get_key("ESC"), '\033');
+	BOOST_CHECK_EQUAL(k.get_key("^A"), '\001');
+
+	std::vector<std::string> params;
+	BOOST_CHECK_EQUAL(k.handle_action("bind-key", params), AHS_TOO_FEW_PARAMS);
+	BOOST_CHECK_EQUAL(k.handle_action("unbind-key", params), AHS_TOO_FEW_PARAMS);
+	BOOST_CHECK_EQUAL(k.handle_action("macro", params), AHS_TOO_FEW_PARAMS);
+	params.push_back("r");
+	BOOST_CHECK_EQUAL(k.handle_action("bind-key", params), AHS_TOO_FEW_PARAMS);
+	BOOST_CHECK_EQUAL(k.handle_action("unbind-key", params), AHS_OK);
+	params.push_back("open");
+	BOOST_CHECK_EQUAL(k.handle_action("bind-key", params), AHS_OK);
+	BOOST_CHECK_EQUAL(k.handle_action("an-invalid-action", params), AHS_INVALID_PARAMS);
 }
 
-BOOST_AUTO_TEST_CASE(TestXmlPullParser) {
+BOOST_AUTO_TEST_CASE(TestTagSoupPullParser) {
 	std::istringstream is("<test><foo quux='asdf' bar=\"qqq\">text</foo>more text<more>&quot;&#33;&#x40;</more></test>");
-	xmlpullparser xpp;
-	xmlpullparser::event e;
+	tagsouppullparser xpp;
+	tagsouppullparser::event e;
 	xpp.setInput(is);
 
 	e = xpp.getEventType();
-	BOOST_CHECK_EQUAL(e, xmlpullparser::START_DOCUMENT);
+	BOOST_CHECK_EQUAL(e, tagsouppullparser::START_DOCUMENT);
 	e = xpp.next();
-	BOOST_CHECK_EQUAL(e, xmlpullparser::START_TAG);
+	BOOST_CHECK_EQUAL(e, tagsouppullparser::START_TAG);
 	BOOST_CHECK_EQUAL(xpp.getText(), "test");
 	BOOST_CHECK_EQUAL(xpp.getAttributeCount(), 0);
 	e = xpp.next();
-	BOOST_CHECK_EQUAL(e, xmlpullparser::START_TAG);
+	BOOST_CHECK_EQUAL(e, tagsouppullparser::START_TAG);
 	BOOST_CHECK_EQUAL(xpp.getText(), "foo");
 	BOOST_CHECK_EQUAL(xpp.getAttributeCount(), 2);
 	BOOST_CHECK_EQUAL(xpp.getAttributeValue("quux"), "asdf");
 	BOOST_CHECK_EQUAL(xpp.getAttributeValue("bar"), "qqq");
 	e = xpp.next();
-	BOOST_CHECK_EQUAL(e, xmlpullparser::TEXT);
+	BOOST_CHECK_EQUAL(e, tagsouppullparser::TEXT);
 	BOOST_CHECK_EQUAL(xpp.getText(), "text");
 	BOOST_CHECK_EQUAL(xpp.getAttributeCount(), -1);
 	e = xpp.next();
-	BOOST_CHECK_EQUAL(e, xmlpullparser::END_TAG);
+	BOOST_CHECK_EQUAL(e, tagsouppullparser::END_TAG);
 	BOOST_CHECK_EQUAL(xpp.getText(), "foo");
 	BOOST_CHECK_EQUAL(xpp.getAttributeCount(), -1);
 	e = xpp.next();
-	BOOST_CHECK_EQUAL(e, xmlpullparser::TEXT);
+	BOOST_CHECK_EQUAL(e, tagsouppullparser::TEXT);
 	BOOST_CHECK_EQUAL(xpp.getText(), "more text");
 	e = xpp.next();
-	BOOST_CHECK_EQUAL(e, xmlpullparser::START_TAG);
+	BOOST_CHECK_EQUAL(e, tagsouppullparser::START_TAG);
 	BOOST_CHECK_EQUAL(xpp.getText(), "more");
 	e = xpp.next();
-	BOOST_CHECK_EQUAL(e, xmlpullparser::TEXT);
+	BOOST_CHECK_EQUAL(e, tagsouppullparser::TEXT);
 	BOOST_CHECK_EQUAL(xpp.getText(), "\"!@");
 	e = xpp.next();
-	BOOST_CHECK_EQUAL(e, xmlpullparser::END_TAG);
+	BOOST_CHECK_EQUAL(e, tagsouppullparser::END_TAG);
 	BOOST_CHECK_EQUAL(xpp.getText(), "more");
 	e = xpp.next();
-	BOOST_CHECK_EQUAL(e, xmlpullparser::END_TAG);
+	BOOST_CHECK_EQUAL(e, tagsouppullparser::END_TAG);
 	BOOST_CHECK_EQUAL(xpp.getText(), "test");
 	e = xpp.next();
-	BOOST_CHECK_EQUAL(e, xmlpullparser::END_DOCUMENT);
+	BOOST_CHECK_EQUAL(e, tagsouppullparser::END_DOCUMENT);
 	e = xpp.next();
-	BOOST_CHECK_EQUAL(e, xmlpullparser::END_DOCUMENT);
+	BOOST_CHECK_EQUAL(e, tagsouppullparser::END_DOCUMENT);
 }
 
 BOOST_AUTO_TEST_CASE(TestUrlReader) {
@@ -252,8 +296,6 @@ struct testmatchable : public matchable {
 };
 
 BOOST_AUTO_TEST_CASE(TestFilterLanguage) {
-	GetLogger().set_logfile("testlog.txt");
-	GetLogger().set_loglevel(LOG_DEBUG);
 
 	FilterParser fp;
 
@@ -348,19 +390,107 @@ BOOST_AUTO_TEST_CASE(TestHistory) {
 	h.add_line("testline");
 	BOOST_CHECK_EQUAL(h.prev(), "testline");
 	BOOST_CHECK_EQUAL(h.prev(), "testline");
-	BOOST_CHECK_EQUAL(h.next(), "");
+	BOOST_CHECK_EQUAL(h.next(), "testline");
 	BOOST_CHECK_EQUAL(h.next(), "");
 
 	h.add_line("foobar");
 	BOOST_CHECK_EQUAL(h.prev(), "foobar");
 	BOOST_CHECK_EQUAL(h.prev(), "testline");
-	BOOST_CHECK_EQUAL(h.next(), "foobar");
+	BOOST_CHECK_EQUAL(h.next(), "testline");
 	BOOST_CHECK_EQUAL(h.prev(), "testline");
 	BOOST_CHECK_EQUAL(h.prev(), "testline");
 	BOOST_CHECK_EQUAL(h.prev(), "testline");
-	BOOST_CHECK_EQUAL(h.next(), "foobar");
+	BOOST_CHECK_EQUAL(h.next(), "testline");
 	BOOST_CHECK_EQUAL(h.prev(), "testline");
+	BOOST_CHECK_EQUAL(h.next(), "testline");
 	BOOST_CHECK_EQUAL(h.next(), "foobar");
 	BOOST_CHECK_EQUAL(h.next(), "");
 	BOOST_CHECK_EQUAL(h.next(), "");
 }
+
+BOOST_AUTO_TEST_CASE(TestStringConversion) {
+	std::string s1 = utils::wstr2str(L"This is a simple string. Let's have a look at the outcome...");
+	BOOST_CHECK_EQUAL(s1, "This is a simple string. Let's have a look at the outcome...");
+
+	std::wstring w1 = utils::str2wstr("And that's another simple string.");
+	BOOST_CHECK_EQUAL(w1 == L"And that's another simple string.", true);
+
+	std::wstring w2 = utils::str2wstr("");
+	BOOST_CHECK_EQUAL(w2 == L"", true);
+
+	std::string s2 = utils::wstr2str(L"");
+	BOOST_CHECK_EQUAL(s2, "");
+	
+}
+
+BOOST_AUTO_TEST_CASE(TestFmtStrFormatter) {
+	fmtstr_formatter fmt;
+	fmt.register_fmt('a', "AAA");
+	fmt.register_fmt('b', "BBB");
+	fmt.register_fmt('c', "CCC");
+	BOOST_CHECK_EQUAL(fmt.do_format(""), "");
+	BOOST_CHECK_EQUAL(fmt.do_format("%"), "");
+	BOOST_CHECK_EQUAL(fmt.do_format("%%"), "%");
+	BOOST_CHECK_EQUAL(fmt.do_format("%a%b%c"), "AAABBBCCC");
+	BOOST_CHECK_EQUAL(fmt.do_format("%%%a%%%b%%%c%%"), "%AAA%BBB%CCC%");
+
+	BOOST_CHECK_EQUAL(fmt.do_format("%4a")," AAA");
+	BOOST_CHECK_EQUAL(fmt.do_format("%-4a"), "AAA ");
+
+	BOOST_CHECK_EQUAL(fmt.do_format("%2a"), "AA");
+	BOOST_CHECK_EQUAL(fmt.do_format("%-2a"), "AA");
+
+	BOOST_CHECK_EQUAL(fmt.do_format("<%a> <%5b> | %-5c%%"), "<AAA> <  BBB> | CCC  %");
+
+	fmtstr_formatter fmt2;
+	fmt2.register_fmt('a',"AAA");
+	BOOST_CHECK_EQUAL(fmt2.do_format("%?a?%a&no?"), "AAA");
+	BOOST_CHECK_EQUAL(fmt2.do_format("%?b?%b&no?"), "no");
+	BOOST_CHECK_EQUAL(fmt2.do_format("%?a?[%-4a]&no?"), "[AAA ]");
+
+	fmt2.register_fmt('b',"BBB");
+	BOOST_CHECK_EQUAL(fmt2.do_format("asdf | %a | %?c?%a%b&%b%a? | qwert"), "asdf | AAA | BBBAAA | qwert");
+	BOOST_CHECK_EQUAL(fmt2.do_format("%?c?asdf?"), "");
+	fmt2.register_fmt('c',"CCC");
+	BOOST_CHECK_EQUAL(fmt2.do_format("asdf | %a | %?c?%a%b&%b%a? | qwert"), "asdf | AAA | AAABBB | qwert");
+	BOOST_CHECK_EQUAL(fmt2.do_format("%?c?asdf?"), "asdf");
+
+	BOOST_CHECK_EQUAL(fmt.do_format("%>X", 3), "XXX");
+	BOOST_CHECK_EQUAL(fmt.do_format("%a%> %b", 10), "AAA    BBB");
+	BOOST_CHECK_EQUAL(fmt.do_format("%a%> %b", 0), "AAA BBB");
+
+}
+
+BOOST_AUTO_TEST_CASE(TestMiscUtilsFunctions) {
+	/* this test assumes some command line utilities to be installed */
+
+	BOOST_CHECK_EQUAL(utils::get_command_output("ls /dev/null"), "/dev/null\n");
+
+	BOOST_CHECK_EQUAL(utils::run_filter("cat", "this is a multine-line\ntest string"), "this is a multine-line\ntest string");
+	BOOST_CHECK_EQUAL(utils::run_filter("echo -n 'hello world'", ""), "hello world");
+
+	BOOST_CHECK_EQUAL(utils::replace_all("aaa", "a", "b"), "bbb");
+	BOOST_CHECK_EQUAL(utils::replace_all("aaa", "aa", "ba"), "baa");
+	BOOST_CHECK_EQUAL(utils::replace_all("aaaaaa", "aa", "ba"), "bababa");
+	BOOST_CHECK_EQUAL(utils::replace_all("", "a", "b"), "");
+	BOOST_CHECK_EQUAL(utils::replace_all("aaaa", "b", "c"), "aaaa");
+	BOOST_CHECK_EQUAL(utils::replace_all("this is a normal test text", " t", " T"), "this is a normal Test Text");
+
+	BOOST_CHECK_EQUAL(utils::to_s(0), "0");
+	BOOST_CHECK_EQUAL(utils::to_s(100), "100");
+	BOOST_CHECK_EQUAL(utils::to_s(65536), "65536");
+	BOOST_CHECK_EQUAL(utils::to_s(65537), "65537");
+
+	std::vector<std::string> foobar;
+	foobar.erase(foobar.begin(), foobar.end());
+}
+
+BOOST_AUTO_TEST_CASE(TestUtilsStrPrintf) {
+	BOOST_CHECK_EQUAL(utils::strprintf(NULL), "");
+	BOOST_CHECK_EQUAL(utils::strprintf("%s", ""), "");
+	BOOST_CHECK_EQUAL(utils::strprintf("%u", 0), "0");
+	BOOST_CHECK_EQUAL(utils::strprintf("%s", NULL), "(null)");
+	BOOST_CHECK_EQUAL(utils::strprintf("%u-%s-%c", 23, "hello world", 'X'), "23-hello world-X");
+}
+
+
